@@ -3,14 +3,15 @@
                              -------------------
     copyright            : (C) 2006 - 2007 by Florian Richter
  ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "../core/editor.h"
 #include "../core/game_core.h"
@@ -23,14 +24,16 @@
 #include "../input/keyboard.h"
 #include "../input/mouse.h"
 #include "../input/joystick.h"
+#include "../user/preferences.h"
 #include "../level/level.h"
 #include "../player/player.h"
 #include "../overworld/worlds.h"
 #include "../video/renderer.h"
 #include "../video/gl_surface.h"
+#include "../core/sprite_manager.h"
+#include "../overworld/overworld.h"
 // boost filesystem
-#include "boost/filesystem/operations.hpp"
-#include "boost/filesystem/path.hpp"
+#include "boost/filesystem/convenience.hpp"
 namespace fs = boost::filesystem;
 
 /* *** *** *** *** *** *** *** *** cEditor_Item_Object *** *** *** *** *** *** *** *** *** */
@@ -62,15 +65,30 @@ void cEditor_Item_Object :: Init( void )
 	list_text->setSelectionBrushImage( "TaharezLook", "ListboxSelectionBrush" );
 
 	// image dimension text
-	// string size_text = int_to_string( (int)image->w ) + "x" + int_to_string( (int)image->h );
-	preview_scale = pVideo->Get_scale( sprite_obj->image, 50, 50 );
+	// string size_text = int_to_string( static_cast<int>(image->w) ) + "x" + int_to_string( static_cast<int>(image->h) );
+
+	// get scale
+	preview_scale = pVideo->Get_scale( sprite_obj->start_image, 100, 50 );
+
+	// check if name is fitting
+	if( sprite_obj->name.length() > 25 )
+	{
+		sprite_obj->name.erase( 25 );
+		sprite_obj->name += "|";
+	}
+
+	// set position
+	sprite_obj->Set_PosX( 20, 1 );
+
+	/* Don't set sprite settings which could get copied
+	 * into the level if selected like sahdow and z position
+	*/
 }
 
-// Return the rendered pixel size of this list box item.
 Size cEditor_Item_Object :: getPixelSize( void ) const
 {
 	Size tmp = list_text->getPixelSize();
-	tmp.d_height += 60;
+	tmp.d_height += 60 * global_upscaley;
 
 	return tmp;
 }
@@ -79,16 +97,12 @@ void cEditor_Item_Object :: draw( const Vector3 &position, float alpha, const Re
 {
 	// draw text
 	list_text->draw( position, alpha, clipper );
-	// update image position
-	sprite_obj->Set_Pos( position.d_x + 20, position.d_y + 50 );
 }
 
 void cEditor_Item_Object :: draw( RenderCache &cache, const Rect &targetRect, float zBase, float alpha, const Rect *clipper ) const
 {
 	// draw text
 	list_text->draw( cache, targetRect, zBase, alpha, clipper );
-	// update image position
-	sprite_obj->Set_Pos( targetRect.d_left + 20, targetRect.d_top + 50 );
 }
 
 void cEditor_Item_Object :: Draw_Image( void )
@@ -107,12 +121,29 @@ void cEditor_Item_Object :: Draw_Image( void )
 		return;
 	}
 
+	// force valid draw
+	sprite_obj->valid_draw = 1;
+
 	// create request
 	cSurfaceRequest *request = new cSurfaceRequest();
-	sprite_obj->start_image->Blit( sprite_obj->posx, sprite_obj->posy, 0.9f, request );
 
-	request->scale_x = preview_scale;
-	request->scale_y = preview_scale;
+	// scale
+	sprite_obj->start_scalex = preview_scale;
+	sprite_obj->start_scaley = preview_scale;
+	// draw image
+	sprite_obj->Draw_Image( request );
+	// reset scale
+	sprite_obj->start_scalex = 1;
+	sprite_obj->start_scaley = 1;
+
+	// ignore camera
+	request->no_camera = 1;
+	// position z
+	request->pos_z = 0.9f;
+
+	// set shadow
+	request->shadow_color = blackalpha128;
+	request->shadow_pos = 2;
 
 	// add request
 	pRenderer_GUI->Add( request );
@@ -168,41 +199,41 @@ void cEditor :: Init( void )
 	pGuiSystem->getGUISheet()->addChildWindow( editor_window );
 
 	// Get TabControl
-	TabControl *tabcontrol_menu = (TabControl *)WindowManager::getSingleton().getWindow( "tabcontrol_editor" );
+	TabControl *tabcontrol_menu = static_cast<TabControl *>(WindowManager::getSingleton().getWindow( "tabcontrol_editor" ));
 	// TabControl Menu Tab Events
 	tabcontrol_menu->getTabContents( "editor_tab_menu" )->subscribeEvent( Window::EventMouseEnters, Event::Subscriber( &cEditor::Editor_Mouse_Enter, this ) );
 	// TabControl Items Tab Events
 	tabcontrol_menu->getTabContents( "editor_tab_items" )->subscribeEvent( Window::EventMouseEnters, Event::Subscriber( &cEditor::Editor_Mouse_Enter, this ) );
 
 	// Get Menu Listbox
-	Listbox *listbox_menu = (Listbox *)WindowManager::getSingleton().getWindow( "editor_menu" );
+	Listbox *listbox_menu = static_cast<Listbox *>(WindowManager::getSingleton().getWindow( "editor_menu" ));
 	// Menu Listbox events
 	listbox_menu->subscribeEvent( Listbox::EventSelectionChanged, Event::Subscriber( &cEditor::Menu_Select, this ) );
 	// Get Items Listbox
-	Listbox *listbox_items = (Listbox *)WindowManager::getSingleton().getWindow( "editor_items" );
+	Listbox *listbox_items = static_cast<Listbox *>(WindowManager::getSingleton().getWindow( "editor_items" ));
 	// Items Listbox events
 	listbox_items->subscribeEvent( Listbox::EventSelectionChanged, Event::Subscriber( &cEditor::Item_Select, this ) );
 
 	// Get Items
-	if( !valid_file( items_filename ) )
+	if( !file_exists( items_filename ) )
 	{
 		printf( "Error : Editor Loading : No Item file found : %s\n", items_filename.c_str() );
 		return;
 	}
 	// Parse Items
-	System::getSingleton().getXMLParser()->parseXMLFile( *this, items_filename.c_str(), SCHEMA_DIR "/Editor_Items.xsd", "" );
+	System::getSingleton().getXMLParser()->parseXMLFile( *this, items_filename.c_str(), DATA_DIR "/" GAME_SCHEMA_DIR "/Editor_Items.xsd", "" );
 
 	// Get all image items
-	Load_Image_Items( PIXMAPS_DIR );
+	Load_Image_Items( DATA_DIR "/" GAME_PIXMAPS_DIR );
 
 	// Get Menu
-	if( !valid_file( menu_filename ) )
+	if( !file_exists( menu_filename ) )
 	{
 		printf( "Error : Editor Loading : No Menu file found : %s\n", menu_filename.c_str() );
 		return;
 	}
 	// Parse Menu
-	System::getSingleton().getXMLParser()->parseXMLFile( *this, menu_filename.c_str(), SCHEMA_DIR "/Editor_Menu.xsd", "" );
+	System::getSingleton().getXMLParser()->parseXMLFile( *this, menu_filename.c_str(), DATA_DIR "/" GAME_SCHEMA_DIR "/Editor_Menu.xsd", "" );
 }
 
 void cEditor :: Unload( void )
@@ -223,12 +254,15 @@ void cEditor :: Unload( void )
 	{
 		delete *itr;
 	}
+
 	tagged_item_objects.clear();
+
 	// Tagged Image Settings
 	for( TaggedItemImageSettingsList::iterator itr = tagged_item_images.begin(), itr_end = tagged_item_images.end(); itr != itr_end; ++itr )
 	{
 		delete *itr;
 	}
+
 	tagged_item_images.clear();
 
 
@@ -264,7 +298,7 @@ void cEditor :: Enable( void )
 	}
 
 	// Draw Loading Text
-	Draw_StaticText( "Loading", &orange, &lightgreyalpha64, &grey, &black, 0 );
+	Draw_StaticText( "Loading", &orange, NULL, 0 );
 
 	// Basic Initialize
 	Init();
@@ -275,25 +309,25 @@ void cEditor :: Enable( void )
 
 	pMouseCursor->visible = 1;
 
-	// object manager
-	cObjectManager *obj_manager = NULL;
+	// sprite manager
+	cSprite_Manager *sprite_manager = NULL;
 
 	if( Game_Mode == MODE_LEVEL )
 	{
-		obj_manager = pObjManager;
+		sprite_manager = pLevel->pSprite_Manager;
 
 		// player
 		pPlayer->Update_Position_Rect();
 	}
 	else
 	{
-		obj_manager = pActive_Overworld->object_manager;
+		sprite_manager = pActive_Overworld->sprite_manager;
 
 		// player
 		pOverworld_Player->Update_Position_Rect();
 	}
 
-	for( ObjectList::iterator itr = obj_manager->items.begin(), itr_end = obj_manager->items.end(); itr != itr_end; ++itr )
+	for( SpriteList::iterator itr = sprite_manager->objects.begin(), itr_end = sprite_manager->objects.end(); itr != itr_end; ++itr )
 	{
 		(*itr)->Update_Position_Rect();
 	}
@@ -302,7 +336,7 @@ void cEditor :: Enable( void )
 	enabled = 1;
 }
 
-void cEditor :: Disable( void )
+void cEditor :: Disable( bool native_mode /* = 1 */ )
 {
 	// already disabled
 	if( !enabled )
@@ -312,45 +346,43 @@ void cEditor :: Disable( void )
 
 	Unload();
 
-	pAudio->PlaySound( "editor/leave.ogg" );
-	debugdisplay->Set_Text( "Editor disabled" );
-
-	// object manager
-	cObjectManager *obj_manager = NULL;
-
-	if( Game_Mode == MODE_LEVEL )
-	{
-		obj_manager = pObjManager;
-
-		// player
-		pPlayer->Update_Position_Rect();
-	}
-	else
-	{
-		obj_manager = pActive_Overworld->object_manager;
-
-		// player
-		pOverworld_Player->Update_Position_Rect();
-	}
-
-	for( ObjectList::iterator itr = obj_manager->items.begin(), itr_end = obj_manager->items.end(); itr != itr_end; ++itr )
-	{
-		(*itr)->Update_Position_Rect();
-	}
-
-	pCamera->Center();
-	pMouseCursor->Reset( 0 );
-	pMouseCursor->visible = 0;
-
+	enabled = 0;
 	// disable help screen
 	show_editor_help = 0;
 
-	enabled = 0;
-
-	// if level should be saved
-	if( Box_Question( "Save Level ?" ) )
+	if( native_mode )
 	{
-		Function_Save();
+		pAudio->PlaySound( "editor/leave.ogg" );
+
+		// sprite manager
+		cSprite_Manager *sprite_manager = NULL;
+
+		if( Game_Mode == MODE_LEVEL )
+		{
+			sprite_manager = pLevel->pSprite_Manager;
+
+			// player
+			pPlayer->Update_Position_Rect();
+		}
+		else
+		{
+			sprite_manager = pActive_Overworld->sprite_manager;
+
+			// player
+			pOverworld_Player->Update_Position_Rect();
+		}
+
+		for( SpriteList::iterator itr = sprite_manager->objects.begin(), itr_end = sprite_manager->objects.end(); itr != itr_end; ++itr )
+		{
+			(*itr)->Update_Position_Rect();
+		}
+
+		pCamera->Center();
+		pMouseCursor->Reset( 0 );
+		pMouseCursor->Set_Visible( 0 );
+
+		// ask if editor should save
+		Function_Save( 1 );
 	}
 }
 
@@ -361,7 +393,6 @@ void cEditor :: Update( void )
 		return;
 	}
 
-
 	// if timed out
 	if( menu_timer >= DESIRED_FPS * 2 )
 	{
@@ -375,6 +406,11 @@ void cEditor :: Update( void )
 				new_alpha = 1;
 				// hide editor window
 				editor_window->setXPosition( UDim( -0.19f, 0 ) );
+				// Hide Listbox
+				Listbox *listbox = static_cast<Listbox *>(WindowManager::getSingleton().getWindow( "editor_menu" ));
+				listbox->hide();
+				listbox = static_cast<Listbox *>(WindowManager::getSingleton().getWindow( "editor_items" ));
+				listbox->hide();
 				menu_timer = 0;
 			}
 
@@ -431,11 +467,11 @@ void cEditor :: Draw( void )
 
 		if( pCamera->x < 0 )
 		{
-			start_x = -(int)pCamera->x;
+			start_x = -static_cast<int>(pCamera->x);
 		}
 
-		Color color = Color( (Uint8)0, 0, 100, 192 );
-		pVideo->DrawLine( (float)start_x, (float)(GAME_RES_H - (int)pCamera->y), (float)GAME_RES_W, (float)( GAME_RES_H - (int)pCamera->y ), 0.124f, &color );
+		Color color = Color( static_cast<Uint8>(0), 0, 100, 192 );
+		pVideo->DrawLine( static_cast<float>(start_x), static_cast<float>(GAME_RES_H - static_cast<int>(pCamera->y)), static_cast<float>(GAME_RES_W), static_cast<float>( GAME_RES_H - static_cast<int>(pCamera->y) ), 0.124f, &color );
 	}
 	// Camera limit top line
 	if( pCamera->y + GAME_RES_H > pCamera->limit_rect.y + pCamera->limit_rect.h && pCamera->y < pCamera->limit_rect.y + pCamera->limit_rect.h )
@@ -444,11 +480,11 @@ void cEditor :: Draw( void )
 
 		if( pCamera->x < pCamera->limit_rect.x )
 		{
-			start_x = -(int)pCamera->x;
+			start_x = -static_cast<int>(pCamera->x);
 		}
 
-		Color color = Color( (Uint8)20, 20, 150, 192 );
-		pVideo->DrawLine( (float)start_x, ( pCamera->limit_rect.y + pCamera->limit_rect.h ) - pCamera->y, (float)GAME_RES_W, ( pCamera->limit_rect.y + pCamera->limit_rect.h ) - pCamera->y, 0.124f, &color );
+		Color color = Color( static_cast<Uint8>(20), 20, 150, 192 );
+		pVideo->DrawLine( static_cast<float>(start_x), ( pCamera->limit_rect.y + pCamera->limit_rect.h ) - pCamera->y, static_cast<float>(GAME_RES_W), ( pCamera->limit_rect.y + pCamera->limit_rect.h ) - pCamera->y, 0.124f, &color );
 	}
 
 	// Camera limit left line
@@ -458,11 +494,11 @@ void cEditor :: Draw( void )
 
 		if( pCamera->y < GAME_RES_H )
 		{
-			start_y = GAME_RES_H - (int)pCamera->y;
+			start_y = GAME_RES_H - static_cast<int>(pCamera->y);
 		}
 
-		Color color = Color( (Uint8)0, 100, 0, 192 );
-		pVideo->DrawLine( (float)( 0 - pCamera->x ), (float)start_y, (float)( -pCamera->x ), 0, 0.124f, &color );
+		Color color = Color( static_cast<Uint8>(0), 100, 0, 192 );
+		pVideo->DrawLine( static_cast<float>( 0 - pCamera->x ), static_cast<float>(start_y), static_cast<float>( -pCamera->x ), 0, 0.124f, &color );
 	}
 	// Camera limit right line
 	if( pCamera->x < pCamera->limit_rect.x + pCamera->limit_rect.w && pCamera->x > -GAME_RES_W + pCamera->limit_rect.x + pCamera->limit_rect.w )
@@ -471,35 +507,46 @@ void cEditor :: Draw( void )
 
 		if( pCamera->y < GAME_RES_H )
 		{
-			start_y = GAME_RES_H - (int)pCamera->y;
+			start_y = GAME_RES_H - static_cast<int>(pCamera->y);
 		}
 
-		Color color = Color( (Uint8)20, 150, 20, 192 );
-		pVideo->DrawLine( (float)( pCamera->limit_rect.x + pCamera->limit_rect.w - pCamera->x ), (float)start_y, (float)( pCamera->limit_rect.x + pCamera->limit_rect.w - pCamera->x ), 0, 0.124f, &color );
+		Color color = Color( static_cast<Uint8>(20), 150, 20, 192 );
+		pVideo->DrawLine( static_cast<float>( pCamera->limit_rect.x + pCamera->limit_rect.w - pCamera->x ), static_cast<float>(start_y), static_cast<float>( pCamera->limit_rect.x + pCamera->limit_rect.w - pCamera->x ), 0, 0.124f, &color );
 	}
 
 	// Get Items Listbox
-	Listbox *listbox_items = (Listbox *)WindowManager::getSingleton().getWindow( "editor_items" );
-
-	// Vertical ScrollBar Position
-	float scroll_pos = listbox_items->getVertScrollbar()->getScrollPosition();
+	Listbox *listbox_items = static_cast<Listbox *>(WindowManager::getSingleton().getWindow( "editor_items" ));
 
 	// if editor window is active
 	if( editor_window->getXPosition().asRelative( 1 ) >= 0 )
 	{
-		// draw the visible items
+		// Listbox dimension
+		float list_posy = listbox_items->getUnclippedPixelRect().d_top * global_downscaley;
+		float list_posx = listbox_items->getUnclippedPixelRect().d_left * global_downscalex;
+		float list_height = listbox_items->getUnclippedPixelRect().getHeight() * global_downscaley;
+		// Vertical ScrollBar Position
+		float scroll_pos = listbox_items->getVertScrollbar()->getScrollPosition() * global_downscaley;
+		// font height
+		float font_height = FontManager::getSingleton().getFont( "bluebold_medium" )->getFontHeight() * global_downscaley;
+
+		// draw items
 		for( unsigned int i = 0; i < listbox_items->getItemCount(); i++ )
 		{
+			// Get item
 			cEditor_Item_Object *item = static_cast<cEditor_Item_Object *>( listbox_items->getListboxItemFromIndex( i ) );
-
-			float item_pos = item->getPixelSize().d_height * ( i + 1 );
+			// Item height
+			float item_height = item->getPixelSize().d_height * global_downscaley;
+			// Item position
+			float item_posy = list_posy + ( item_height * i );
+			float item_image_posy = item_posy + ( font_height * 2 );
 
 			// not visible
-			if( item_pos > listbox_items->getListRenderArea().getHeight() + scroll_pos || item_pos - scroll_pos < 50 )
+			if( item_posy + item_height > list_posy + list_height + scroll_pos || item_image_posy < list_posy + scroll_pos )
 			{
 				continue;
 			}
 
+			item->sprite_obj->Set_PosY( item_image_posy - scroll_pos, 1 );
 			item->Draw_Image();
 		}
 	}
@@ -520,7 +567,7 @@ void cEditor :: Process_Input( void )
 	// Drag Delete
 	if( ( pKeyboard->keys[SDLK_RCTRL] || pKeyboard->keys[SDLK_LCTRL] ) && pMouseCursor->right && pMouseCursor->Editor_Collsion_Check() )
 	{
-		pMouseCursor->Delete( static_cast<cMovingSprite *>(pMouseCursor->mouse_object->obj) );
+		pMouseCursor->Delete( pMouseCursor->mouse_object->obj );
 	}
 
 	// Camera Movement
@@ -588,55 +635,6 @@ bool cEditor :: Handle_Event( SDL_Event *ev )
 
 		break;
 	}
-	case SDL_MOUSEBUTTONDOWN:
-	{
-		// left
-		if( ev->button.button == 1 )
-		{
-			pMouseCursor->Click();
-		}
-		// middle
-		else if( ev->button.button == 2 )
-		{
-			// Activate fast copy mode
-			if( pMouseCursor->mouse_object->obj )
-			{
-				pMouseCursor->mode_fastcopy = 1;
-				return 1;
-			}
-			// Mover mode
-			else
-			{
-				pMouseCursor->Toggle_Mover_mode();
-				return 1;
-			}
-		}
-		// right
-		else if( ev->button.button == 3 )
-		{
-			if( !pMouseCursor->left )
-			{
-				pMouseCursor->Delete( static_cast<cMovingSprite *>(pMouseCursor->mouse_object->obj) );
-				return 1;
-			}
-		}
-
-		break;
-	}
-	case SDL_MOUSEBUTTONUP:
-	{
-		// left
-		if( ev->button.button == 1 )
-		{
-			pMouseCursor->End_selection();
-		}
-		// middle
-		else if( ev->button.button == 2 )
-		{
-			pMouseCursor->mode_fastcopy = 0;
-		}
-		break;
-	}
 	default: // other events
 	{
 		break;
@@ -675,17 +673,21 @@ bool cEditor :: Key_Down( SDLKey key )
 	// push selected objects into the front
 	else if( key == SDLK_KP_PLUS )
 	{
-		for( unsigned int i = 0; i < pMouseCursor->selected_objects.size(); i++ )
+		for( SelectedObjectList::iterator itr = pMouseCursor->selected_objects.begin(), itr_end = pMouseCursor->selected_objects.end(); itr != itr_end; ++itr )
 		{
-			Change_DrawPosition( static_cast<cMovingSprite *>(pMouseCursor->selected_objects[i]->obj), 0 );
+			cSelectedObject *sel_obj = (*itr);
+
+			Change_DrawPosition( sel_obj->obj, 0 );
 		}
 	}
 	// push selected objects into the back
 	else if( key == SDLK_KP_MINUS )
 	{
-		for( unsigned int i = 0; i < pMouseCursor->selected_objects.size(); i++ )
+		for( SelectedObjectList::iterator itr = pMouseCursor->selected_objects.begin(), itr_end = pMouseCursor->selected_objects.end(); itr != itr_end; ++itr )
 		{
-			Change_DrawPosition( static_cast<cMovingSprite *>(pMouseCursor->selected_objects[i]->obj), 1 );
+			cSelectedObject *sel_obj = (*itr);
+
+			Change_DrawPosition( sel_obj->obj, 1 );
 		}
 	}
 	// ## only with mouse_object functions
@@ -722,12 +724,20 @@ bool cEditor :: Key_Down( SDLKey key )
 			// copy objects
 			SpriteList new_objects = Copy_Direction( objects, dir );
 
-			for( unsigned int i = 0; i < objects.size(); i++ )
+			// add new objects
+			for( SpriteList::iterator itr = new_objects.begin(), itr_end = new_objects.end(); itr != itr_end; ++itr )
 			{
-				// add new object
-				pMouseCursor->Add_selected_object( new_objects[i], 1 );
-				// deselect old object
-				pMouseCursor->Remove_selected_object( objects[i] );
+				cSprite *obj = (*itr);
+
+				pMouseCursor->Add_selected_object( obj, 1 );
+			}
+			
+			// deselect old objects
+			for( SpriteList::iterator itr = objects.begin(), itr_end = objects.end(); itr != itr_end; ++itr )
+			{
+				cSprite *obj = (*itr);
+
+				pMouseCursor->Remove_selected_object( obj );
 			}
 		}
 		else
@@ -759,25 +769,25 @@ bool cEditor :: Key_Down( SDLKey key )
 	{
 		pMouseCursor->Clear_selected_objects();
 
-		cObjectManager *obj_manager = NULL;
+		cSprite_Manager *sprite_manager = NULL;
 
 		if( editor_level_enabled )
 		{
-			obj_manager = pObjManager;
+			sprite_manager = pLevel->pSprite_Manager;
 
 			// player
 			pMouseCursor->Add_selected_object( pPlayer, 1 );
 		}
 		else
 		{
-			obj_manager = pActive_Overworld->object_manager;
+			sprite_manager = pActive_Overworld->sprite_manager;
 
 			// player
 			pMouseCursor->Add_selected_object( pOverworld_Player, 1 );
 		}
 
 		// all level objects
-		for( ObjectList::iterator itr = obj_manager->items.begin(), itr_end = obj_manager->items.end(); itr != itr_end; ++itr )
+		for( SpriteList::iterator itr = sprite_manager->objects.begin(), itr_end = sprite_manager->objects.end(); itr != itr_end; ++itr )
 		{
 			cSprite *obj = (*itr);
 
@@ -787,16 +797,18 @@ bool cEditor :: Key_Down( SDLKey key )
 	// Paste copy buffer objects
 	else if( key == SDLK_INSERT || ( key == SDLK_v && ( input_event.key.keysym.mod & KMOD_LCTRL || input_event.key.keysym.mod & KMOD_RCTRL ) ) )
 	{
-		pMouseCursor->Paste_copy_objects( pMouseCursor->posx, pMouseCursor->posy );
+		pMouseCursor->Paste_copy_objects( static_cast<float>(static_cast<int>(pMouseCursor->posx)), static_cast<float>(static_cast<int>(pMouseCursor->posy)) );
 	}
 	// Cut selected Sprites to the copy buffer
 	else if( key == SDLK_x && ( input_event.key.keysym.mod & KMOD_LCTRL || input_event.key.keysym.mod & KMOD_RCTRL ) )
 	{
 		pMouseCursor->Clear_copy_objects();
 
-		for( unsigned int i = 0; i < pMouseCursor->selected_objects.size(); i++ )
+		for( SelectedObjectList::iterator itr = pMouseCursor->selected_objects.begin(), itr_end = pMouseCursor->selected_objects.end(); itr != itr_end; ++itr )
 		{
-			pMouseCursor->Add_copy_object( pMouseCursor->selected_objects[i]->obj );
+			cSelectedObject *sel_obj = (*itr);
+
+			pMouseCursor->Add_copy_object( sel_obj->obj );
 		}
 
 		pMouseCursor->Delete_selected_objects();
@@ -806,15 +818,17 @@ bool cEditor :: Key_Down( SDLKey key )
 	{
 		pMouseCursor->Clear_copy_objects();
 
-		for( unsigned int i = 0; i < pMouseCursor->selected_objects.size(); i++ )
+		for( SelectedObjectList::iterator itr = pMouseCursor->selected_objects.begin(), itr_end = pMouseCursor->selected_objects.end(); itr != itr_end; ++itr )
 		{
-			pMouseCursor->Add_copy_object( pMouseCursor->selected_objects[i]->obj );
+			cSelectedObject *sel_obj = (*itr);
+
+			pMouseCursor->Add_copy_object( sel_obj->obj );
 		}
 	}
 	// Delete mouse object
 	else if( key == SDLK_DELETE && pMouseCursor->mouse_object->obj )
 	{
-		pMouseCursor->Delete( static_cast<cMovingSprite *>(pMouseCursor->mouse_object->obj) );
+		pMouseCursor->Delete( pMouseCursor->mouse_object->obj );
 	}
 	// if shift got pressed remove mouse object for possible mouse selection
 	else if( ( key == SDLK_RSHIFT || key == SDLK_LSHIFT ) && pMouseCursor->mouse_object->obj )
@@ -836,6 +850,92 @@ bool cEditor :: Key_Down( SDLKey key )
 	return 1;
 }
 
+bool cEditor :: Mouse_Down( Uint8 button )
+{
+	if( !enabled )
+	{
+		return 0;
+	}
+
+	// left
+	if( button == SDL_BUTTON_LEFT )
+	{
+		pMouseCursor->Click();
+		
+		// auto hide if enabled
+		if( pMouseCursor->mouse_object->obj && pPreferences->editor_mouse_auto_hide )
+		{
+			pMouseCursor->Set_Visible( 0 );
+		}
+	}
+	// middle
+	else if( button == SDL_BUTTON_MIDDLE )
+	{
+		// Activate fast copy mode
+		if( pMouseCursor->mouse_object->obj )
+		{
+			pMouseCursor->mode_fastcopy = 1;
+			return 1;
+		}
+		// Mover mode
+		else
+		{
+			pMouseCursor->Toggle_Mover_mode();
+			return 1;
+		}
+	}
+	// right
+	else if( button == SDL_BUTTON_RIGHT )
+	{
+		if( !pMouseCursor->left )
+		{
+			pMouseCursor->Delete( pMouseCursor->mouse_object->obj );
+			return 1;
+		}
+	}
+	else
+	{
+		// not processed
+		return 0;
+	}
+
+	// button got processed
+	return 1;
+}
+
+bool cEditor :: Mouse_Up( Uint8 button )
+{
+	if( !enabled )
+	{
+		return 0;
+	}
+
+	// left
+	if( button == SDL_BUTTON_LEFT )
+	{
+		pMouseCursor->End_selection();
+
+		// unhide
+		if( pPreferences->editor_mouse_auto_hide )
+		{
+			pMouseCursor->Set_Visible( 1 );
+		}
+	}
+	// middle
+	else if( button == SDL_BUTTON_MIDDLE )
+	{
+		pMouseCursor->mode_fastcopy = 0;
+	}
+	else
+	{
+		// not processed
+		return 0;
+	}
+
+	// button got processed
+	return 1;
+}
+
 void cEditor :: Add_Object( cSprite *object )
 {
 	// virtual
@@ -886,7 +986,7 @@ void cEditor :: Add_Menu_Object( string name, string tags, colour normal_color /
 	new_menu->Init();
 
 	// Get Listbox
-	Listbox *listbox_editor_menu = (Listbox *)WindowManager::getSingleton().getWindow( "editor_menu" );
+	Listbox *listbox_editor_menu = static_cast<Listbox *>(WindowManager::getSingleton().getWindow( "editor_menu" ));
 	// Add Listbox item
 	listbox_editor_menu->addItem( new_menu );
 }
@@ -916,7 +1016,7 @@ void cEditor :: Activate_Menu( cEditor_Menu_Object *entry )
 		if( Load_Item_Menu( entry->tags ) )
 		{
 			// Get TabControl
-			TabControl *tabcontrol_menu = (TabControl *)WindowManager::getSingleton().getWindow( "tabcontrol_editor" );
+			TabControl *tabcontrol_menu = static_cast<TabControl *>(WindowManager::getSingleton().getWindow( "tabcontrol_editor" ));
 			// Select Items Tab
 			tabcontrol_menu->setSelectedTab( "editor_tab_items" );
 		}
@@ -975,8 +1075,8 @@ bool cEditor :: Load_Item_Menu( string item_tags )
 			if( tag_pos == array_tags.size() )
 			{
 				GL_Surface *image = pVideo->Get_Surface( settings->base );
-				// Create Sprite
-				cMovingSprite *new_sprite = new cMovingSprite( image );
+				// Create sprite
+				cSprite *new_sprite = new cSprite( image );
 				// default massivetype
 				new_sprite->Set_Sprite_Type( (SpriteType)image->type );
 				// Add new Sprite
@@ -1024,7 +1124,7 @@ void cEditor :: Unload_Item_Menu( void )
 	}
 
 	// Get Items Listbox
-	Listbox *listbox_items = (Listbox *)WindowManager::getSingleton().getWindow( "editor_items" );
+	Listbox *listbox_items = static_cast<Listbox *>(WindowManager::getSingleton().getWindow( "editor_items" ));
 	// Clear Listbox
 	listbox_items->resetList();
 }
@@ -1042,6 +1142,8 @@ void cEditor :: Add_Item_Object( cSprite *sprite, string new_name /* = "" */, GL
 	// set correct array if not given
 	if( sprite->sprite_array == ARRAY_UNDEFINED )
 	{
+		printf( "Warning : Editor sprite %s array not set\n", sprite->name.c_str() );
+
 		if( sprite->massivetype == MASS_PASSIVE )
 		{
 			sprite->sprite_array = ARRAY_PASSIVE;
@@ -1059,6 +1161,8 @@ void cEditor :: Add_Item_Object( cSprite *sprite, string new_name /* = "" */, GL
 	// set correct type if not given
 	if( sprite->type == TYPE_UNDEFINED )
 	{
+		printf( "Warning : Editor sprite %s type not set\n", sprite->name.c_str() );
+
 		if( sprite->massivetype == MASS_PASSIVE )
 		{
 			sprite->type = TYPE_PASSIVE;
@@ -1080,6 +1184,13 @@ void cEditor :: Add_Item_Object( cSprite *sprite, string new_name /* = "" */, GL
 	// if no image is given use the sprite start image
 	if( !image )
 	{
+		// special object
+		if( sprite->type == TYPE_ENEMYSTOPPER || sprite->type == TYPE_LEVELEXIT || sprite->type == TYPE_LEVEL_ENTRY )
+		{
+			sprite->image = pVideo->Get_Surface( "game/editor/special.png" );
+			sprite->start_image = sprite->image;
+		}
+
 		image = sprite->start_image;
 	}
 
@@ -1106,13 +1217,6 @@ void cEditor :: Add_Item_Object( cSprite *sprite, string new_name /* = "" */, GL
 		printf( "Warning : editor object %s with no name given\n", obj_name.c_str() );
 	}
 
-	// check if name is fitting
-	if( obj_name.length() > 25 )
-	{
-		obj_name.erase( 25 );
-		obj_name += "|";
-	}
-
 	cEditor_Item_Object *new_item = new cEditor_Item_Object( obj_name );
 
 	// object pointer
@@ -1122,57 +1226,35 @@ void cEditor :: Add_Item_Object( cSprite *sprite, string new_name /* = "" */, GL
 	new_item->Init();
 
 	// Get Items Listbox
-	Listbox *listbox_items = (Listbox *)WindowManager::getSingleton().getWindow( "editor_items" );
+	Listbox *listbox_items = static_cast<Listbox *>(WindowManager::getSingleton().getWindow( "editor_items" ));
 	// Add Item
 	listbox_items->addItem( new_item );
 }
 
 void cEditor :: Load_Image_Items( string dir )
 {
-	fs::path full_path( dir );
-	fs::directory_iterator end_iter;
+	vector<string> image_files = Get_Directory_files( dir, ".settings" );
 
 	// load all available objects
-	for( fs::directory_iterator dir_itr( full_path ); dir_itr != end_iter; ++dir_itr )
+	for( vector<string>::iterator itr = image_files.begin(), itr_end = image_files.end(); itr != itr_end; ++itr )
 	{
-		try
-		{
-			// if directory
-			if( fs::is_directory( *dir_itr ) )
-			{
-				// ignore hidden directories
-				if( dir_itr->leaf().find( "." ) == 0 )
-				{
-					continue;
-				}
+		// get filename
+		string filename = (*itr);
 
-				// load all items from the sub-directory
-				Load_Image_Items( dir + "/" + dir_itr->leaf() );
-			}
-			// valid file
-			else if( dir_itr->leaf().rfind( ".settings" ) != string::npos )
-			{
-				// load settings
-				string settings_filename = dir + "/" + dir_itr->leaf();
-				cImage_settings_data *settings = pSettingsParser->Get( settings_filename );
+		// load settings
+		cImage_settings_data *settings = pSettingsParser->Get( filename );
 
-				// if settings are available
-				if( settings )
-				{
-					// if required editor tag is available
-					if( settings->editor_tags.find( editor_item_tag ) != string::npos )
-					{
-						// set base to the filename
-						settings->base = dir + "/" + dir_itr->leaf();
-						// add real image
-						tagged_item_images.push_back( settings );
-					}
-				}
-			}
-		}
-		catch( const std::exception &ex )
+		// if settings are available
+		if( settings )
 		{
-			printf( "%s %s\n", dir_itr->leaf().c_str(), ex.what() );
+			// if required editor tag is available
+			if( settings->editor_tags.find( editor_item_tag ) != string::npos )
+			{
+				// set base to the filename
+				settings->base = filename;
+				// add real image
+				tagged_item_images.push_back( settings );
+			}
 		}
 	}
 }
@@ -1187,7 +1269,7 @@ void cEditor :: Activate_Item( cEditor_Item_Object *entry )
 	}
 
 	// create copy from editor item
-	cSprite *new_sprite = Copy_Object( entry->sprite_obj, pMouseCursor->posx, pMouseCursor->posy );
+	cSprite *new_sprite = entry->sprite_obj->Copy();
 
 	// if copying failed
 	if( !new_sprite )
@@ -1195,16 +1277,23 @@ void cEditor :: Activate_Item( cEditor_Item_Object *entry )
 		printf( "Error : Editor Sprite %s copy failed\n", entry->sprite_obj->name.c_str() );
 		return;
 	}
+	
+	new_sprite->Set_Pos( pMouseCursor->posx, pMouseCursor->posy, 1 );
 
 	// hide editor window
 	editor_window->setXPosition( UDim( -0.19f, 0 ) );
+	// Hide Listbox
+	Listbox *listbox = static_cast<Listbox *>(WindowManager::getSingleton().getWindow( "editor_menu" ));
+	listbox->hide();
+	listbox = static_cast<Listbox *>(WindowManager::getSingleton().getWindow( "editor_items" ));
+	listbox->hide();
 
 	Add_Object( new_sprite );
 
 	// Set Mouse Objects
 	pMouseCursor->left = 1;
-	pMouseCursor->mouse_object->mouse_h = (int)( new_sprite->col_rect.h / 2 );
-	pMouseCursor->mouse_object->mouse_w = (int)( new_sprite->col_rect.w / 2 );
+	pMouseCursor->mouse_object->mouse_h = static_cast<int>( new_sprite->col_rect.h / 2 );
+	pMouseCursor->mouse_object->mouse_w = static_cast<int>( new_sprite->col_rect.w / 2 );
 	pMouseCursor->Set_mouse_object( new_sprite );
 }
 
@@ -1257,8 +1346,8 @@ void cEditor :: Draw_Editor_Help( void )
 		Add_Help_Line( "N", "Step one screen to the right ( Next Screen )" );
 		Add_Help_Line( "P", "Step one screen to the left ( Previous Screen )" );
 		Add_Help_Line( "M", "Cycle through massive types (use for object groups)" );
-		Add_Help_Line( "Massive types (with hover color):" );
-		Add_Help_Line( "Massive ->   Halfmassive ->   Climbable ->   Passive ->   Front Passive", "" , 0, 80 );
+		Add_Help_Line( "Massive types (color):" );
+		Add_Help_Line( "Massive(red) ->   Halfmassive(orange) ->   Climbable(lila) ->   Passive(green) ->   Front Passive(green)", "" , 0, 80 );
 		Add_Help_Line( "Ctrl + S", "Quicksave current Level" );
 		Add_Help_Line( "Ctrl + D", "Toggle debug mode" );
 		Add_Help_Line( "Ctrl + A", "Select all objects" );
@@ -1286,13 +1375,6 @@ void cEditor :: Draw_Editor_Help( void )
 	{
 		(*itr)->Draw();
 	}
-
-	// Draw Massivetype colors
-	pVideo->Draw_Rect( 70, 218, 10, 10, 0.581f, &Get_Massivetype_Color( MASS_MASSIVE ) );
-	pVideo->Draw_Rect( 132, 218, 10, 10, 0.581f, &Get_Massivetype_Color( MASS_HALFMASSIVE ) );
-	pVideo->Draw_Rect( 213, 218, 10, 10, 0.581f, &Get_Massivetype_Color( MASS_CLIMBABLE ) );
-    pVideo->Draw_Rect( 284, 218, 10, 10, 0.581f, &Get_Massivetype_Color( MASS_PASSIVE ) );
-    pVideo->Draw_Rect( 343, 218, 10, 10, 0.581f, &Get_Massivetype_Color( MASS_PASSIVE ) );
 }
 
 void cEditor :: Add_Help_Line( string key_text, string text /* = "" */, float spacing /* = 0 */, float pos_x /* = 60 */ )
@@ -1359,15 +1441,15 @@ void cEditor :: Change_DrawPosition( cSprite *obj, bool push_back )
 		return;
 	}
 
-	cObjectManager *obj_manager = NULL;
+	cSprite_Manager *sprite_manager = NULL;
 
 	if( editor_level_enabled )
 	{
-		obj_manager = pObjManager;
+		sprite_manager = pLevel->pSprite_Manager;
 	}
 	else
 	{
-		obj_manager = pActive_Overworld->object_manager;
+		sprite_manager = pActive_Overworld->sprite_manager;
 	}
 
 	cSprite *temp = NULL;
@@ -1375,7 +1457,7 @@ void cEditor :: Change_DrawPosition( cSprite *obj, bool push_back )
 	// push front
 	if( !push_back )
 	{
-		temp = obj_manager->items.back();
+		temp = sprite_manager->objects.back();
 
 		// if already in front
 		if( obj == temp )
@@ -1383,16 +1465,16 @@ void cEditor :: Change_DrawPosition( cSprite *obj, bool push_back )
 			return;
 		}
 
-		obj_manager->Delete( obj, 0 );
-		obj_manager->items.back() = obj;
-		obj_manager->items.insert( obj_manager->items.end() - 1, temp );
+		sprite_manager->Delete( obj, 0 );
+		sprite_manager->objects.back() = obj;
+		sprite_manager->objects.insert( sprite_manager->objects.end() - 1, temp );
 
-		obj->posz = obj_manager->Get_Last( obj->type )->posz + 0.000001f;
+		obj->posz = sprite_manager->Get_Last( obj->type )->posz + 0.000001f;
 	}
 	// push back
 	else
 	{
-		temp = obj_manager->items.front();
+		temp = sprite_manager->objects.front();
 
 		// if already in back
 		if( obj == temp )
@@ -1400,11 +1482,11 @@ void cEditor :: Change_DrawPosition( cSprite *obj, bool push_back )
 			return;
 		}
 
-		obj_manager->Delete( obj, 0 );
-		obj_manager->items.front() = obj;
-		obj_manager->items.insert( obj_manager->items.begin() + 1, temp );
+		sprite_manager->Delete( obj, 0 );
+		sprite_manager->objects.front() = obj;
+		sprite_manager->objects.insert( sprite_manager->objects.begin() + 1, temp );
 
-		obj->posz = obj_manager->Get_First( obj->type )->posz - 0.000001f;
+		obj->posz = sprite_manager->Get_First( obj->type )->posz - 0.000001f;
 	}
 }
 
@@ -1439,7 +1521,7 @@ SpriteList cEditor :: Copy_Direction( SpriteList objects, ObjectDirection dir )
 		}
 
 		// Set X offset
-		offset = (int)( last->startposx - first->startposx + last->start_rect.w );
+		offset = static_cast<int>( last->startposx - first->startposx + last->start_rect.w );
 	}
 	else if( dir == DIR_UP || dir == DIR_DOWN )
 	{
@@ -1466,15 +1548,17 @@ SpriteList cEditor :: Copy_Direction( SpriteList objects, ObjectDirection dir )
 		}
 
 		// Set Y offset
-		offset = (int)( last->startposy - first->startposy + last->start_rect.h );
+		offset = static_cast<int>( last->startposy - first->startposy + last->start_rect.h );
 	}
 
 	// new copied objects
 	SpriteList new_objects;
 
-	for( unsigned int i = 0; i < objects.size(); i++ )
+	for( SpriteList::iterator itr = objects.begin(), itr_end = objects.end(); itr != itr_end; ++itr )
 	{
-		new_objects.push_back( Copy_Direction( objects[i], dir, offset ) );
+		cSprite *obj = (*itr);
+
+		new_objects.push_back( Copy_Direction( obj, dir, offset ) );
 	}
 
 	// return only new objects
@@ -1489,7 +1573,7 @@ cSprite *cEditor :: Copy_Direction( cSprite *obj, ObjectDirection dir, int offse
 	{
 		if( offset )
 		{
-			w = -(float)offset;
+			w = -static_cast<float>(offset);
 		}
 		else
 		{
@@ -1500,7 +1584,7 @@ cSprite *cEditor :: Copy_Direction( cSprite *obj, ObjectDirection dir, int offse
 	{
 		if( offset )
 		{
-			w = (float)offset;
+			w = static_cast<float>(offset);
 		}
 		else
 		{
@@ -1511,7 +1595,7 @@ cSprite *cEditor :: Copy_Direction( cSprite *obj, ObjectDirection dir, int offse
 	{
 		if( offset )
 		{
-			h = -(float)offset;
+			h = -static_cast<float>(offset);
 		}
 		else
 		{
@@ -1522,7 +1606,7 @@ cSprite *cEditor :: Copy_Direction( cSprite *obj, ObjectDirection dir, int offse
 	{
 		if( offset )
 		{
-			h = (float)offset;
+			h = static_cast<float>(offset);
 		}
 		else
 		{
@@ -1552,6 +1636,12 @@ bool cEditor :: Editor_Mouse_Enter( const EventArgs &event )
 	{
 		editor_window->setXPosition( UDim( 0, 0 ) );
 		editor_window->setAlpha( 0 );
+
+		// Show Listbox
+		Listbox *listbox = static_cast<Listbox *>(WindowManager::getSingleton().getWindow( "editor_menu" ));
+		listbox->show();
+		listbox = static_cast<Listbox *>(WindowManager::getSingleton().getWindow( "editor_items" ));
+		listbox->show();
 	}
 	// if active but fading out
 	else if( editor_window->getAlpha() < 1 )
@@ -1560,7 +1650,6 @@ bool cEditor :: Editor_Mouse_Enter( const EventArgs &event )
 		menu_timer = 0;
 	}
 
-
 	return 1;
 }
 
@@ -1572,7 +1661,7 @@ bool cEditor :: Menu_Select( const EventArgs &event )
 	// set item
 	if( item )
 	{
-		Activate_Menu( (cEditor_Menu_Object *)item );
+		Activate_Menu( static_cast<cEditor_Menu_Object *>(item) );
 	}
 	// clear ?
 	else
@@ -1591,7 +1680,7 @@ bool cEditor :: Item_Select( const EventArgs &event )
 	// activate item
 	if( item )
 	{
-		Activate_Item( (cEditor_Item_Object *)item );
+		Activate_Item( static_cast<cEditor_Item_Object *>(item) );
 	}
 
 	return 1;
