@@ -3,14 +3,15 @@
                              -------------------
     copyright            :	(C) 2003 - 2007 by Florian Richter
  ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "../core/game_core.h"
 #include "../core/main.h"
@@ -26,19 +27,35 @@
 #include "../player/player.h"
 #include "../video/gl_surface.h"
 #include "../video/renderer.h"
+#include "../level/level.h"
+// boost filesystem
+#include "boost/filesystem/convenience.hpp"
+namespace fs = boost::filesystem;
+
+// needed to get the user directory
+#ifdef __unix__
+#include <sys/stat.h>
+#elif _WIN32
+#include <shlobj.h>
+#endif
 
 /* *** *** *** *** *** *** *** *** Variables *** *** *** *** *** *** *** *** *** */
 
 bool done = 0;
 
 GameMode Game_Mode = MODE_NOTHING;
+GameModeType Game_Mode_Type = MODE_TYPE_DEFAULT;
+GameAction Game_Action = GA_NONE;
+XMLAttributes Game_Action_Data;
+void *Game_Action_ptr = NULL;
 bool Game_debug = 0;
 
 bool editor_enabled = 0;
 bool editor_level_enabled = 0;
 bool editor_world_enabled = 0;
 
-float global_scalex = 1, global_scaley = 1;
+float global_upscalex = 1, global_upscaley = 1;
+float global_downscalex = 1, global_downscaley = 1;
 
 SDL_Event input_event;
 
@@ -83,7 +100,7 @@ void cDialogBox :: Exit( void )
 	// hide mouse
 	if( mouse_hide )
 	{
-		pMouseCursor->visible = 0;
+		pMouseCursor->Set_Visible( 0 );
 	}
 
 	pGuiSystem->getGUISheet()->removeChildWindow( window );
@@ -146,8 +163,6 @@ string cDialogBox_Text :: Enter( string default_text, string title_text, bool au
 
 	while( !finished )
 	{
-		Draw();
-
 		while( SDL_PollEvent( &input_event ) )
 		{
 			if( input_event.type == SDL_KEYDOWN )
@@ -173,6 +188,10 @@ string cDialogBox_Text :: Enter( string default_text, string title_text, bool au
 					pKeyboard->CEGUI_Handle_Key_Down( input_event.key.keysym.sym );
 				}
 			}
+			else if( input_event.type == SDL_KEYUP )
+			{
+				pKeyboard->CEGUI_Handle_Key_Up( input_event.key.keysym.sym );
+			}
 			else
 			{
 				pMouseCursor->Handle_Event( &input_event );
@@ -180,6 +199,7 @@ string cDialogBox_Text :: Enter( string default_text, string title_text, bool au
 		}
 
 		Update();
+		Draw();
 	}
 
 	string return_string = box_editbox->getText().c_str();
@@ -286,6 +306,10 @@ int cDialogBox_Question :: Enter( string text, bool with_cancel /* = 0 */ )
 					pKeyboard->CEGUI_Handle_Key_Down( input_event.key.keysym.sym );
 				}
 			}
+			else if( input_event.type == SDL_KEYUP )
+			{
+				pKeyboard->CEGUI_Handle_Key_Up( input_event.key.keysym.sym );
+			}
 			else
 			{
 				pMouseCursor->Handle_Event( &input_event );
@@ -339,6 +363,10 @@ void Change_Game_Mode( GameMode new_mode )
 	// mode was level
 	if( old_mode == MODE_LEVEL )
 	{
+		pAnimationManager->Delete_All();
+
+		pJoystick->Reset_keys();
+
 		// hide editor window if visible
 		if( pLevel_Editor->enabled )
 		{
@@ -358,6 +386,8 @@ void Change_Game_Mode( GameMode new_mode )
 	// mode was overworld
 	else if( old_mode == MODE_OVERWORLD )
 	{
+		pAnimationManager->Delete_All();
+
 		// hide editor window if visible
 		if( pWorld_Editor->enabled )
 		{
@@ -372,7 +402,17 @@ void Change_Game_Mode( GameMode new_mode )
 		{
 			// update hud
 			pHudManager->Update_Text();
+
+			// fade out music
+			pAudio->FadeOutMusic( 1000 );
+
+			// reset speedfactor
 			pFramerate->Reset();
+
+			// center camera position
+			pCamera->Center();
+
+			// clear input
 			ClearInputEvents();
 		}
 	}
@@ -382,13 +422,13 @@ void Change_Game_Mode( GameMode new_mode )
 		// hide mouse
 		if( !editor_enabled )
 		{
-			pMouseCursor->visible = 0;
+			pMouseCursor->Set_Visible( 0 );
 		}
 
 		// menu to level
 		if( new_mode == MODE_LEVEL )
 		{
-			pCamera->Center( DIR_ALL );
+			pCamera->Center();
 		}
 		// menu to overworld
 		else if( new_mode == MODE_OVERWORLD )
@@ -396,11 +436,26 @@ void Change_Game_Mode( GameMode new_mode )
 			pHudManager->Update_Text();
 		}
 	}
+	// mode was settings
+	else if( old_mode == MODE_LEVEL_SETTINGS )
+	{
+		//pMouseCursor->Reset( 0 );
+	}
 
 
 	// mode gets level
 	if( new_mode == MODE_LEVEL )
 	{
+		// play music
+		if( pLevel->valid_music )
+		{
+			pAudio->PlayMusic( pLevel->musicfile, -1, 0, 1000 );
+		}
+		else if( pAudio->music_enabled )
+		{
+			printf( "Warning : Music file not found %s\n", pLevel->musicfile.c_str() );
+		}
+
 		// disable world editor
 		pWorld_Editor->Disable();
 
@@ -415,10 +470,22 @@ void Change_Game_Mode( GameMode new_mode )
 				pMouseCursor->visible = 1;
 			}
 		}
+		// enable editor
+		else if( Game_Mode_Type == MODE_TYPE_LEVEL_CUSTOM_EDITOR )
+		{
+			Game_Mode_Type = MODE_TYPE_LEVEL_CUSTOM;
+			pLevel_Editor->Enable();
+		}
 	}
 	// mode gets overworld
 	else if( new_mode == MODE_OVERWORLD )
 	{
+		// reset custom level mode type
+		if( Game_Mode_Type == MODE_TYPE_LEVEL_CUSTOM )
+		{
+			Game_Mode_Type = MODE_TYPE_DEFAULT;
+		}
+
 		// disable level editor
 		pLevel_Editor->Disable();
 
@@ -450,31 +517,14 @@ void Change_Game_Mode( GameMode new_mode )
 	else if( new_mode == MODE_LEVEL_SETTINGS )
 	{
 		editor_enabled = 0;
+
+		if( pMouseCursor->active_object )
+		{
+			pMouseCursor->Clear_active_object();
+		}
 	}
 }
 
-cSprite *Copy_Object( cSprite *copy_object, float x, float y )
-{
-	if( !copy_object )
-	{
-		return NULL;
-	}
-
-	//  copy moving sprite
-	cSprite *new_sprite = copy_object->Copy();
-
-	// if not copyable object
-	if( !new_sprite )
-	{
-		return NULL;
-	}
-
-	// set new position
-	new_sprite->Set_Pos( x, y, 1 );
-
-	return new_sprite;
-}
-
 string Time_to_String( time_t t, const char *format )
 {
 	char str_time[60];
@@ -741,6 +791,46 @@ SpriteType Get_Sprite_Type_id( string str_type )
 	return TYPE_UNDEFINED;
 }
 
+Color Get_Sprite_Color( cSprite *sprite )
+{
+	switch( sprite->sprite_array )
+	{
+	case ARRAY_ENEMY:
+	{
+		return red;
+	}
+	case ARRAY_ACTIVE:
+	{
+		return blue;
+	}
+	case ARRAY_MASSIVE:
+	{
+		if( sprite->type == TYPE_PLAYER )
+		{
+			return lila;
+		}
+
+		return orange;
+	}
+	case ARRAY_PASSIVE:
+	{
+		return green;
+	}
+	case ARRAY_FRONT_PASSIVE:
+	{
+		return greenyellow;
+	}
+	case ARRAY_HUD:
+	{
+		return blackalpha128;
+	}
+	default:
+	    break;
+	}
+
+	return lightgreyalpha64;
+}
+
 string Get_Massivetype_name( MassiveType mtype )
 {
 	switch( mtype )
@@ -817,6 +907,71 @@ Color Get_Massivetype_Color( MassiveType mtype )
 	return white;
 }
 
+string Get_Groundtype_name( GroundType gtype )
+{
+	switch( gtype )
+	{
+	case GROUND_NORMAL:
+	{
+		return "normal";
+	}
+	case GROUND_EARTH:
+	{
+		return "earth";
+	}
+	case GROUND_ICE:
+	{
+		return "ice";
+	}
+	case GROUND_SAND:
+	{
+		return "sand";
+	}
+	case GROUND_STONE:
+	{
+		return "stone";
+	}
+	case GROUND_PLASTIC:
+	{
+		return "plastic";
+	}
+	default:
+	    break;
+	}
+
+	return "";
+}
+
+GroundType Get_Groundtype_id( string str_groundtype )
+{
+	if( str_groundtype.compare( "normal" ) == 0 )
+	{
+		return GROUND_NORMAL;
+	}
+	else if( str_groundtype.compare( "earth" ) == 0 )
+	{
+		return GROUND_EARTH;
+	}
+	else if( str_groundtype.compare( "ice" ) == 0 )
+	{
+		return GROUND_ICE;
+	}
+	else if( str_groundtype.compare( "sand" ) == 0 )
+	{
+		return GROUND_SAND;
+	}
+	else if( str_groundtype.compare( "stone" ) == 0 )
+	{
+		return GROUND_STONE;
+	}
+	else if( str_groundtype.compare( "plastic" ) == 0 )
+	{
+		return GROUND_PLASTIC;
+	}
+
+	return GROUND_NORMAL;
+}
+
 string Get_Color_name( DefaultColor color )
 {
 	switch( color )
@@ -918,34 +1073,37 @@ void Gui_handle_Time( void )
 	float t = 0.001f * SDL_GetTicks();
 
 	// inject the time that passed since the last call
-	System::getSingleton().injectTimePulse( float( t - last_time_pulse ) );
+	System::getSingleton().injectTimePulse( static_cast<float>( t - last_time_pulse ) );
 
 	// store the new time as the last time
 	last_time_pulse = t;
 }
 
-void Draw_StaticText( string text, Color *color_text, Color *color_bg, Color *color_1, Color *color_2, bool wait_for_input )
+void Draw_StaticText( string text, Color *color_text /* = &white */, Color *color_bg /* = NULL */, bool wait_for_input /* = 1 */ )
 {
+	// todo : Can't handle multiple lines. Change to MultiLineEditbox or use HorzFormatting=WordWrapLeftAligned property.
 	bool draw = 1;
 
-	// statictext window
+	// Statictext window
 	Window *window_statictext = WindowManager::getSingleton().loadWindowLayout( "statictext.layout" );
 	pGuiSystem->getGUISheet()->addChildWindow( window_statictext );
 	// get default text
-	Window *text_default = (Window *)WindowManager::getSingleton().getWindow( "text_default" );
+	Window *text_default = static_cast<Window *>(WindowManager::getSingleton().getWindow( "text_default" ));
 
 	// set text
-	text_default->setProperty( "TextColours", PropertyHelper::colourToString( colour( (float)color_text->red / 255, (float)color_text->green / 255, (float)color_text->blue / 255, 1 ) ) );
+	text_default->setProperty( "TextColours", PropertyHelper::colourToString( colour( static_cast<float>(color_text->red) / 255, static_cast<float>(color_text->green) / 255, static_cast<float>(color_text->blue) / 255, 1 ) ) );
 	text_default->setText( text.c_str() );
 
 	// align text
 	Font *font = FontManager::getSingleton().getFont( "bluebold_medium" );
-	float text_width = font->getTextExtent( text.c_str() );
-	text_default->setXPosition( UDim( 0, GAME_RES_W / 2 - text_width / 2 ) );
+	float text_width = font->getTextExtent( text.c_str() ) * global_downscalex;
+
+	text_default->setWidth( UDim( 0, ( text_width + 15 ) * global_upscalex ) );
+	text_default->setXPosition( UDim( 0, ( GAME_RES_W / 2 - text_width / 2 ) * global_upscalex ) );
 	text_default->moveToFront();
 
-	// create centered rect
-	GL_point pos( text_default->getXPosition().asAbsolute( GAME_RES_W ), text_default->getYPosition().asAbsolute( GAME_RES_H ) );
+	// set window height
+	text_default->setHeight( UDim( 0, font->getFontHeight() * font->getFormattedLineCount( text.c_str(), text_default->getUnclippedInnerRect(), LeftAligned ) + ( 12 * global_upscaley ) ) );
 
 	while( draw )
 	{
@@ -964,20 +1122,6 @@ void Draw_StaticText( string text, Color *color_text, Color *color_bg, Color *co
 			pRenderer->Add( request );
 		}
 
-		// create request
-		cRectRequest *request = new cRectRequest();
-		pVideo->Draw_Rect( pos.x - 40, pos.y - 5, text_width + 80, 20 + 20, 0.901f, color_1, request );
-		request->render_count = wait_for_input ? 4 : 1;
-		// add request
-		pRenderer->Add( request );
-
-		// create request
-		request = new cRectRequest();
-		pVideo->Draw_Rect( pos.x - 35, pos.y, text_width + 70, 20 + 10, 0.902f, color_2, request );
-		request->render_count = wait_for_input ? 4 : 1;
-		// add request
-		pRenderer->Add( request );
-
 		pVideo->Render();
 
 		if( wait_for_input )
@@ -1031,6 +1175,8 @@ int Box_Question( string text, bool with_cancel /* = 0 */ )
 
 void Preload_images( void )
 {
+	Draw_Loading_Screen( "Preloading Images" );
+
 	// Maryo
 	Maryo_type old_maryo_type = pPlayer->maryo_type;
 
@@ -1086,12 +1232,16 @@ void Preload_images( void )
 	// Mushrooms
 	pVideo->Get_Surface( "game/items/mushroom_red.png" );
 	pVideo->Get_Surface( "game/items/mushroom_green.png" );
-	// Flower
-	pVideo->Get_Surface( "game/items/fire_flower.png" );
-	pVideo->Get_Surface( "game/items/fire_flower_left.png" );
-	pVideo->Get_Surface( "game/items/fire_flower_right.png" );
+	pVideo->Get_Surface( "game/items/mushroom_blue.png" );
+	pVideo->Get_Surface( "game/items/mushroom_ghost.png" );
+	// Fireplant
+	pVideo->Get_Surface( "game/items/fireplant.png" );
+	pVideo->Get_Surface( "game/items/fireplant_left.png" );
+	pVideo->Get_Surface( "game/items/fireplant_right.png" );
 	// Star
 	pVideo->Get_Surface( "game/items/star.png" );
+	// Feather
+	pVideo->Get_Surface( "game/items/feather_1.png" );
 	// Yellow Goldpiece
 	pVideo->Get_Surface( "game/items/goldpiece/yellow/1.png" );
 	pVideo->Get_Surface( "game/items/goldpiece/yellow/2.png" );
@@ -1099,6 +1249,10 @@ void Preload_images( void )
 	pVideo->Get_Surface( "game/items/goldpiece/yellow/4.png" );
 	pVideo->Get_Surface( "game/items/goldpiece/yellow/5.png" );
 	pVideo->Get_Surface( "game/items/goldpiece/yellow/6.png" );
+	pVideo->Get_Surface( "game/items/goldpiece/yellow/7.png" );
+	pVideo->Get_Surface( "game/items/goldpiece/yellow/8.png" );
+	pVideo->Get_Surface( "game/items/goldpiece/yellow/9.png" );
+	pVideo->Get_Surface( "game/items/goldpiece/yellow/10.png" );
 	// Red Goldpiece
 	pVideo->Get_Surface( "game/items/goldpiece/red/1.png" );
 	pVideo->Get_Surface( "game/items/goldpiece/red/2.png" );
@@ -1106,24 +1260,13 @@ void Preload_images( void )
 	pVideo->Get_Surface( "game/items/goldpiece/red/4.png" );
 	pVideo->Get_Surface( "game/items/goldpiece/red/5.png" );
 	pVideo->Get_Surface( "game/items/goldpiece/red/6.png" );
+	pVideo->Get_Surface( "game/items/goldpiece/red/7.png" );
+	pVideo->Get_Surface( "game/items/goldpiece/red/8.png" );
+	pVideo->Get_Surface( "game/items/goldpiece/red/9.png" );
+	pVideo->Get_Surface( "game/items/goldpiece/red/10.png" );
 
 	// Brown Box
 	pVideo->Get_Surface( "game/box/brown1_1.png" );	
-	// Bonus Box
-	pVideo->Get_Surface( "game/box/yellow/bonus/1.png" );
-	pVideo->Get_Surface( "game/box/yellow/bonus/2.png" );
-	pVideo->Get_Surface( "game/box/yellow/bonus/3.png" );
-	pVideo->Get_Surface( "game/box/yellow/bonus/4.png" );
-	pVideo->Get_Surface( "game/box/yellow/bonus/5.png" );
-	pVideo->Get_Surface( "game/box/yellow/bonus/6.png" );
-	// Default Box
-	pVideo->Get_Surface( "game/box/yellow/default.png" );
-	// Spin Box
-	pVideo->Get_Surface( "game/box/yellow/spin/1.png" );
-	pVideo->Get_Surface( "game/box/yellow/spin/2.png" );
-	pVideo->Get_Surface( "game/box/yellow/spin/3.png" );
-	pVideo->Get_Surface( "game/box/yellow/spin/4.png" );
-	pVideo->Get_Surface( "game/box/yellow/spin/5.png" );
 
 	// Goldpiece animation
 	pVideo->Get_Surface( "animation/light_1/1.png" );
@@ -1131,10 +1274,16 @@ void Preload_images( void )
 	pVideo->Get_Surface( "animation/light_1/3.png" );
 	// Particle animation
 	pVideo->Get_Surface( "animation/particles/smoke.png" );
+	pVideo->Get_Surface( "animation/particles/smoke_black.png" );
 	pVideo->Get_Surface( "animation/particles/light.png" );
+	pVideo->Get_Surface( "animation/particles/dirt.png" );
+	pVideo->Get_Surface( "animation/particles/ice_1.png" );
+	pVideo->Get_Surface( "animation/particles/cloud.png" );
+	pVideo->Get_Surface( "animation/particles/axis.png" );
 
-	// Fireball
+	// Ball
 	pVideo->Get_Surface( "animation/fireball/1.png" );
+	pVideo->Get_Surface( "animation/iceball/1.png" );
 
 	// HUD
 	pVideo->Get_Surface( "game/maryo_l.png" );
@@ -1144,17 +1293,20 @@ void Preload_images( void )
 
 void Preload_sounds( void )
 {
+	Draw_Loading_Screen( "Preloading Sounds" );
+
 	// player
 	pAudio->Get_Sound( "tock.ogg" );
 	pAudio->Get_Sound( "player/dead.ogg" );
 	pAudio->Get_Sound( "itembox_get.ogg" );
 	pAudio->Get_Sound( "itembox_set.ogg" );
-	pAudio->Get_Sound( "player/jump_big_0.ogg" );
-	pAudio->Get_Sound( "player/jump_big_1.ogg" );
-	pAudio->Get_Sound( "player/jump_big_2.ogg" );
 	pAudio->Get_Sound( "player/jump_small.ogg" );
-	pAudio->Get_Sound( "player/maryo_au.ogg" );
+	pAudio->Get_Sound( "player/jump_big.ogg" );
+	pAudio->Get_Sound( "player/jump_ghost.ogg" );
+	// todo : create again
+	//pAudio->Get_Sound( "player/maryo_au.ogg" );
 	pAudio->Get_Sound( "player/powerdown.ogg" );
+	pAudio->Get_Sound( "player/ghost_end.ogg" );
 	pAudio->Get_Sound( "enter_pipe.ogg" );
 	pAudio->Get_Sound( "leave_pipe.ogg" );
 
@@ -1166,32 +1318,36 @@ void Preload_sounds( void )
 	pAudio->Get_Sound( "item/goldpiece_1.ogg" );
 	pAudio->Get_Sound( "item/live_up.ogg" );
 	pAudio->Get_Sound( "item/mushroom.ogg" );
+	pAudio->Get_Sound( "item/mushroom_ghost.ogg" );
 	pAudio->Get_Sound( "item/moon.ogg" );
 
 	// box
 	pAudio->Get_Sound( "death_box.ogg" );
 
 	// enemies
+	// eato
+	pAudio->Get_Sound( "enemy/eato/die.ogg" );
+	// gee
+	pAudio->Get_Sound( "enemy/gee/die.ogg" );
+	// gumba
+	pAudio->Get_Sound( "enemy/gumba/die.ogg" );
+	// jpiranha
+	pAudio->Get_Sound( "enemy/jpiranha/die.ogg" );
+	// rex
+	pAudio->Get_Sound( "enemy/rex/die.ogg" );
 	// rokko
 	pAudio->Get_Sound( "enemy/rokko/activate.ogg" );
+	// spika
+	pAudio->Get_Sound( "enemy/spika/move.ogg" );
 	// thromp
 	pAudio->Get_Sound( "enemy/thromp/hit.ogg" );
 	pAudio->Get_Sound( "enemy/thromp/die.ogg" );
-	// rex
-	pAudio->Get_Sound( "enemy/rex/die.ogg" );
-	// jpiranha
-	pAudio->Get_Sound( "enemy/jpiranha/die.ogg" );
-	// gumba
-	pAudio->Get_Sound( "enemy/gumba/die.ogg" );
 	// turtle
 	pAudio->Get_Sound( "enemy/turtle/hit.ogg" );
 	pAudio->Get_Sound( "enemy/turtle/shell/hit.ogg" );
-	pAudio->Get_Sound( "enemy/turtle/shell/smash_default.ogg" );
-	pAudio->Get_Sound( "enemy/turtle/power_up.ogg" );
-	// gee
-	pAudio->Get_Sound( "enemy/gee/die.ogg" );
-	// spika
-	pAudio->Get_Sound( "enemy/spika/move.ogg" );
+	// todo : create again
+	//pAudio->Get_Sound( "enemy/turtle/power_up.ogg" );
+
 	// default
 	pAudio->Get_Sound( "sprout_1.ogg" );
 	pAudio->Get_Sound( "stomp_1.ogg" );
@@ -1210,31 +1366,6 @@ void Preload_sounds( void )
 	pAudio->Get_Sound( "waypoint_reached.ogg" );
 }
 
-bool Delete_file( const string &filename )
-{
-	if( remove( filename.c_str() ) == 0 )
-	{
-		return 1;
-	}
-	else
-	{
-		return 0;
-	}
-}
-
-bool valid_file( const string &filename )
-{
-	ifstream ifs( filename.c_str(), ios::in );
-
-	if( ifs )
-	{
-		ifs.close();
-		return 1;
-	}
-
-	return 0;
-}
-
 bool is_valid_number( string num, bool accept_floating_point /* = 1 */ )
 {
 	// accept negative numbers
@@ -1257,6 +1388,30 @@ bool is_valid_number( string num, bool accept_floating_point /* = 1 */ )
 	return 0;
 }
 
+bool Delete_file( const string &filename )
+{
+	if( remove( filename.c_str() ) == 0 )
+	{
+		return 1;
+	}
+
+	return 0;
+}
+
+bool file_exists( const string &filename )
+{
+	try
+	{
+		return fs::exists( fs::path( filename, fs::native ) );
+	}
+	catch( const std::exception &ex )
+	{
+		printf( "Warning : file %s exists error : %s\n", filename.c_str(), ex.what() );
+	}
+
+	return 0;
+}
+
 void Convert_path_separators( string &str )
 {
 	for( string::iterator itr = str.begin(); itr != str.end(); ++itr )
@@ -1269,6 +1424,74 @@ void Convert_path_separators( string &str )
 	}
 }
 
+vector<string> Get_Directory_files( string dir, string file_type /* = "" */, bool with_directories /* = 0 */ )
+{
+	vector<string> valid_files;
+
+	fs::path full_path( dir, fs::native );
+	fs::directory_iterator end_iter;
+
+	// load all available objects
+	for( fs::directory_iterator dir_itr( full_path ); dir_itr != end_iter; ++dir_itr )
+	{
+		try
+		{
+			// if directory
+			if( fs::is_directory( *dir_itr ) )
+			{
+				// ignore hidden directories
+				if( dir_itr->leaf().find( "." ) == 0 )
+				{
+					continue;
+				}
+
+				if( with_directories )
+				{
+					valid_files.push_back( dir + "/" + dir_itr->leaf() );
+				}
+
+				// load all items from the sub-directory
+				vector<string> new_valid_files = Get_Directory_files( dir + "/" + dir_itr->leaf(), file_type, with_directories );
+				valid_files.insert( valid_files.end(), new_valid_files.begin(), new_valid_files.end() );
+			}
+			// valid file
+			else if( file_type.empty() || dir_itr->leaf().rfind( file_type ) != string::npos )
+			{
+				valid_files.push_back( dir + "/" + dir_itr->leaf() );
+			}
+		}
+		catch( const std::exception &ex )
+		{
+			printf( "%s %s\n", dir_itr->leaf().c_str(), ex.what() );
+		}
+	}
+
+	return valid_files;
+}
+
+string Get_User_Directory( void )
+{
+#ifdef _WIN32
+	char path[MAX_PATH + 1];
+
+	if( !SUCCEEDED( SHGetFolderPath( NULL, CSIDL_APPDATA, NULL, 0, path ) ) )
+	{
+		printf( "Error : Couldn't get Windows user directory. Using Application directory.\n" );
+		return "";
+	}
+
+	string str_path = path;
+	Convert_path_separators( str_path );
+
+	return str_path + "/smc/";
+#elif __unix__
+	// get linux user data dir ( home directory )
+	return (string)getenv( "HOME" ) + "/.smc/";
+#else
+	return "";
+#endif
+}
+
 string int_to_string( int number )
 {
 	std::ostringstream temp;
@@ -1319,40 +1542,54 @@ double string_to_double( const string &str )
 	return num;
 }
 
-string string_to_xml_string( const string &text )
+string string_to_xml_string( const string &str )
 {
-    string res;
-    res.reserve( text.size() * 2 );
+	string res;
+	res.reserve( str.size() * 2 );
 
-    const string::const_iterator iterEnd = text.end();
-    for( string::const_iterator iter = text.begin(); iter != iterEnd ; ++iter )
-    {
-        switch( *iter )
-        {
-            case '<':
-                res += "&lt;";
-                break;
-      
-            case '>':
-                res += "&gt;";
-                break;
-        
-            case '&':
-                res += "&amp;";
-                break;
-        
-            case '\'':
-                res += "&apos;";
-                break;
+	const string::const_iterator iterEnd = str.end();
+	for( string::const_iterator iter = str.begin(); iter != iterEnd ; ++iter )
+	{
+		switch( *iter )
+		{
+			case '<':
+				res += "&lt;";
+				break;
 
-            case '"':
-                res += "&quot;";
-                break;
-      
-            default:
-                res += *iter;
-        }
-    }
+			case '>':
+				res += "&gt;";
+				break;
 
-    return res;
+			case '&':
+				res += "&amp;";
+				break;
+
+			case '\'':
+				res += "&apos;";
+				break;
+
+			case '"':
+				res += "&quot;";
+				break;
+
+			case '\n':
+				res += "&lt;br/&gt;";
+				break;
+
+			default:
+				res += *iter;
+		}
+	}
+
+	return res;
+}
+
+string xml_string_to_string( string str )
+{
+	while( str.find( "<br/>" ) != string::npos )
+	{
+		str.replace( str.find( "<br/>" ), 5, "\n" );
+	}
+
+	return str;
 }
