@@ -4,21 +4,22 @@
     copyright            :	(C) 2003 - 2007 by Florian Richter
 							(C) 2006 by Tobias Maasland
  ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "../../enemies/bosses/turtle_boss.h"
 #include "../../core/game_core.h"
 #include "../../objects/box.h"
 #include "../../video/animation.h"
 #include "../../player/player.h"
-#include "../../core/obj_manager.h"
+#include "../../level/level.h"
 #include "../../gui/hud.h"
 #include "../../video/gl_surface.h"
 
@@ -84,7 +85,7 @@ cTurtleBoss *cTurtleBoss :: Copy( void )
 void cTurtleBoss :: Create_from_Stream( XMLAttributes &attributes )
 {
 	// position
-	Set_Pos( (float)attributes.getValueAsInteger( "posx" ), (float)attributes.getValueAsInteger( "posy" ), 1 );
+	Set_Pos( static_cast<float>(attributes.getValueAsInteger( "posx" )), static_cast<float>(attributes.getValueAsInteger( "posy" )), 1 );
 	// direction
 	Set_Direction( Get_Direction_id( attributes.getValueAsString( "direction", Get_Direction_name( start_direction ) ).c_str() ), 1 );
 	// color
@@ -104,8 +105,8 @@ void cTurtleBoss :: Save_to_Stream( ofstream &file )
 	// name
 	file << "\t\t<Property name=\"type\" value=\"turtleboss\" />" << std::endl;
 	// position
-	file << "\t\t<Property name=\"posx\" value=\"" << (int)startposx << "\" />" << std::endl;
-	file << "\t\t<Property name=\"posy\" value=\"" << (int)startposy << "\" />" << std::endl;
+	file << "\t\t<Property name=\"posx\" value=\"" << static_cast<int>(startposx) << "\" />" << std::endl;
+	file << "\t\t<Property name=\"posy\" value=\"" << static_cast<int>(startposy) << "\" />" << std::endl;
 	// color
 	file << "\t\t<Property name=\"color\" value=\"" << Get_Color_name( color_type ) << "\" />" << std::endl;
 	// direction
@@ -204,11 +205,10 @@ void cTurtleBoss :: Set_Color( DefaultColor col )
 		return;
 	}
 
-	// clear old images
-	Clear_Images();
-
 	color_type = col;
 
+	Clear_Images();
+
 	if( color_type == COL_RED )
 	{
 		// Walk
@@ -217,7 +217,8 @@ void cTurtleBoss :: Set_Color( DefaultColor col )
 		images.push_back( pVideo->Get_Surface( "enemy/bosses/turtle/walk_2.png" ) );
 		images.push_back( pVideo->Get_Surface( "enemy/bosses/turtle/walk_1.png" ) );
 		// Walk Turn
-		images.push_back( pVideo->Get_Surface( "enemy/bosses/turtle/turn_1.png" ) );
+		//images.push_back( pVideo->Get_Surface( "enemy/bosses/turtle/turn_1.png" ) );
+		images.push_back( NULL );
 		// Shell
 		images.push_back( pVideo->Get_Surface( "enemy/bosses/turtle/shell_front.png" ) );
 		images.push_back( pVideo->Get_Surface( "enemy/bosses/turtle/shell_move_1.png" ) );
@@ -282,12 +283,14 @@ void cTurtleBoss :: DownGrade( bool force /* = 0 */ )
 					pAudio->PlayMusic( "game/courseclear.ogg", 0, 0 );
 
 					// set shell image
-                    Set_Image( 5 );
+					Set_Image( 5 );
 
 					// death state
-                    massivetype = MASS_PASSIVE;
-                    counter = 8;
-                    dead = 1;
+					Set_Dead( 1 );
+					massivetype = MASS_PASSIVE;
+					counter = 8;
+					velx = 0;
+					vely = 0;
                 }
 				// downgrade
                 else
@@ -327,8 +330,8 @@ void cTurtleBoss :: DownGrade( bool force /* = 0 */ )
 	// falling death
 	else
 	{
+		Set_Dead( 1 );
 		counter = 8;
-		dead = 1;
 		massivetype = MASS_PASSIVE;
 
 		if( turtle_state == TURTLEBOSS_WALK )
@@ -374,7 +377,7 @@ void cTurtleBoss :: DieStep( void )
 				anim->Set_Time_to_Live( 1 );
 				anim->Set_Speed( 1, 4 );
 				anim->Set_Scale( 0.4f, 0.3f );
-				anim->Set_Color( orange, Color( (Uint8)6, 60, 20 ) );
+				anim->Set_Color( orange, Color( static_cast<Uint8>(6), 60, 20 ) );
 				anim->Set_Blending( BLEND_ADD );
 				pAnimationManager->Add( anim );
 
@@ -398,10 +401,12 @@ void cTurtleBoss :: DieStep( void )
 	// if below disable
 	else
 	{
-		visible = 0;
-		rotz = 0;
 		state = STA_STAY;
 		turtle_state = TURTLEBOSS_DEAD;
+		Set_Visible( 0 );
+		rotz = 0;
+		velx = 0;
+		vely = 0;
 	}
 }
 
@@ -409,7 +414,7 @@ void cTurtleBoss :: Update( void )
 {
 	cEnemy::Update();
 
-	if( dead || freeze_counter || !is_Player_range() )
+	if( !valid_update || !is_Player_range() )
 	{
 		return;
 	}
@@ -426,7 +431,7 @@ void cTurtleBoss :: Update( void )
 			// if turn around image force the new
 			if( curr_img == 4 )
 			{
-				Set_Image( (int)walk_count );
+				Set_Image( static_cast<int>(walk_count) );
 				Update_Rotation_velx();
 			}
 		}
@@ -434,7 +439,7 @@ void cTurtleBoss :: Update( void )
 		// if turn around image
 		if( curr_img != 4 )
 		{
-			Set_Image( (int)walk_count );
+			Set_Image( static_cast<int>(walk_count) );
 		}
 		else
 		{
@@ -467,7 +472,7 @@ void cTurtleBoss :: Update( void )
 			// animation
 			if( counter < 192 )
 			{
-				if( (int)counter % 5 == 1 )
+				if( static_cast<int>(counter) % 5 == 1 )
 				{
 					Set_Image( 9 ); // active
 				}
@@ -487,7 +492,7 @@ void cTurtleBoss :: Update( void )
 
 		if( velx != 0 )
 		{
-			AddVel( -velx * 0.2f, 0 );
+			Add_Velocity( -velx * 0.2f, 0 );
 
 			if( velx < 0.3f && velx > -0.3f )
 			{
@@ -500,7 +505,7 @@ void cTurtleBoss :: Update( void )
 	{
         run_time += pFramerate->speedfactor;
 		// time = seconds since start
-        float time = ( run_time / DESIRED_FPS );
+        float time = run_time / DESIRED_FPS;
 
         if( time > max_downgrade_time )
 		{
@@ -535,17 +540,17 @@ void cTurtleBoss :: Update( void )
 			walk_count = 0;
 		}
 
-		Set_Image( 6 + (int)walk_count );
+		Set_Image( 6 + static_cast<int>(walk_count) );
 	}
 	else if( turtle_state == TURTLEBOSS_STAND_ANGRY )
 	{
 		counter += pFramerate->speedfactor;
 
 		// angry
-	    if( (int)counter % 2 == 1 )
+	    if( static_cast<int>(counter) % 2 == 1 )
         {
 			// slowly fade to the color
-			cMovingSprite::Set_Color( Color( (Uint8)255, 250 - (Uint8)( counter * 1.5f ), 250 - (Uint8)( counter * 4 ) ) );
+			cMovingSprite::Set_Color( Color( static_cast<Uint8>(255), 250 - static_cast<Uint8>( counter * 1.5f ), 250 - static_cast<Uint8>( counter * 4 ) ) );
         }
 		// default
         else
@@ -554,7 +559,7 @@ void cTurtleBoss :: Update( void )
         }
 
 		// randomize direction
-		if( ( (int)counter / 10 ) % 2 == 1 )
+		if( ( static_cast<int>(counter) / 10 ) % 2 == 1 )
 		{
 			if( direction == DIR_RIGHT )
 			{
@@ -581,8 +586,8 @@ void cTurtleBoss :: Update( void )
 			// shell attack sound
 			pAudio->PlaySound( "enemy/boss/turtle/shell_attack.ogg" );
 
-			// throw more fireballs if hit
-		    Throw_Fireballs( 6 + ( hits * 2 ) );
+			// throw more fireballs with downgrades
+		    Throw_Fireballs( 6 + ( downgrade_counts * 2 ) );
 
             Col_Move( 0, images[0]->col_h - images[5]->col_h, 1, 1 );
             Set_Image( 5 );
@@ -600,8 +605,6 @@ void cTurtleBoss :: Update( void )
 
 	// gravity
 	Update_Gravity();
-
-	CollideMove();
 }
 
 void cTurtleBoss :: Throw_Fireballs( unsigned int amount /* = 6 */ )
@@ -614,9 +617,9 @@ void cTurtleBoss :: Throw_Fireballs( unsigned int amount /* = 6 */ )
 
 		cBall *ball = new cBall( posx + col_rect.w / 2, posy + col_rect.h / 2, 0, 0, this, FIREBALL_EXPLOSION );
 		ball->Set_Direction( ball_angle, 15 );
-		ball->Col_Move( ball->velx * 6, ball->vely + 6, 1 );
+		ball->Col_Move( ball->velx * 2, ball->vely * 2, 1 );
 
-		pObjManager->Add( ball );
+		pLevel->pSprite_Manager->Add( ball );
 	}
 
 	cFireAnimation *anim = new cFireAnimation( posx + ( col_rect.w / 2 ), posy + ( col_rect.h / 3 ), 10 );
@@ -625,6 +628,16 @@ void cTurtleBoss :: Throw_Fireballs( unsigned int amount /* = 6 */ )
 	pAnimationManager->Add( anim );
 }
 
+bool cTurtleBoss :: is_Update_valid( void )
+{
+	if( dead || freeze_counter )
+	{
+		return 0;
+	}
+
+	return 1;
+}
+
 unsigned int cTurtleBoss :: Validate_Collision( cSprite *obj )
 {
 	// basic validation checking
@@ -719,14 +732,14 @@ unsigned int cTurtleBoss :: Validate_Collision( cSprite *obj )
 	return 0;
 }
 
-void cTurtleBoss :: Handle_Collision_Player( ObjectDirection cdirection )
+void cTurtleBoss :: Handle_Collision_Player( cObjectCollision *collision )
 {
-	if( cdirection == DIR_UNDEFINED || ( turtle_state == TURTLEBOSS_SHELL_RUN && playercounter > 0 ) || state == STA_OBJ_LINKED )
+	if( collision->direction == DIR_UNDEFINED || ( turtle_state == TURTLEBOSS_SHELL_RUN && playercounter > 0 ) || state == STA_OBJ_LINKED )
 	{
 		return;
 	}
 
-	if( cdirection == DIR_TOP && pPlayer->state != STA_FLY )
+	if( collision->direction == DIR_TOP && pPlayer->state != STA_FLY )
 	{
 		if( turtle_state == TURTLEBOSS_WALK )
 		{
@@ -738,7 +751,7 @@ void cTurtleBoss :: Handle_Collision_Player( ObjectDirection cdirection )
 			}
 			else
 			{
-				pAudio->PlaySound( "enemy/turtle/hit.ogg" );
+				pAudio->PlaySound( "enemy/boss/turtle/hit.ogg" );
 			}
 		}
 		else if( turtle_state == TURTLEBOSS_SHELL_STAND )
@@ -787,7 +800,7 @@ void cTurtleBoss :: Handle_Collision_Player( ObjectDirection cdirection )
 		if( turtle_state == TURTLEBOSS_WALK )
 		{
 			pPlayer->DownGrade();
-			Turn_Around( cdirection );
+			Turn_Around( collision->direction );
 		}
 		else if( turtle_state == TURTLEBOSS_SHELL_STAND )
 		{
@@ -796,13 +809,13 @@ void cTurtleBoss :: Handle_Collision_Player( ObjectDirection cdirection )
 
 			cParticleAnimation *anim = NULL;
 
-			if( cdirection == DIR_RIGHT )
+			if( collision->direction == DIR_RIGHT )
 			{
 				anim = new cParticleAnimation( posx + col_pos.x + col_rect.w, posy + ( col_rect.h / 2 ) );
 				anim->Set_DirectionRange( 90, 180 );
 				velx = -speed_shell;
 			}
-			else if( cdirection == DIR_LEFT )
+			else if( collision->direction == DIR_LEFT )
 			{
 				anim = new cParticleAnimation( posx, posy + ( col_rect.h / 2 ) );
 				anim->Set_DirectionRange( 270, 180 );
@@ -832,7 +845,7 @@ void cTurtleBoss :: Handle_Collision_Player( ObjectDirection cdirection )
 			anim->Set_Speed( 4, 0.5f );
 			anim->Set_Scale( 0.8f );
 			anim->Set_Fading_Size( 1 );
-			anim->Set_Color( Color( (Uint8)254, 200, 100 ) );
+			anim->Set_Color( Color( static_cast<Uint8>(254), 200, 100 ) );
 			pAnimationManager->Add( anim );
 			
 			Update_Direction();
@@ -840,7 +853,7 @@ void cTurtleBoss :: Handle_Collision_Player( ObjectDirection cdirection )
 			playercounter = DESIRED_FPS * 0.13f;
 
 			// small upwards kick
-			if( cdirection == DIR_BOTTOM )
+			if( collision->direction == DIR_BOTTOM )
 			{
 				vely = -5 + pPlayer->vely / 3;
 			}
@@ -848,10 +861,10 @@ void cTurtleBoss :: Handle_Collision_Player( ObjectDirection cdirection )
 		else if( turtle_state == TURTLEBOSS_SHELL_RUN )
 		{
 			// bottom kicks upwards
-			if( cdirection == DIR_BOTTOM )
+			if( collision->direction == DIR_BOTTOM )
 			{
 				// small upwards kick
-				if( cdirection == DIR_BOTTOM )
+				if( collision->direction == DIR_BOTTOM )
 				{
 					vely = -5 + pPlayer->vely / 3;
 				}
@@ -860,7 +873,7 @@ void cTurtleBoss :: Handle_Collision_Player( ObjectDirection cdirection )
 			else
 			{
 				pPlayer->DownGrade();
-				Turn_Around( cdirection );
+				Turn_Around( collision->direction );
 			}
 		}
 	}
@@ -868,7 +881,7 @@ void cTurtleBoss :: Handle_Collision_Player( ObjectDirection cdirection )
 
 void cTurtleBoss :: Handle_Collision_Enemy( cObjectCollision *collision )
 {
-	cEnemy *enemy = (cEnemy *)pObjManager->Get_Pointer( collision->number );
+	cEnemy *enemy = static_cast<cEnemy *>(pLevel->pSprite_Manager->Get_Pointer( collision->number ));
 
 	if( turtle_state == TURTLEBOSS_SHELL_STAND )
 	{
@@ -883,7 +896,7 @@ void cTurtleBoss :: Handle_Collision_Enemy( cObjectCollision *collision )
 
 			// hit enemy
 			pAudio->PlaySound( enemy->kill_sound );
-			pointsdisplay->Add_Points( enemy->kill_points, posx + image->w / 3, posy - 5, "", (Uint8)255, 1 );
+			pointsdisplay->Add_Points( enemy->kill_points, posx + image->w / 3, posy - 5, "", static_cast<Uint8>(255), 1 );
 			enemy->DownGrade( 1 );
 
 			DownGrade( 1 );
@@ -900,7 +913,7 @@ void cTurtleBoss :: Handle_Collision_Enemy( cObjectCollision *collision )
 
 		// hit enemy
 		pAudio->PlaySound( enemy->kill_sound );
-		pointsdisplay->Add_Points( enemy->kill_points, posx + image->w / 3, posy - 5, "", (Uint8)255, 1 );
+		pointsdisplay->Add_Points( enemy->kill_points, posx + image->w / 3, posy - 5, "", static_cast<Uint8>(255), 1 );
 		enemy->DownGrade( 1 );
 
 		pPlayer->Add_Kill_Multiplier();
@@ -918,7 +931,7 @@ void cTurtleBoss :: Handle_Collision_Enemy( cObjectCollision *collision )
 
 			// hit enemy
 			pAudio->PlaySound( enemy->kill_sound );
-			pointsdisplay->Add_Points( enemy->kill_points, posx + image->w / 3, posy - 5, "", (Uint8)255, 1 );
+			pointsdisplay->Add_Points( enemy->kill_points, posx + image->w / 3, posy - 5, "", static_cast<Uint8>(255), 1 );
 			enemy->DownGrade( 1 );
 		}
 		else
@@ -946,7 +959,7 @@ void cTurtleBoss :: Handle_Collision_Massive( cObjectCollision *collision )
 	{
 		if( collision->direction == DIR_RIGHT || collision->direction == DIR_LEFT )
 		{
-			cSprite *col_object = pObjManager->Get_Pointer( collision->number );
+			cSprite *col_object = pLevel->pSprite_Manager->Get_Pointer( collision->number );
 
 			// animation
 			cParticleAnimation *anim = NULL;
@@ -985,7 +998,7 @@ void cTurtleBoss :: Handle_Collision_Massive( cObjectCollision *collision )
 			if( collision->type == CO_ACTIVE )
 			{
 				// get colliding object
-				cSprite *col_object = pObjManager->Get_Pointer( collision->number );
+				cSprite *col_object = pLevel->pSprite_Manager->Get_Pointer( collision->number );
 
 				if( col_object->type == TYPE_BONUSBOX || col_object->type == TYPE_SPINBOX )
 				{
@@ -1110,7 +1123,7 @@ void cTurtleBoss :: Activate_Turtle( void )
 		Col_Move( 0, images[5]->col_h - images[0]->col_h, 1, 1 );
 	}
 
-	Set_Image( (int)walk_count );
+	Set_Image( static_cast<int>(walk_count) );
 }
 
 void cTurtleBoss :: Create_Name( void )
