@@ -3,23 +3,25 @@
                              -------------------
     copyright            :	(C) 2003 - 2007 by Florian Richter
  ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "../enemies/turtle.h"
 #include "../core/game_core.h"
 #include "../objects/box.h"
 #include "../video/animation.h"
 #include "../player/player.h"
-#include "../core/obj_manager.h"
+#include "../level/level.h"
 #include "../gui/hud.h"
 #include "../video/gl_surface.h"
+#include "../user/savegame.h"
 
 /* *** *** *** *** *** *** cTurtle *** *** *** *** *** *** *** *** *** *** *** */
 
@@ -69,7 +71,7 @@ cTurtle *cTurtle :: Copy( void )
 void cTurtle :: Create_from_Stream( XMLAttributes &attributes )
 {
 	// position
-	Set_Pos( (float)attributes.getValueAsInteger( "posx" ), (float)attributes.getValueAsInteger( "posy" ), 1 );
+	Set_Pos( static_cast<float>(attributes.getValueAsInteger( "posx" )), static_cast<float>(attributes.getValueAsInteger( "posy" )), 1 );
 	// direction
 	Set_Direction( Get_Direction_id( attributes.getValueAsString( "direction", Get_Direction_name( start_direction ) ).c_str() ), 1 );
 	// color
@@ -84,8 +86,8 @@ void cTurtle :: Save_to_Stream( ofstream &file )
 	// name
 	file << "\t\t<Property name=\"type\" value=\"turtle\" />" << std::endl;
 	// position
-	file << "\t\t<Property name=\"posx\" value=\"" << (int)startposx << "\" />" << std::endl;
-	file << "\t\t<Property name=\"posy\" value=\"" << (int)startposy << "\" />" << std::endl;
+	file << "\t\t<Property name=\"posx\" value=\"" << static_cast<int>(startposx) << "\" />" << std::endl;
+	file << "\t\t<Property name=\"posy\" value=\"" << static_cast<int>(startposy) << "\" />" << std::endl;
 	// color
 	file << "\t\t<Property name=\"color\" value=\"" << Get_Color_name( color_type ) << "\" />" << std::endl;
 	// direction
@@ -95,6 +97,39 @@ void cTurtle :: Save_to_Stream( ofstream &file )
 	file << "\t</enemy>" << std::endl;
 }
 
+void cTurtle :: Load_from_Savegame( cSave_Level_Object *save_object )
+{
+	cEnemy::Load_from_Savegame( save_object );
+
+	// turtle_state
+	if( save_object->exists( "turtle_state" ) )
+	{
+		turtle_state = (Turtle_state)string_to_int( save_object->Get_Value( "turtle_state" ) );
+
+		if( turtle_state == TURTLE_SHELL_STAND || turtle_state == TURTLE_SHELL_RUN )
+		{
+			// todo : create a Set_State function
+			// set shell image without position changes
+			cSprite::Set_Image( images[5] );
+		}
+	}
+	
+	Update_Rotation_velx();
+}
+
+cSave_Level_Object *cTurtle :: Save_to_Savegame( void )
+{
+	cSave_Level_Object *save_object = cEnemy::Save_to_Savegame();
+
+	// turtle_state ( only save if needed )
+	if( turtle_state != TURTLE_WALK )
+	{
+		save_object->properties.push_back( cSave_Level_Object_Property( "turtle_state", int_to_string( turtle_state ) ) );
+	}
+
+	return save_object;
+}
+
 void cTurtle :: Set_Direction( ObjectDirection dir, bool new_start_direction /* = 0 */ )
 {
 	if( dir != DIR_RIGHT && dir != DIR_LEFT )
@@ -144,9 +179,6 @@ void cTurtle :: Set_Color( DefaultColor col )
 		return;
 	}
 
-	// clear old images
-	Clear_Images();
-
 	color_type = col;
 
 	string filename_dir;
@@ -173,13 +205,16 @@ void cTurtle :: Set_Color( DefaultColor col )
 		printf( "Error : Unknown Turtle color : %d\n", color_type );
 	}
 
+	Clear_Images();
+
 	// Walk
 	images.push_back( pVideo->Get_Surface( "enemy/turtle/" + filename_dir + "/walk_0.png" ) );
 	images.push_back( pVideo->Get_Surface( "enemy/turtle/" + filename_dir + "/walk_1.png" ) );
 	images.push_back( pVideo->Get_Surface( "enemy/turtle/" + filename_dir + "/walk_2.png" ) );
 	images.push_back( pVideo->Get_Surface( "enemy/turtle/" + filename_dir + "/walk_1.png" ) );
 	// Walk Turn
-	images.push_back( pVideo->Get_Surface( "enemy/turtle/" + filename_dir + "/turn_1.png" ) );
+	//images.push_back( pVideo->Get_Surface( "enemy/turtle/" + filename_dir + "/turn_1.png" ) );
+	images.push_back( NULL );
 	// Shell
 	images.push_back( pVideo->Get_Surface( "enemy/turtle/" + filename_dir + "/shell_front.png" ) );
 	images.push_back( pVideo->Get_Surface( "enemy/turtle/" + filename_dir + "/shell_move_1.png" ) );
@@ -247,8 +282,10 @@ void cTurtle :: DownGrade( bool force /* = 0 */ )
 	else
 	{
 		counter = 8;
-		dead = 1;
+		Set_Dead( 1 );
 		massivetype = MASS_PASSIVE;
+		velx = 0;
+		vely = 0;
 
 		if( turtle_state == TURTLE_WALK )
 		{
@@ -293,7 +330,7 @@ void cTurtle :: DieStep( void )
 	// if below disable
 	else
 	{
-		visible = 0;
+		Set_Visible( 0 );
 		rotz = 0;
 		state = STA_STAY;
 		turtle_state = TURTLE_DEAD;
@@ -304,7 +341,7 @@ void cTurtle :: Update( void )
 {
 	cEnemy::Update();
 
-	if( dead || freeze_counter || state == STA_OBJ_LINKED || !is_Player_range() )
+	if( !valid_update || !is_Player_range() )
 	{
 		return;
 	}
@@ -321,7 +358,7 @@ void cTurtle :: Update( void )
 			// if turn around image force the new
 			if( curr_img == 4 )
 			{
-				Set_Image( (int)walk_count );
+				Set_Image( static_cast<int>(walk_count) );
 				Update_Rotation_velx();
 			}
 		}
@@ -329,7 +366,7 @@ void cTurtle :: Update( void )
 		// if not turn around image
 		if( curr_img != 4 )
 		{
-			Set_Image( (int)walk_count );
+			Set_Image( static_cast<int>(walk_count) );
 		}
 		else
 		{
@@ -362,7 +399,7 @@ void cTurtle :: Update( void )
 			// animation
 			if( counter < 192 )
 			{
-				if( (int)counter % 5 == 1 )
+				if( static_cast<int>(counter) % 5 == 1 )
 				{
 					Set_Image( 9 ); // active
 				}
@@ -390,7 +427,8 @@ void cTurtle :: Update( void )
 				Update_Direction();
 				Update_Rotation_velx();
 
-				pAudio->PlaySound( "enemy/turtle/power_up.ogg" );
+				// todo : create again
+				//pAudio->PlaySound( "enemy/turtle/power_up.ogg" );
 
 				if( curr_img > 4 )
 				{
@@ -404,7 +442,7 @@ void cTurtle :: Update( void )
 
 		if( velx != 0 )
 		{
-			AddVel( -velx * 0.2f, 0 );
+			Add_Velocity( -velx * 0.2f, 0 );
 
 			if( velx < 0.3f && velx > -0.3f )
 			{
@@ -417,14 +455,12 @@ void cTurtle :: Update( void )
 	{
 		walk_count += pFramerate->speedfactor * 0.4f;
 
-
-
 		if( walk_count >= 3 )
 		{
 			walk_count = 0;
 		}
 
-		Set_Image( 6 + (int)walk_count );
+		Set_Image( 6 + static_cast<int>(walk_count) );
 	}
 
 	if( playercounter > 0 )
@@ -447,8 +483,16 @@ void cTurtle :: Update( void )
 
 	// gravity
 	Update_Gravity();
+}
 
-	CollideMove();
+bool cTurtle :: is_Update_valid( void )
+{
+	if( dead || freeze_counter || state == STA_OBJ_LINKED )
+	{
+		return 0;
+	}
+
+	return 1;
 }
 
 unsigned int cTurtle :: Validate_Collision( cSprite *obj )
@@ -540,14 +584,14 @@ unsigned int cTurtle :: Validate_Collision( cSprite *obj )
 	return 0;
 }
 
-void cTurtle :: Handle_Collision_Player( ObjectDirection cdirection )
+void cTurtle :: Handle_Collision_Player( cObjectCollision *collision )
 {
-	if( cdirection == DIR_UNDEFINED || ( turtle_state == TURTLE_SHELL_RUN && playercounter > 0 ) || state == STA_OBJ_LINKED )
+	if( collision->direction == DIR_UNDEFINED || ( turtle_state == TURTLE_SHELL_RUN && playercounter > 0 ) || state == STA_OBJ_LINKED )
 	{
 		return;
 	}
 
-	if( cdirection == DIR_TOP && pPlayer->state != STA_FLY )
+	if( collision->direction == DIR_TOP && pPlayer->state != STA_FLY )
 	{
 		if( turtle_state == TURTLE_WALK )
 		{
@@ -600,7 +644,7 @@ void cTurtle :: Handle_Collision_Player( ObjectDirection cdirection )
 		if( turtle_state == TURTLE_WALK )
 		{
 			pPlayer->DownGrade();
-			Turn_Around( cdirection );
+			Turn_Around( collision->direction );
 		}
 		else if( turtle_state == TURTLE_SHELL_STAND )
 		{
@@ -608,13 +652,13 @@ void cTurtle :: Handle_Collision_Player( ObjectDirection cdirection )
 			DownGrade();
 
 			cParticleAnimation *anim = NULL;
-			if( cdirection == DIR_RIGHT )
+			if( collision->direction == DIR_RIGHT )
 			{
 				anim = new cParticleAnimation( posx + col_pos.x + col_rect.w, posy + ( col_rect.h / 2 ) );
 				anim->Set_DirectionRange( 90, 180 );
 				velx = -speed_shell;
 			}
-			else if( cdirection == DIR_LEFT )
+			else if( collision->direction == DIR_LEFT )
 			{
 				anim = new cParticleAnimation( posx, posy + ( col_rect.h / 2 ) );
 				anim->Set_DirectionRange( 270, 180 );
@@ -644,7 +688,7 @@ void cTurtle :: Handle_Collision_Player( ObjectDirection cdirection )
 			anim->Set_Speed( 4, 0.5f );
 			anim->Set_Scale( 0.6f );
 			anim->Set_Fading_Size( 1 );
-			anim->Set_Color( Color( (Uint8)254, 200, 100 ) );
+			anim->Set_Color( Color( static_cast<Uint8>(254), 200, 100 ) );
 			pAnimationManager->Add( anim );
 
 			Update_Direction();
@@ -652,7 +696,7 @@ void cTurtle :: Handle_Collision_Player( ObjectDirection cdirection )
 			playercounter = DESIRED_FPS * 0.13f;
 
 			// small upwards kick
-			if( cdirection == DIR_BOTTOM )
+			if( collision->direction == DIR_BOTTOM )
 			{
 				vely = -5 + pPlayer->vely / 3;
 			}
@@ -660,10 +704,10 @@ void cTurtle :: Handle_Collision_Player( ObjectDirection cdirection )
 		else if( turtle_state == TURTLE_SHELL_RUN )
 		{
 			// bottom kicks upwards
-			if( cdirection == DIR_BOTTOM )
+			if( collision->direction == DIR_BOTTOM )
 			{
 				// small upwards kick
-				if( cdirection == DIR_BOTTOM )
+				if( collision->direction == DIR_BOTTOM )
 				{
 					vely = -5 + pPlayer->vely / 3;
 				}
@@ -672,7 +716,7 @@ void cTurtle :: Handle_Collision_Player( ObjectDirection cdirection )
 			else
 			{
 				pPlayer->DownGrade();
-				Turn_Around( cdirection );
+				Turn_Around( collision->direction );
 			}
 		}
 	}
@@ -682,7 +726,7 @@ void cTurtle :: Handle_Collision_Enemy( cObjectCollision *collision )
 {
 	if( turtle_state == TURTLE_SHELL_STAND )
 	{
-		cEnemy *enemy = (cEnemy *)pObjManager->Get_Pointer( collision->number );
+		cEnemy *enemy = static_cast<cEnemy *>(pLevel->pSprite_Manager->Get_Pointer( collision->number ));
 
 		// if able to collide
 		if( state == STA_OBJ_LINKED || vely < -5 )
@@ -695,7 +739,7 @@ void cTurtle :: Handle_Collision_Enemy( cObjectCollision *collision )
 			
 			// hit enemy
 			pAudio->PlaySound( enemy->kill_sound );
-			pointsdisplay->Add_Points( enemy->kill_points, posx + image->w / 3, posy - 5, "", (Uint8)255, 1 );
+			pointsdisplay->Add_Points( enemy->kill_points, posx + image->w / 3, posy - 5, "", static_cast<Uint8>(255), 1 );
 			enemy->DownGrade( 1 );
 
 			DownGrade( 1 );
@@ -704,7 +748,7 @@ void cTurtle :: Handle_Collision_Enemy( cObjectCollision *collision )
 	}
 	else if( turtle_state == TURTLE_SHELL_RUN )
 	{
-		cEnemy *enemy = (cEnemy *)pObjManager->Get_Pointer( collision->number );
+		cEnemy *enemy = static_cast<cEnemy *>(pLevel->pSprite_Manager->Get_Pointer( collision->number ));
 
 		
 		// don't collide with already dead enemies
@@ -733,9 +777,34 @@ void cTurtle :: Handle_Collision_Enemy( cObjectCollision *collision )
 
 		// hit enemy
 		pAudio->PlaySound( enemy->kill_sound );
-		pointsdisplay->Add_Points( enemy->kill_points, posx + image->w / 3, posy - 5, "", (Uint8)255, 1 );
+		pointsdisplay->Add_Points( enemy->kill_points, posx + image->w / 3, posy - 5, "", static_cast<Uint8>(255), 1 );
 		enemy->DownGrade( 1 );
 
+		// create animation
+		cParticleAnimation *anim = new cParticleAnimation();
+
+		anim->Set_Emitter_Rect( col_rect.x + ( col_rect.w * 0.2f ), col_rect.y + ( col_rect.h * 0.2f ), col_rect.w * 0.6f, col_rect.h * 0.8f );
+		anim->Set_Image( pVideo->Get_Surface( "animation/particles/light.png" ) );
+		anim->Set_Quota( 5 );
+		anim->Set_ZPos( posz + 0.000001f );
+		anim->Set_Time_to_Live( 0.3f );
+		anim->Set_Speed( 1.2f, 0.8f );
+		anim->Set_Scale( 0.7f );
+		anim->Set_Fading_Alpha( 1 );
+		anim->Set_Blending( BLEND_ADD );
+
+		if( collision->direction == DIR_RIGHT )
+		{
+			anim->Set_DirectionRange( 0 );
+		}
+		else if( collision->direction == DIR_LEFT )
+		{
+			anim->Set_DirectionRange( 180 );
+		}
+		
+		// add animation
+		pAnimationManager->Add( anim );
+
 		pPlayer->Add_Kill_Multiplier();
 	}
 	else if( turtle_state == TURTLE_WALK )
@@ -762,7 +831,7 @@ void cTurtle :: Handle_Collision_Massive( cObjectCollision *collision )
 	{
 		if( collision->direction == DIR_RIGHT || collision->direction == DIR_LEFT )
 		{
-			cSprite *col_object = pObjManager->Get_Pointer( collision->number );
+			cSprite *col_object = pLevel->pSprite_Manager->Get_Pointer( collision->number );
 
 			// animation
 			cParticleAnimation *anim = NULL;
@@ -800,7 +869,7 @@ void cTurtle :: Handle_Collision_Massive( cObjectCollision *collision )
 			if( collision->type == CO_ACTIVE )
 			{
 				// get colliding object
-				cSprite *col_object = pObjManager->Get_Pointer( collision->number );
+				cSprite *col_object = pLevel->pSprite_Manager->Get_Pointer( collision->number );
 
 				if( col_object->type == TYPE_BONUSBOX || col_object->type == TYPE_SPINBOX )
 				{
