@@ -3,14 +3,15 @@
                              -------------------
     copyright            :	(C) 2003 - 2007 by Florian Richter
  ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/ 
+/*
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "../gui/hud.h"
 #include "../core/game_core.h"
@@ -20,7 +21,7 @@
 #include "../video/font.h"
 #include "../core/framerate.h"
 #include "../level/level.h"
-#include "../core/obj_manager.h"
+#include "../core/sprite_manager.h"
 #include "../objects/bonusbox.h"
 #include "../video/gl_surface.h"
 #include "../video/renderer.h"
@@ -58,6 +59,7 @@ cHudSprite *cHudSprite :: Copy( void )
 /* *** *** *** *** *** *** *** cHudManager *** *** *** *** *** *** *** *** *** *** */
 
 cHudManager :: cHudManager( void )
+: cObject_Manager<cHudSprite>()
 {
 	loaded = 0;
 
@@ -82,25 +84,25 @@ void cHudManager :: Load( void )
 	}
 
 	// Menu Background ( Maryo head and the Goldpiece )
-	Add( (cHudSprite *)new cMenuBackground() );
+	Add( static_cast<cHudSprite *>(new cMenuBackground()) );
 	// Point Display
 	pointsdisplay = new cPlayerPoints();
-	Add( (cHudSprite *)pointsdisplay );
+	Add( static_cast<cHudSprite *>(pointsdisplay) );
 	// Time Display
 	timedisplay = new cTimeDisplay();
-	Add( (cHudSprite *)timedisplay );
+	Add( static_cast<cHudSprite *>(timedisplay) );
 	// Live Display
 	livedisplay = new cLiveDisplay();
-	Add( (cHudSprite *)livedisplay );
+	Add( static_cast<cHudSprite *>(livedisplay) );
 	// Gold Display
 	golddisplay = new cGoldDisplay();
-	Add( (cHudSprite *)golddisplay );
+	Add( static_cast<cHudSprite *>(golddisplay) );
 	// Itembox
 	Itembox = new cItemBox();
-	Add( (cHudSprite *)Itembox );
+	Add( static_cast<cHudSprite *>(Itembox) );
 	// Debug Display
 	debugdisplay = new cDebugDisplay();
-	Add( (cHudSprite *)debugdisplay );
+	Add( static_cast<cHudSprite *>(debugdisplay) );
 
 	loaded = 1;
 }
@@ -112,12 +114,7 @@ void cHudManager :: Unload( void )
 		return;
 	}
 	
-	for( HudSpriteList::iterator itr = objects.begin(), itr_end = objects.end(); itr != itr_end; ++itr )
-	{
-		delete *itr;
-	}
-
-	objects.clear();
+	Delete_All();
 
 	livedisplay = NULL;
 	golddisplay = NULL;
@@ -129,23 +126,13 @@ void cHudManager :: Unload( void )
 	loaded = 0;
 }
 
-void cHudManager :: Add( cHudSprite *obj )
-{
-	if( !obj )
-	{
-		return;
-	}
-
-	objects.push_back( obj );
-}
-
 void cHudManager :: Update_Text( void )
 {
-	// note : update the lifedisplay before you update the timedisplay
+	// note : update the life display before you update the timedisplay
 	
 	if( objects.size() > 0 )
 	{
-		cMenuBackground *item = (cMenuBackground *)objects[0];
+		cMenuBackground *item = static_cast<cMenuBackground *>(objects[0]);
 
 		if( Game_Mode != MODE_OVERWORLD )
 		{
@@ -155,7 +142,7 @@ void cHudManager :: Update_Text( void )
 		else
 		{
 			// goldpiece
-			item->rect_goldpiece.y = 4;
+			item->rect_goldpiece.y = 7;
 		}
 	}
 
@@ -163,7 +150,7 @@ void cHudManager :: Update_Text( void )
 	{
 		if( Game_Mode != MODE_OVERWORLD )
 		{
-			livedisplay->Set_Pos( GAME_RES_W - GAME_RES_W / 10, 18.0f );
+			livedisplay->Set_Pos( GAME_RES_W - GAME_RES_W * 0.1f, 18 );
 		}
 		else
 		{
@@ -177,11 +164,11 @@ void cHudManager :: Update_Text( void )
 	{
 		if( Game_Mode != MODE_OVERWORLD )
 		{
-			golddisplay->Set_Pos( 280.0, 18.0 );
+			golddisplay->Set_Pos( 280, 18 );
 		}
 		else
 		{
-			golddisplay->Set_Pos( 280.0, 1.0 );
+			golddisplay->Set_Pos( 280, 4 );
 		}
 
 		golddisplay->AddGold( 0 );
@@ -191,11 +178,11 @@ void cHudManager :: Update_Text( void )
 	{
 		if( Game_Mode != MODE_OVERWORLD )
 		{
-			pointsdisplay->Set_Pos( 50.0, 18.0 );
+			pointsdisplay->Set_Pos( 50, 18 );
 		}
 		else
 		{
-			pointsdisplay->Set_Pos( 50.0, 4 );
+			pointsdisplay->Set_Pos( 50, 4 );
 		}
 
 		pointsdisplay->Add_Points( 0 );
@@ -203,19 +190,19 @@ void cHudManager :: Update_Text( void )
 
 	if( timedisplay )
 	{
-		timedisplay->Set_Pos( GAME_RES_W * 0.75, 18.0 );
+		timedisplay->Set_Pos( GAME_RES_W * 0.70f, 18 );
 		timedisplay->Update();
 	}
 
 	if( debugdisplay )
 	{
-		debugdisplay->Set_Pos( GAME_RES_W / 2 - 100, 80.0 );
+		debugdisplay->Set_Pos( GAME_RES_W * 0.45f, 80 );
 		debugdisplay->Update();
 	}
 
 	if( Itembox ) 
 	{
-		Itembox->Set_Pos( GAME_RES_W / 2 - 20, 10 );
+		Itembox->Set_Pos( GAME_RES_W * 0.49f, 10 );
 		Itembox->Update();
 	}
 }
@@ -238,11 +225,6 @@ void cHudManager :: Draw( void )
 	}
 }
 
-unsigned int cHudManager :: Get_Size( void )
-{
-	return (unsigned int)objects.size();
-}
-
 /* *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** *** */
 
 cHudManager *pHudManager = NULL;
@@ -395,7 +377,7 @@ void cPlayerPoints :: Draw( cSurfaceRequest *request /* = NULL */ )
 			// out in right
 			else if( x > GAME_RES_W )
 			{
-				x = (float)GAME_RES_W - obj->col_rect.w - 2; 
+				x = static_cast<float>(GAME_RES_W) - obj->col_rect.w - 2; 
 			}
 
 			// out on bottom
@@ -406,7 +388,7 @@ void cPlayerPoints :: Draw( cSurfaceRequest *request /* = NULL */ )
 			// out on top
 			else if( y > GAME_RES_H )
 			{
-				y = (float)GAME_RES_H - obj->col_rect.h - 20;
+				y = static_cast<float>(GAME_RES_H) - obj->col_rect.h - 20;
 			}
 
 			// create request
@@ -418,7 +400,7 @@ void cPlayerPoints :: Draw( cSurfaceRequest *request /* = NULL */ )
 			request->shadow_pos = 1;
 
 			// color
-			request->color = Color( (Uint8)( 255 - ( obj->points / 150 ) ), (Uint8)( 255 - ( obj->points / 150 ) ), (Uint8)( 255 - ( obj->points / 30 ) ) );
+			request->color = Color( static_cast<Uint8>( 255 - ( obj->points / 150 ) ), static_cast<Uint8>( 255 - ( obj->points / 150 ) ), static_cast<Uint8>( 255 - ( obj->points / 30 ) ) );
 
 			// add request
 			pRenderer->Add( request );
@@ -433,11 +415,11 @@ void cPlayerPoints :: Set_Points( unsigned int points )
 	pPlayer->points = points;
 	
 	char text[70];
-	sprintf( text, "Points %08d", (int)pPlayer->points );
+	sprintf( text, "Points %08d", static_cast<int>(pPlayer->points) );
 	Set_Image( pFont->RenderText( pFont->font_normal, text, white ), 0, 1 );
 }
 
-void cPlayerPoints :: Add_Points( unsigned int points, float x /* = 0 */, float y /* = 0 */, string strtext /* = "" */, Color color /* = (Uint8)255 */, bool allow_multiplier /* = 0 */ )
+void cPlayerPoints :: Add_Points( unsigned int points, float x /* = 0 */, float y /* = 0 */, string strtext /* = "" */, Color color /* = static_cast<Uint8>(255) */, bool allow_multiplier /* = 0 */ )
 {
 	if( !pPlayer )
 	{
@@ -446,7 +428,7 @@ void cPlayerPoints :: Add_Points( unsigned int points, float x /* = 0 */, float
 	
 	if( allow_multiplier )
 	{
-		points = (unsigned int)( pPlayer->kill_multiplier * (float)points );
+		points = static_cast<unsigned int>( pPlayer->kill_multiplier * static_cast<float>(points) );
 	}
 
 	Set_Points( pPlayer->points + points );
@@ -502,7 +484,8 @@ cTimeDisplay :: cTimeDisplay( float x /* = 0 */, float y /* = 0 */ )
 	sprite_array = ARRAY_HUD;
 	type = TYPE_GAMETIMEDISPLAY;
 
-	counter = -1;
+	old_seconds = 0;
+	counter = 0;
 }
 
 cTimeDisplay :: ~cTimeDisplay( void )
@@ -510,7 +493,7 @@ cTimeDisplay :: ~cTimeDisplay( void )
 	//
 }
 
-void cTimeDisplay :: Draw( cSurfaceRequest *request /* = NULL */ )
+void cTimeDisplay :: Update( void )
 {
 	if( editor_enabled || Game_Mode == MODE_OVERWORLD || Game_Mode == MODE_MENU )
 	{
@@ -519,15 +502,27 @@ void cTimeDisplay :: Draw( cSurfaceRequest *request /* = NULL */ )
 
 	counter += pFramerate->speedfactor;
 
-	int time = (int)( counter / DESIRED_FPS );	// time = seconds since start
-	int minutes = (int)( time / 60 );
+	// get seconds since start
+	unsigned int seconds = static_cast<unsigned int>( counter / DESIRED_FPS );
 
-	sprintf( text, "Time %02d:%02d", minutes, time - ( minutes * 60 ) );
-
-	if( strcmp( text, text_old ) != 0 )
+	// no change
+	if( seconds == old_seconds )
 	{
-		strcpy( text_old, text );
-		Set_Image( pFont->RenderText( pFont->font_normal, text, white ), 0, 1 );
+		return;
+	}
+
+	unsigned int minutes = seconds / 60;
+
+	// Set new time
+	sprintf( text, "Time %02d:%02d", minutes, seconds - ( minutes * 60 ) );
+	Set_Image( pFont->RenderText( pFont->font_normal, text, white ), 0, 1 );
+}
+
+void cTimeDisplay :: Draw( cSurfaceRequest *request /* = NULL */ )
+{
+	if( editor_enabled || Game_Mode == MODE_OVERWORLD || Game_Mode == MODE_MENU )
+	{
+		return;
 	}
 
 	cHudSprite::Draw();
@@ -563,20 +558,11 @@ cItemBox :: ~cItemBox( void )
 	delete item;
 }
 
-void cItemBox :: Draw( cSurfaceRequest *request /* = NULL */ )
+void cItemBox :: Update( void )
 {
-	if( editor_enabled || Game_Mode == MODE_OVERWORLD || Game_Mode == MODE_MENU )
+	if( item_counter )
 	{
-		return;
-	}
-
-	if( pPlayer->maryo_type == MARYO_DEAD )
-	{
-		item_counter = 90;
-	}
-	else if( item_counter )
-	{
-		item->posy += pFramerate->speedfactor * 4;
+		item->Move( 0, 4 );
 
 		if( item_counter_mod )
 		{
@@ -604,23 +590,31 @@ void cItemBox :: Draw( cSurfaceRequest *request /* = NULL */ )
 			Reset();
 		}
 
-		cObjectCollisionType col_list = item->ColCheckAuto( 0, 0, 0, 0, COLLIDE_ONLY_CHECK );
+		cObjectCollisionType col_list = item->Collision_Check( &item->col_rect, COLLIDE_ONLY_CHECK );
 
-		// if colliding with the Player
-		if( col_list.find( CO_PLAYER ) )
+		// if colliding with the player
+		if( col_list.find( TYPE_PLAYER ) )
 		{
 			SpriteType item_id_temp = item_id; // player can set the item back
 			Reset();
 			pPlayer->Get_Item( item_id_temp, 1 );
 		}
 	}
+}
+
+void cItemBox :: Draw( cSurfaceRequest *request /* = NULL */ )
+{
+	if( editor_enabled || Game_Mode == MODE_OVERWORLD || Game_Mode == MODE_MENU )
+	{
+		return;
+	}
 
 	if( item_id && item->image )
 	{
 		// with alpha
 		if( item_counter )
 		{
-			item->Set_Color( 255, 255, 255, 100 + (Uint8)item_counter );
+			item->Set_Color( 255, 255, 255, 100 + static_cast<Uint8>(item_counter) );
 		}
 		
 		item->Draw();
@@ -630,9 +624,14 @@ void cItemBox :: Draw( cSurfaceRequest *request /* = NULL */ )
 	cHudSprite::Draw();
 }
 
-void cItemBox :: Set_Item( SpriteType item_type )
+void cItemBox :: Set_Item( SpriteType item_type, bool sound /* = 1 */ )
 {
-	pAudio->PlaySound( "itembox_set.ogg" );
+	// play sound
+	if( sound )
+	{
+		pAudio->PlaySound( "itembox_set.ogg" );
+	}
+	// reset data
 	Reset();
 
 	// reset startposition
@@ -642,17 +641,17 @@ void cItemBox :: Set_Item( SpriteType item_type )
 
 	if( item_type == TYPE_MUSHROOM_DEFAULT )
 	{
-		box_color = Color( (Uint8)250, 50, 50 );
+		box_color = Color( static_cast<Uint8>(250), 50, 50 );
 		item->Set_Image( pVideo->Get_Surface( "game/items/mushroom_red.png" ) );
 	}
 	else if( item_type == TYPE_FIREPLANT )
 	{
-		box_color = Color( (Uint8)250, 200, 150 );
-		item->Set_Image( pVideo->Get_Surface( "game/items/fire_flower.png" ) );
+		box_color = Color( static_cast<Uint8>(250), 200, 150 );
+		item->Set_Image( pVideo->Get_Surface( "game/items/fireplant.png" ) );
 	}
 	else if( item_type == TYPE_MUSHROOM_BLUE )
 	{
-		box_color = Color( (Uint8)100, 100, 250 );
+		box_color = Color( static_cast<Uint8>(100), 100, 250 );
 		item->Set_Image( pVideo->Get_Surface( "game/items/mushroom_blue.png" ) );
 	}
 
@@ -708,21 +707,31 @@ cDebugDisplay :: cDebugDisplay( float x /* = 0 */, float y /* = 0 */ )
 	sprite_array = ARRAY_HUD;
 	type = TYPE_DEBUGDISPLAY;
 
-	level_old = ".";
 	text.clear();
 	text_old.clear();
 
+	// debug box text data
+	Game_Mode_last = MODE_NOTHING;
+	level_old = ".";
+	obj_counter = -1;
+	pass_counter = -1;
+	mass_counter = -1;
+	enemy_counter = -1;
+	active_counter = -1;
+
 	// debug text window
 	window_debugtext = WindowManager::getSingleton().loadWindowLayout( "debugtext.layout" );
 	pGuiSystem->getGUISheet()->addChildWindow( window_debugtext );
 	// debug text
-	text_debugtext = (Window *)WindowManager::getSingleton().getWindow( "text_debugmessage" );
+	text_debugtext = static_cast<Window *>(WindowManager::getSingleton().getWindow( "text_debugmessage" ));
+	// hide
+	text_debugtext->setVisible( 0 );
 
 	// debug box positions
 	float tempx = GAME_RES_W - 200;
 	float tempy = GAME_RES_H / 2 - 250;
 
-	for( unsigned int i = 0; i < 20; i++ )
+	for( unsigned int i = 0; i < 23; i++ )
 	{
 		sprites.push_back( new cHudSprite( NULL, tempx, tempy ) );
 		sprites[i]->Set_Shadow( black, 1 );
@@ -735,7 +744,7 @@ cDebugDisplay :: cDebugDisplay( float x /* = 0 */, float y /* = 0 */ )
 		}
 
 		// active box
-		if( i > 9 && i < 14 )
+		if( i > 10 && i < 16 )
 		{
 			sprites[i]->Set_PosX( sprites[i]->startposx += 5, 1 );
 		}
@@ -744,18 +753,13 @@ cDebugDisplay :: cDebugDisplay( float x /* = 0 */, float y /* = 0 */ )
 	// fps display position
 	sprites[0]->Set_Pos( 15, 5, 1 );
 	// average
-	sprites[1]->Set_Pos( 280, 5, 1 );
+	sprites[1]->Set_Pos( 290, 5, 1 );
 	// speedfactor
-	sprites[2]->Set_Pos( 500, 5, 1 );
+	sprites[2]->Set_Pos( 480, 5, 1 );
 
 	// Debug type text
-	sprites[4]->Set_Image( pFont->RenderText( pFont->font_small, "Level information", lightblue ), 0, 1 );
-	sprites[16]->Set_Image( pFont->RenderText( pFont->font_small, "Player information", lightblue ), 0, 1 );
-
-	passcount = -1;
-	masscount = -1;
-	enemycount = -1;
-	activecount = -1;
+	sprites[4]->Set_Image( pFont->RenderText( pFont->font_small, "Level", lightblue ), 0, 1 );
+	sprites[16]->Set_Image( pFont->RenderText( pFont->font_small, "Player", lightblue ), 0, 1 );
 
 	counter = 0;
 }
@@ -765,19 +769,16 @@ cDebugDisplay :: ~cDebugDisplay( void )
 	pGuiSystem->getGUISheet()->removeChildWindow( window_debugtext );
 	WindowManager::getSingleton().destroyWindow( window_debugtext );
 
-	for( unsigned int i = 0; i < sprites.size(); i++ )
+	for( HudSpriteList::iterator itr = sprites.begin(), itr_end = sprites.end(); itr != itr_end; ++itr )
 	{
-		delete sprites[i];
+		delete *itr;
 	}
 
 	sprites.clear();
 }
 
-void cDebugDisplay :: Draw( cSurfaceRequest *request /* = NULL */ )
+void cDebugDisplay :: Update( void )
 {
-	// draw complete debug info
-	GameDebugDraw();
-
 	// no text to display
 	if( text.empty() )
 	{
@@ -808,9 +809,17 @@ void cDebugDisplay :: Draw( cSurfaceRequest *request /* = NULL */ )
 
 		// update position
 		Font *font = FontManager::getSingleton().getFont( "bluebold_medium" );
-		text_debugtext->setXPosition( UDim( 0, GAME_RES_W / 2 - font->getTextExtent( text ) / 2 ) );
-	}
+		float text_width = font->getTextExtent( text.c_str() ) * global_downscalex;
 
+		text_debugtext->setWidth( UDim( 0, ( text_width + 15 ) * global_upscalex ) );
+		text_debugtext->setXPosition( UDim( 0, ( GAME_RES_W / 2 - text_width / 2 ) * global_upscalex ) );
+	}
+}
+
+void cDebugDisplay :: Draw( cSurfaceRequest *request /* = NULL */ )
+{
+	// debugbox info
+	DebugBox_Draw();
 }
 
 void cDebugDisplay :: Set_Text( string ntext, float display_time /* = DESIRED_FPS * 2 */ )
@@ -829,7 +838,7 @@ void cDebugDisplay :: Set_Text( string ntext, float display_time /* = DESIRED_FP
 	counter = display_time;
 }
 
-void cDebugDisplay :: GameDebugDraw( void )
+void cDebugDisplay :: DebugBox_Draw( void )
 {
 	if( !Game_debug || ( Game_Mode == MODE_LEVEL && pPlayer->maryo_type == MARYO_DEAD ) ) 
 	{
@@ -839,73 +848,93 @@ void cDebugDisplay :: GameDebugDraw( void )
 	string temp_text;
 
 	/* ### Extend
-	- Other Special ( Active ) Objects
+	- Ground Object name, position and velocity
 	*/
 
 	// black background
 	Color color = blackalpha128;
-	pVideo->Draw_Rect( GAME_RES_W - 205, GAME_RES_H / 12, 180, 325, posz - 0.00001f, &color );
+	pVideo->Draw_Rect( GAME_RES_W - 205, GAME_RES_H * 0.08f, 190, 390, posz - 0.00001f, &color );
+
+	// Active objects rect
+
+	// create request
+	cRectRequest *request = new cRectRequest();
+	pVideo->Draw_Rect( sprites[11]->posx - 8, sprites[11]->posy - 4, 135, 95, posz, &white, request );
+
+	// not filled
+	request->filled = 0;
+
+	// add request
+	pRenderer->Add( request );
 
 	// ### Frames per Second
-	temp_text = "FPS : best " + int_to_string( (int)pFramerate->fps_best ) + ", worst " + int_to_string( (int)pFramerate->fps_worst ) + ", current " + int_to_string( (int)pFramerate->fps );
+	temp_text = "FPS : best " + int_to_string( static_cast<int>(pFramerate->fps_best) ) + ", worst " + int_to_string( static_cast<int>(pFramerate->fps_worst) ) + ", current " + int_to_string( static_cast<int>(pFramerate->fps) );
 	sprites[0]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
 	// average
-	temp_text = "average " + int_to_string( (int)pFramerate->fps_average );
+	temp_text = "average " + int_to_string( static_cast<int>(pFramerate->fps_average) );
 	sprites[1]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
 	// speedfactor
 	temp_text = "Speedfactor " + float_to_string( pFramerate->speedfactor );
 	sprites[2]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
 
 	// ### Camera position
-	temp_text = "Camera : X " + int_to_string( (int)pCamera->x ) + ", Y " + int_to_string( (int)pCamera->y );
+	temp_text = "Camera : X " + int_to_string( static_cast<int>(pCamera->x) ) + ", Y " + int_to_string( static_cast<int>(pCamera->y) );
 	sprites[3]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
 
 	// ### Level informations
 	if( pLevel->levelfile.compare( level_old ) != 0 ) 
 	{
-		string lvl_text = "Level : " + Get_filename( pLevel->data_file, 0, 0 );
+		string lvl_text = "Name : " + Get_filename( pLevel->data_file, 0, 0 );
 		level_old = pLevel->levelfile;
 		
 		sprites[5]->Set_Image( pFont->RenderText( pFont->font_very_small, lvl_text, white ), 0, 1 );
 	}
 
 	// ### Level Objects
-	// # Passive
-	if( passcount != (int)(pObjManager->Get_Size_array( ARRAY_PASSIVE ) + pObjManager->Get_Size_array( ARRAY_FRONT_PASSIVE )) ) 
+	// All
+	if( obj_counter != static_cast<int>(pLevel->pSprite_Manager->size()) )
 	{
-		passcount = pObjManager->Get_Size_array( ARRAY_PASSIVE ) + pObjManager->Get_Size_array( ARRAY_FRONT_PASSIVE );
+		obj_counter = pLevel->pSprite_Manager->size();
 
-		temp_text = "Passive Objects : " + int_to_string( passcount );
+		temp_text = "Objects : " + int_to_string( obj_counter );
 		sprites[6]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
 	}
-	// # Massive
-	if( masscount != (int)pObjManager->Get_Size_array( ARRAY_MASSIVE ) ) 
+	// Passive
+	if( pass_counter != static_cast<int>(pLevel->pSprite_Manager->Get_Size_array( ARRAY_PASSIVE ) + pLevel->pSprite_Manager->Get_Size_array( ARRAY_FRONT_PASSIVE )) )
 	{
-		masscount = pObjManager->Get_Size_array( ARRAY_MASSIVE );
+		pass_counter = pLevel->pSprite_Manager->Get_Size_array( ARRAY_PASSIVE ) + pLevel->pSprite_Manager->Get_Size_array( ARRAY_FRONT_PASSIVE );
 
-		temp_text = "Massive Objects : " + int_to_string( masscount );
+		temp_text = "Passive : " + int_to_string( pass_counter );
 		sprites[7]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
 	}
-	// # Enemy
-	if( enemycount != (int)pObjManager->Get_Size_array( ARRAY_ENEMY ) ) 
+	// Massive
+	if( mass_counter != static_cast<int>(pLevel->pSprite_Manager->Get_Size_array( ARRAY_MASSIVE )) )
 	{
-		enemycount = pObjManager->Get_Size_array( ARRAY_ENEMY );
+		mass_counter = pLevel->pSprite_Manager->Get_Size_array( ARRAY_MASSIVE );
 
-		temp_text ="Enemy Objects : " + int_to_string( enemycount );
+		temp_text = "Massive : " + int_to_string( mass_counter );
 		sprites[8]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
 	}
-	// # Active
-	if( activecount != (int)pObjManager->Get_Size_array( ARRAY_ACTIVE ) ) 
+	// Enemy
+	if( enemy_counter != static_cast<int>(pLevel->pSprite_Manager->Get_Size_array( ARRAY_ENEMY )) ) 
 	{
-		activecount = pObjManager->Get_Size_array( ARRAY_ACTIVE );
+		enemy_counter = pLevel->pSprite_Manager->Get_Size_array( ARRAY_ENEMY );
 
-		temp_text = "Active Objects : " + int_to_string( activecount );
+		temp_text ="Enemy : " + int_to_string( enemy_counter );
 		sprites[9]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
+	}
+	// Active
+	if( active_counter != static_cast<int>(pLevel->pSprite_Manager->Get_Size_array( ARRAY_ACTIVE )) )
+	{
+		active_counter = pLevel->pSprite_Manager->Get_Size_array( ARRAY_ACTIVE );
 
-		// # Active : Halfmassive
+		temp_text = "Active : " + int_to_string( active_counter );
+		sprites[10]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
+
+		// Halfmassive
 		unsigned int halfmassive = 0;
 
-		for( ObjectList::iterator itr = pObjManager->items.begin(), itr_end = pObjManager->items.end(); itr != itr_end; ++itr )
+		for( SpriteList::iterator itr = pLevel->pSprite_Manager->objects.begin(), itr_end = pLevel->pSprite_Manager->objects.end(); itr != itr_end; ++itr )
 		{
 			// get object pointer
 			cSprite *obj = (*itr);
@@ -917,27 +946,29 @@ void cDebugDisplay :: GameDebugDraw( void )
 		}
 
 		temp_text = "Halfmassive : " + int_to_string( halfmassive );
-		sprites[10]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
-		// # Active : Moving Platform
-		unsigned int clouds = 0;
+		sprites[11]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
 
-		for( ObjectList::iterator itr = pObjManager->items.begin(), itr_end = pObjManager->items.end(); itr != itr_end; ++itr )
+		// Moving Platform
+		unsigned int moving_platform = 0;
+
+		for( SpriteList::iterator itr = pLevel->pSprite_Manager->objects.begin(), itr_end = pLevel->pSprite_Manager->objects.end(); itr != itr_end; ++itr )
 		{
 			// get object pointer
 			cSprite *obj = (*itr);
 
 			if( obj->type == TYPE_MOVING_PLATFORM )
 			{
-				clouds++;
+				moving_platform++;
 			}
 		}
 
-		temp_text = "Moving Platform : " + int_to_string( clouds );
-		sprites[11]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
-		// # Active : Goldbox
+		temp_text = "Moving Platform : " + int_to_string( moving_platform );
+		sprites[12]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
+
+		// Goldbox
 		unsigned int goldbox = 0;
 
-		for( ObjectList::iterator itr = pObjManager->items.begin(), itr_end = pObjManager->items.end(); itr != itr_end; ++itr )
+		for( SpriteList::iterator itr = pLevel->pSprite_Manager->objects.begin(), itr_end = pLevel->pSprite_Manager->objects.end(); itr != itr_end; ++itr )
 		{
 			// get object pointer
 			cSprite *obj = (*itr);
@@ -954,11 +985,12 @@ void cDebugDisplay :: GameDebugDraw( void )
 		}
 
 		temp_text = "Goldbox : " + int_to_string( goldbox );
-		sprites[12]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
-		// # Active : Bonusbox
+		sprites[13]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
+
+		// Bonusbox
 		unsigned int bonusbox = 0;
 
-		for( ObjectList::iterator itr = pObjManager->items.begin(), itr_end = pObjManager->items.end(); itr != itr_end; ++itr )
+		for( SpriteList::iterator itr = pLevel->pSprite_Manager->objects.begin(), itr_end = pLevel->pSprite_Manager->objects.end(); itr != itr_end; ++itr )
 		{
 			// get object pointer
 			cSprite *obj = (*itr);
@@ -975,61 +1007,47 @@ void cDebugDisplay :: GameDebugDraw( void )
 		}
 
 		temp_text = "Bonusbox : " + int_to_string( bonusbox );
-		sprites[13]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
-		// # Active : Other
-		unsigned int active_other = pObjManager->Get_Size_array( ARRAY_ACTIVE );
+		sprites[14]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
 
-		for( ObjectList::iterator itr = pObjManager->items.begin(), itr_end = pObjManager->items.end(); itr != itr_end; ++itr )
-		{
-			// get object pointer
-			cSprite *obj = (*itr);
-
-			if( obj->type == TYPE_HALFMASSIVE || obj->type == TYPE_BONUSBOX || obj->type == TYPE_MOVING_PLATFORM )
-			{
-				active_other--;
-			}
-		}
+		// Other
+		unsigned int active_other = active_counter - halfmassive - moving_platform - goldbox - bonusbox;
 
 		temp_text = "Other : " + int_to_string( active_other );
-		sprites[14]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
+		sprites[15]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
 	}
 
-	// # Active Objects Rect
 
-	// create request
-	cRectRequest *request = new cRectRequest();
-	GL_rect r( sprites[10]->posx - 8, sprites[10]->posy, 135, 75 );
-	pVideo->Draw_Rect( &r, posz, &white, request );
-
-	// not filled
-	request->filled = 0;
-
-	// add request
-	pRenderer->Add( request );
-
-
-	// ### Player informations
-	// position
-	temp_text = "Position X " + int_to_string( (int)pPlayer->posx ) + " ,Y " + int_to_string( (int)pPlayer->posy );
-	sprites[16]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
-	// velocity
-	temp_text = "Velocity X " + int_to_string( (int)pPlayer->velx ) + " ,Y " + int_to_string( (int)pPlayer->vely );
+	// ### Player information
+	// position x
+	temp_text = "X1 " + float_to_string( pPlayer->posx ) + "  X2 " + float_to_string( pPlayer->col_rect.x + pPlayer->col_rect.w );
 	sprites[17]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
-	// state
-	temp_text = "Moving State " + int_to_string( (int)pPlayer->state );
+	// position y
+	temp_text = "Y1 " + float_to_string( pPlayer->posy ) + "  Y2 " + float_to_string( pPlayer->col_rect.y + pPlayer->col_rect.h );
 	sprites[18]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
-	// onground
-	int groundtype = 0;
+	// velocity
+	temp_text = "Velocity X " + int_to_string( static_cast<int>(pPlayer->velx) ) + " ,Y " + int_to_string( (int)pPlayer->vely );
+	sprites[19]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
+	// state
+	temp_text = "Moving State " + int_to_string( static_cast<int>(pPlayer->state) );
+	sprites[20]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
+	// ground type
+	string ground_type;
 	if( pPlayer->ground_object )
 	{
-		groundtype = (int)pPlayer->ground_object->massivetype;
+		ground_type = int_to_string( pPlayer->ground_object->massivetype ) + " (" + Get_Massivetype_name( pPlayer->ground_object->massivetype ) + ")";
 	}
-	temp_text = "Ground type " + int_to_string( groundtype );
-	sprites[19]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
-
-	for( unsigned int i = 0; i < sprites.size(); i++ )
+	temp_text = "Ground " + ground_type;
+	sprites[21]->Set_Image( pFont->RenderText( pFont->font_very_small, temp_text, white ), 0, 1 );
+	// game mode
+	if( Game_Mode != Game_Mode_last )
 	{
-		sprites[i]->Draw();
+		sprites[22]->Set_Image( pFont->RenderText( pFont->font_very_small, "Game Mode : " + int_to_string( Game_Mode ), white ), 0, 1 );
+	}
+
+	// draw text
+	for( HudSpriteList::iterator itr = sprites.begin(), itr_end = sprites.end(); itr != itr_end; ++itr )
+	{
+		(*itr)->Draw();
 	}
 }
 
@@ -1064,7 +1082,7 @@ void cGoldDisplay :: SetGold( int gold )
 	if( gold >= 100 )
 	{
 		gold -= 100;
-		pAudio->PlaySound( "item/live_up.ogg" );	
+		pAudio->PlaySound( "item/live_up_2.ogg" );	
 		livedisplay->AddLives( 1 );
 
 		pointsdisplay->Add_Points( 0, pPlayer->posx + pPlayer->image->w/3, pPlayer->posy + 5, "1UP", lightred );
@@ -1073,7 +1091,7 @@ void cGoldDisplay :: SetGold( int gold )
 	pPlayer->goldpieces = gold;
 	string text = int_to_string( pPlayer->goldpieces );
 
-	Color color = Color( (Uint8)255, 255, 255 - ( gold * 2 ) );
+	Color color = Color( static_cast<Uint8>(255), 255, 255 - ( gold * 2 ) );
 
 	Set_Image( pFont->RenderText( pFont->font_normal, text, color ), 0, 1 );
 }
@@ -1116,10 +1134,10 @@ void cLiveDisplay :: SetLives( int lives )
 {
 	pPlayer->lives = lives;
 	
-	string text;
-
 	if( lives >= 0 )
 	{
+		string text;
+
 		// if not in Overworld
 		if( Game_Mode != MODE_OVERWORLD )
 		{
