@@ -3,14 +3,15 @@
                              -------------------
     copyright            :	(C) 2003 - 2007 by Florian Richter
  ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "../core/globals.h"
 #include "../core/main.h"
@@ -23,7 +24,7 @@
 #include "../overworld/ow_player.h"
 #include "../gui/menu.h"
 #include "../core/camera.h"
-#include "../core/obj_manager.h"
+#include "../level/level.h"
 #include "../player/player.h"
 #include "../user/preferences.h"
 #include "../core/framerate.h"
@@ -89,6 +90,9 @@ cMouseCursor :: cMouseCursor( void )
 	Reset_keys();
 
 	SDL_GetMouseState( &x, &y );
+	// scale to the virtual game size
+	x = static_cast<int>( static_cast<float>(x) * global_downscalex );
+	y = static_cast<int>( static_cast<float>(y) * global_downscaley );
 }
 
 cMouseCursor :: ~cMouseCursor( void )
@@ -141,28 +145,17 @@ bool cMouseCursor :: Handle_Event( SDL_Event *ev )
 	{
 		case SDL_MOUSEMOTION:
 		{
-			pGuiSystem->injectMousePosition( (float)ev->motion.x, (float)ev->motion.y );
+			pGuiSystem->injectMousePosition( static_cast<float>(ev->motion.x), static_cast<float>(ev->motion.y) );
+			Update_Position();
 			break;
 		}
 		case SDL_MOUSEBUTTONUP:
 		{
 			if( Handle_Mouse_up( ev->button.button ) )
 			{
-				// GUI handled
+				// processed
 				return 1;
 			}
-			else if( ev->button.button == SDL_BUTTON_LEFT )
-			{
-				left = 0;
-			}
-			else if( ev->button.button == SDL_BUTTON_MIDDLE )
-			{
-				middle = 0;
-			}
-			else if( ev->button.button == SDL_BUTTON_RIGHT )
-			{
-				right = 0;
-			}
 
 			break;
 		}
@@ -170,21 +163,10 @@ bool cMouseCursor :: Handle_Event( SDL_Event *ev )
 		{
 			if( Handle_Mouse_down( ev->button.button ) )
 			{
-				// GUI handled
+				// processed
 				return 1;
 			}
-			else if( ev->button.button == SDL_BUTTON_LEFT )
-			{
-				left = 1;
-			}
-			else if( ev->button.button == SDL_BUTTON_MIDDLE )
-			{
-				middle = 1;
-			}
-			else if( ev->button.button == SDL_BUTTON_RIGHT )
-			{
-				right = 1;
-			}
+
 			break;
 		}
 		default:
@@ -193,54 +175,56 @@ bool cMouseCursor :: Handle_Event( SDL_Event *ev )
 		}
 	}
 
-	// editor events
-	if( pWorld_Editor->enabled )
-	{
-		if( pWorld_Editor->Handle_Event( ev ) )
-		{
-			return 1;
-		}
-	}
-	else if( pLevel_Editor->enabled )
-	{
-		if( pLevel_Editor->Handle_Event( ev ) )
-		{
-			return 1;
-		}
-	}
-
 	return 0;
 }
 
 bool cMouseCursor :: Handle_Mouse_down( Uint8 button )
 {
-	switch ( button )
+	switch( button )
 	{
 		// mouse buttons
 		case SDL_BUTTON_LEFT:
 		{
-			return System::getSingleton().injectMouseButtonDown( LeftButton );
+			if( System::getSingleton().injectMouseButtonDown( LeftButton ) )
+			{
+				return 1;
+			}
+			left = 1;
 			break;
 		}
 		case SDL_BUTTON_MIDDLE:
 		{
-			return System::getSingleton().injectMouseButtonDown( MiddleButton );
+			if( System::getSingleton().injectMouseButtonDown( MiddleButton ) )
+			{
+				return 1;
+			}
+			middle = 1;
 			break;
 		}
 		case SDL_BUTTON_RIGHT:
 		{
-			return System::getSingleton().injectMouseButtonDown( RightButton );
+			if( System::getSingleton().injectMouseButtonDown( RightButton ) )
+			{
+				return 1;
+			}
+			right = 1;
 			break;
 		}
 		// mouse wheel
 		case SDL_BUTTON_WHEELDOWN:
 		{
-			return System::getSingleton().injectMouseWheelChange( -1 );
+			if( System::getSingleton().injectMouseWheelChange( -1 ) )
+			{
+				return 1;
+			}
 			break;
 		}
 		case SDL_BUTTON_WHEELUP:
 		{
-			return System::getSingleton().injectMouseWheelChange( +1 );
+			if( System::getSingleton().injectMouseWheelChange( +1 ) )
+			{
+				return 1;
+			}
 			break;
 		}
 		default:
@@ -249,6 +233,32 @@ bool cMouseCursor :: Handle_Mouse_down( Uint8 button )
 		}
 	}
 
+	// handle button in the current mode
+	if( Game_Mode == MODE_LEVEL )
+	{
+		// processed by the level
+		if( pLevel->Mouse_Down( button ) )
+		{
+			return 1;
+		}
+	}
+	else if( Game_Mode == MODE_OVERWORLD )
+	{
+		// processed by the overworld
+		if( pActive_Overworld->Mouse_Down( button ) )
+		{
+			return 1;
+		}
+	}
+	else if( Game_Mode == MODE_MENU )
+	{
+		// processed by the menu
+		if( pMenuCore->Mouse_Down( button ) )
+		{
+			return 1;
+		}
+	}
+
 	return 0;
 }
 
@@ -258,17 +268,29 @@ bool cMouseCursor :: Handle_Mouse_up( Uint8 button )
 	{
 		case SDL_BUTTON_LEFT:
 		{
-			return System::getSingleton().injectMouseButtonUp( LeftButton );
+			left = 0;
+			if( System::getSingleton().injectMouseButtonUp( LeftButton ) )
+			{
+				return 1;
+			}
 		}
 		break;
 		case SDL_BUTTON_MIDDLE:
 		{
-			return System::getSingleton().injectMouseButtonUp( MiddleButton );
+			middle = 0;
+			if( System::getSingleton().injectMouseButtonUp( MiddleButton ) )
+			{
+				return 1;
+			}
 		}
 		break;
 		case SDL_BUTTON_RIGHT:
 		{
-			return System::getSingleton().injectMouseButtonUp( RightButton );
+			right = 0;
+			if( System::getSingleton().injectMouseButtonUp( RightButton ) )
+			{
+				return 1;
+			}
 		}
 		break;
 		default:
@@ -277,6 +299,32 @@ bool cMouseCursor :: Handle_Mouse_up( Uint8 button )
 		}
 	}
 
+	// handle button in the current mode
+	if( Game_Mode == MODE_LEVEL )
+	{
+		// processed by the level
+		if( pLevel->Mouse_Up( button ) )
+		{
+			return 1;
+		}
+	}
+	else if( Game_Mode == MODE_OVERWORLD )
+	{
+		// processed by the overworld
+		if( pActive_Overworld->Mouse_Up( button ) )
+		{
+			return 1;
+		}
+	}
+	else if( Game_Mode == MODE_MENU )
+	{
+		// processed by the menu
+		if( pMenuCore->Mouse_Up( button ) )
+		{
+			return 1;
+		}
+	}
+
 	return 0;
 }
 
@@ -374,11 +422,11 @@ bool cMouseCursor :: Editor_Collsion_Check( float px /* = 0 */, float py /* = 0
 
 bool cMouseCursor :: Get_Collision_Level( GL_rect *mouse_rect )
 {
-	ObjectList level_objects;
-	pObjManager->Get_Objects_sorted( level_objects, 1, 1 );
+	SpriteList level_objects;
+	pLevel->pSprite_Manager->Get_Objects_sorted( level_objects, 1, 1 );
 
 	// objects before player
-	for( ObjectList::reverse_iterator itr = level_objects.rbegin(), itr_end = level_objects.rend(); itr != itr_end; ++itr )
+	for( SpriteList::reverse_iterator itr = level_objects.rbegin(), itr_end = level_objects.rend(); itr != itr_end; ++itr )
 	{
 		cSprite *obj = (*itr);
 
@@ -396,7 +444,7 @@ bool cMouseCursor :: Get_Collision_Level( GL_rect *mouse_rect )
 			}
 			else
 			{
-				Collision_Add( this, obj, pObjManager->Get_Array_num( obj ), Get_CollisionType( obj->sprite_array ) );
+				Collision_Add( this, obj, pLevel->pSprite_Manager->Get_Array_num( obj ), Get_CollisionType( obj->sprite_array ) );
 			}
 
 			return 1;
@@ -408,12 +456,12 @@ bool cMouseCursor :: Get_Collision_Level( GL_rect *mouse_rect )
 
 bool cMouseCursor :: Get_Collision_World( GL_rect *mouse_rect )
 {
-	ObjectList level_objects;
-	pActive_Overworld->object_manager->Get_Objects_sorted( level_objects, 1, 1 );
+	SpriteList level_objects;
+	pActive_Overworld->sprite_manager->Get_Objects_sorted( level_objects, 1, 1 );
 
 
 	// objects before player
-	for( ObjectList::reverse_iterator itr = level_objects.rbegin(), itr_end = level_objects.rend(); itr != itr_end; ++itr )
+	for( SpriteList::reverse_iterator itr = level_objects.rbegin(), itr_end = level_objects.rend(); itr != itr_end; ++itr )
 	{
 		cSprite *obj = (*itr);
 
@@ -431,7 +479,7 @@ bool cMouseCursor :: Get_Collision_World( GL_rect *mouse_rect )
 			}
 			else
 			{
-				Collision_Add( this, obj, pActive_Overworld->object_manager->Get_Array_num( obj ), Get_CollisionType( obj->sprite_array ) );
+				Collision_Add( this, obj, pActive_Overworld->sprite_manager->Get_Array_num( obj ), Get_CollisionType( obj->sprite_array ) );
 			}
 
 			return 1;
@@ -449,7 +497,7 @@ void cMouseCursor :: Update( void )
 		return;
 	}
 
-	// o.0
+	Update_Doubleclick();
 }
 
 void cMouseCursor :: Draw( void )
@@ -469,12 +517,6 @@ void cMouseCursor :: Draw( void )
 		if( !mover_mode )
 		{
 			Draw_object_rects();
-
-			// auto hide on selection if enabled
-			if( left && mouse_object->obj && pPreferences->editor_mouse_auto_hide )
-			{
-				visible = 0;
-			}
 		}
 	}
 }
@@ -496,7 +538,13 @@ void cMouseCursor :: Render( void )
 
 void cMouseCursor :: Update_Position( void )
 {
-	SDL_GetMouseState( &x, &y );
+	if( !mover_mode )
+	{
+		SDL_GetMouseState( &x, &y );
+		// scale to the virtual game size
+		x = static_cast<int>( static_cast<float>(x) * global_downscalex );
+		y = static_cast<int>( static_cast<float>(y) * global_downscaley );
+	}
 
 	if( editor_enabled )
 	{
@@ -510,7 +558,7 @@ void cMouseCursor :: Update_Position( void )
 	}
 	else
 	{
-		Set_Pos( (float)x, (float)y, 1 );
+		Set_Pos( static_cast<float>(x), static_cast<float>(y), 1 );
 	}
 }
 
@@ -531,6 +579,11 @@ void cMouseCursor :: Click( void )
 {
 	Editor_Collsion_Check();
 
+	if( mover_mode )
+	{
+		return;
+	}
+
 	// no colliding sprite
 	if( !mouse_object->obj )
 	{
@@ -549,7 +602,7 @@ void cMouseCursor :: Click( void )
 				}
 				else
 				{
-					col_obj = pObjManager->items[col->number];
+					col_obj = pLevel->pSprite_Manager->objects[col->number];
 				}
 			}
 			// World
@@ -563,7 +616,7 @@ void cMouseCursor :: Click( void )
 				// object manager object
 				else
 				{
-					col_obj = pActive_Overworld->object_manager->Get_Pointer( col->number );
+					col_obj = pActive_Overworld->sprite_manager->Get_Pointer( col->number );
 				}
 			}
 
@@ -646,7 +699,7 @@ void cMouseCursor :: Update_mouse_object( void )
 	if( left )
 	{
 		// set new position
-		mouse_object->obj->Set_Pos( (float)( (int)posx - mouse_object->mouse_w ), (float)( (int)posy - mouse_object->mouse_h ), 1 );
+		mouse_object->obj->Set_Pos( static_cast<float>( static_cast<int>(posx) - mouse_object->mouse_w ), static_cast<float>( static_cast<int>(posy) - mouse_object->mouse_h ), 1 );
 
 		// update object settings position
 		if( active_object && active_object == mouse_object->obj )
@@ -658,8 +711,8 @@ void cMouseCursor :: Update_mouse_object( void )
 	else
 	{
 		// update mouse object positioning mouse width and height
-		mouse_object->mouse_w = (int)posx - (int)mouse_object->obj->startposx;
-		mouse_object->mouse_h = (int)posy - (int)mouse_object->obj->startposy;
+		mouse_object->mouse_w = static_cast<int>(posx) - static_cast<int>(mouse_object->obj->startposx);
+		mouse_object->mouse_h = static_cast<int>(posy) - static_cast<int>(mouse_object->obj->startposy);
 	}
 }
 
@@ -676,9 +729,9 @@ void cMouseCursor :: Add_copy_object( cSprite *sprite )
 	}
 
 	// check if not already added
-	for( unsigned int i = 0; i < copy_objects.size(); i++ )
+	for( CopyObjectList::iterator itr = copy_objects.begin(), itr_end = copy_objects.end(); itr != itr_end; ++itr )
 	{
-		if( copy_objects[i]->obj == sprite )
+		if( (*itr)->obj == sprite )
 		{
 			return;
 		}
@@ -706,9 +759,11 @@ void cMouseCursor :: Add_copy_objects( SpriteList spritelist )
 	}
 
 	// inserts all objects
-	for( unsigned int i = 0; i < spritelist.size(); i++ )
+	for( SpriteList::iterator itr = spritelist.begin(), itr_end = spritelist.end(); itr != itr_end; ++itr )
 	{
-		Add_copy_object( spritelist[i] );
+		cSprite *obj = (*itr);
+
+		Add_copy_object( obj );
 	}
 }
 
@@ -719,12 +774,15 @@ bool cMouseCursor :: Remove_copy_object( cSprite *sprite )
 		return 0;
 	}
 
-	for( unsigned int i = 0; i < copy_objects.size(); i++ )
+	for( CopyObjectList::iterator itr = copy_objects.begin(), itr_end = copy_objects.end(); itr != itr_end; ++itr )
 	{
-		if( copy_objects[i]->obj == sprite )
+		cCopyObject *copy_obj = (*itr);
+
+		if( copy_obj->obj == sprite )
 		{
-			delete copy_objects[i];
-			copy_objects.erase( copy_objects.begin() + i );
+			copy_objects.erase( itr );
+			delete copy_obj;
+			
 			return 1;
 		}
 	}
@@ -734,9 +792,9 @@ bool cMouseCursor :: Remove_copy_object( cSprite *sprite )
 
 void cMouseCursor :: Clear_copy_objects( void )
 {
-	for( unsigned int i = 0; i < copy_objects.size(); i++ )
+	for( CopyObjectList::iterator itr = copy_objects.begin(), itr_end = copy_objects.end(); itr != itr_end; ++itr )
 	{
-		delete copy_objects[i];
+		delete *itr;
 	}
 
 	copy_objects.clear();
@@ -755,13 +813,15 @@ GL_Vector cMouseCursor :: Get_copy_object_base( float px, float py )
 	float base_x = copy_objects[0]->obj->posx;
 	float base_y = copy_objects[0]->obj->posy;
 
-	for( unsigned int i = 1; i < copy_objects.size(); i++ )
+	for( CopyObjectList::iterator itr = copy_objects.begin(), itr_end = copy_objects.end(); itr != itr_end; ++itr )
 	{
+		cCopyObject *copy_obj = (*itr);
+
 		// if more in top or left
-		if( copy_objects[i]->obj->posx < base_x || copy_objects[i]->obj->posy < base_y )
+		if( copy_obj->obj->posx < base_x || copy_obj->obj->posy < base_y )
 		{
-			base_x = copy_objects[i]->obj->posx;
-			base_y = copy_objects[i]->obj->posy;
+			base_x = copy_obj->obj->posx;
+			base_y = copy_obj->obj->posy;
 		}
 	}
 
@@ -780,13 +840,15 @@ void cMouseCursor :: Paste_copy_objects( float px, float py )
 
 	SpriteList new_objects;
 
-	for( unsigned int i = 0; i < copy_objects.size(); i++ )
+	for( CopyObjectList::iterator itr = copy_objects.begin(), itr_end = copy_objects.end(); itr != itr_end; ++itr )
 	{
+		cCopyObject *copy_obj = (*itr);
+
 		// set base vector
-		GL_Vector base_vec = Get_copy_object_base( copy_objects[i]->obj->posx, copy_objects[i]->obj->posy );
+		GL_Vector base_vec = Get_copy_object_base( copy_obj->obj->posx, copy_obj->obj->posy );
 
 		// copy
-		cSprite* new_object = static_cast<cSprite *>(Copy( static_cast<cMovingSprite *>(copy_objects[i]->obj), px + base_vec.x, py + base_vec.y ));
+		cSprite* new_object = Copy( copy_obj->obj, px + base_vec.x, py + base_vec.y );
 
 		if( new_object )
 		{
@@ -796,8 +858,8 @@ void cMouseCursor :: Paste_copy_objects( float px, float py )
 
 	if( copy_objects.size() )
 	{
-		mouse_object->mouse_h = (int)( copy_objects[0]->obj->col_rect.h / 2 );
-		mouse_object->mouse_w = (int)( copy_objects[0]->obj->col_rect.w / 2 );
+		mouse_object->mouse_h = static_cast<int>( copy_objects[0]->obj->col_rect.h / 2 );
+		mouse_object->mouse_w = static_cast<int>( copy_objects[0]->obj->col_rect.w / 2 );
 	}
 
 	// pasted objects are selected
@@ -813,14 +875,16 @@ bool cMouseCursor :: Add_selected_object( cSprite *sprite, bool from_user /* = 0
 	}
 
 	// check if not already added
-	for( unsigned int i = 0; i < selected_objects.size(); i++ )
+	for( SelectedObjectList::iterator itr = selected_objects.begin(), itr_end = selected_objects.end(); itr != itr_end; ++itr )
 	{
-		if( selected_objects[i]->obj == sprite )
+		cSelectedObject *sel_obj = (*itr);
+
+		if( sel_obj->obj == sprite )
 		{
 			// overwrite user if given
-			if( from_user && !selected_objects[i]->user )
+			if( from_user && !sel_obj->user )
 			{
-				selected_objects[i]->user = 1;
+				sel_obj->user = 1;
 				return 1;
 			}
 
@@ -845,9 +909,11 @@ void cMouseCursor :: Add_selected_objects( SpriteList spritelist, bool from_user
 	}
 
 	// inserts all objects
-	for( unsigned int i = 0; i < spritelist.size(); i++ )
+	for( SpriteList::iterator itr = spritelist.begin(), itr_end = spritelist.end(); itr != itr_end; ++itr )
 	{
-		Add_selected_object( spritelist[i], from_user );
+		cSprite *obj = (*itr);
+
+		Add_selected_object( obj, from_user );
 	}
 }
 
@@ -858,18 +924,21 @@ bool cMouseCursor :: Remove_selected_object( cSprite *sprite, bool no_user /* =
 		return 0;
 	}
 
-	for( unsigned int i = 0; i < selected_objects.size(); i++ )
+	for( SelectedObjectList::iterator itr = selected_objects.begin(), itr_end = selected_objects.end(); itr != itr_end; ++itr )
 	{
-		if( selected_objects[i]->obj == sprite )
+		cSelectedObject *sel_obj = (*itr);
+
+		if( sel_obj->obj == sprite )
 		{
 			// don't delete user added selected object
-			if( no_user && selected_objects[i]->user )
+			if( no_user && sel_obj->user )
 			{
 				return 0;
 			}
 
-			delete selected_objects[i];
-			selected_objects.erase( selected_objects.begin() + i );
+			selected_objects.erase( itr );
+			delete sel_obj;
+			
 			return 1;
 		}
 	}
@@ -881,9 +950,11 @@ SpriteList cMouseCursor :: Get_selected_objects( void )
 {
 	SpriteList spritelist;
 
-	for( unsigned int i = 0; i < selected_objects.size(); i++ )
+	for( SelectedObjectList::iterator itr = selected_objects.begin(), itr_end = selected_objects.end(); itr != itr_end; ++itr )
 	{
-		spritelist.push_back( selected_objects[i]->obj );
+		cSelectedObject *object = (*itr);
+
+		spritelist.push_back( object->obj );
 	}
 
 	return spritelist;
@@ -891,9 +962,9 @@ SpriteList cMouseCursor :: Get_selected_objects( void )
 
 void cMouseCursor :: Clear_selected_objects( void )
 {
-	for( unsigned int i = 0; i < selected_objects.size(); i++ )
+	for( SelectedObjectList::iterator itr = selected_objects.begin(), itr_end = selected_objects.end(); itr != itr_end; ++itr )
 	{
-		delete selected_objects[i];
+		delete *itr;
 	}
 
 	selected_objects.clear();
@@ -907,12 +978,14 @@ void cMouseCursor :: Update_selected_objects( void )
 	}
 
 	// update selected objects positioning mouse width and height
-	for( unsigned int i = 0; i < selected_objects.size(); i++ )
+	for( SelectedObjectList::iterator itr = selected_objects.begin(), itr_end = selected_objects.end(); itr != itr_end; ++itr )
 	{
+		cSelectedObject *sel_obj = (*itr);
+
 		if( !left || mode_selection )
 		{
-			selected_objects[i]->mouse_w = (int)posx - (int)selected_objects[i]->obj->startposx;
-			selected_objects[i]->mouse_h = (int)posy - (int)selected_objects[i]->obj->startposy;
+			sel_obj->mouse_w = static_cast<int>(posx) - static_cast<int>(sel_obj->obj->startposx);
+			sel_obj->mouse_h = static_cast<int>(posy) - static_cast<int>(sel_obj->obj->startposy);
 		}
 	}
 
@@ -922,16 +995,18 @@ void cMouseCursor :: Update_selected_objects( void )
 		return;
 	}
 
-	for( unsigned int i = 0; i < selected_objects.size(); i++ )
+	for( SelectedObjectList::iterator itr = selected_objects.begin(), itr_end = selected_objects.end(); itr != itr_end; ++itr )
 	{
+		cSelectedObject *sel_obj = (*itr);
+
 		// left mouse is pressed
 		if( left )
 		{
 			// set new position
-			selected_objects[i]->obj->Set_Pos( (float)( (int)posx - selected_objects[i]->mouse_w ), (float)( (int)posy - selected_objects[i]->mouse_h ), 1 );
+			sel_obj->obj->Set_Pos( static_cast<float>(static_cast<int>(posx) - sel_obj->mouse_w), static_cast<float>(static_cast<int>(posy) - sel_obj->mouse_h), 1 );
 
 			// update object settings position
-			if( active_object && active_object == selected_objects[i]->obj )
+			if( active_object && active_object == sel_obj->obj )
 			{
 				active_object->Editor_pos_update();
 			}
@@ -946,12 +1021,14 @@ bool cMouseCursor :: is_selected_object( cSprite *sprite, bool only_user /* = 0
 		return 0;
 	}
 
-	for( unsigned int i = 0; i < selected_objects.size(); i++ )
+	for( SelectedObjectList::iterator itr = selected_objects.begin(), itr_end = selected_objects.end(); itr != itr_end; ++itr )
 	{
-		if( selected_objects[i]->obj == sprite )
+		cSelectedObject *sel_obj = (*itr);
+
+		if( sel_obj->obj == sprite )
 		{
 			// if only user objects
-			if( only_user && !selected_objects[i]->user )
+			if( only_user && !sel_obj->user )
 			{
 				continue;
 			}
@@ -968,7 +1045,7 @@ void cMouseCursor :: Delete_selected_objects( void )
 {
 	for( int i = selected_objects.size() - 1; i >= 0; i-- )
 	{
-		Delete( static_cast<cMovingSprite *>( selected_objects[i]->obj ) );
+		Delete( selected_objects[i]->obj );
 	}
 
 	Clear_selected_objects();
@@ -1007,7 +1084,7 @@ cSprite *cMouseCursor :: Copy( cSprite *ncopy_object, float px, float py )
 		return NULL;
 	}
 
-	cSprite *new_sprite = Copy_Object( ncopy_object, px, py );
+	cSprite *new_sprite = ncopy_object->Copy();
 
 	if( !new_sprite )
 	{
@@ -1015,20 +1092,28 @@ cSprite *cMouseCursor :: Copy( cSprite *ncopy_object, float px, float py )
 		return NULL;
 	}
 
+	new_sprite->Set_Pos( px, py, 1 );
+
 	if( new_sprite->sprite_array == ARRAY_MASSIVE || new_sprite->sprite_array == ARRAY_PASSIVE || new_sprite->sprite_array == ARRAY_FRONT_PASSIVE || new_sprite->sprite_array == ARRAY_ACTIVE || new_sprite->sprite_array == ARRAY_ENEMY )
 	{
 		if( editor_level_enabled )
 		{
-			pObjManager->Add( new_sprite );
+			pLevel->pSprite_Manager->Add( new_sprite );
 		}
 		else if( editor_world_enabled )
 		{
-			pActive_Overworld->object_manager->Add( new_sprite );
+			pActive_Overworld->sprite_manager->Add( new_sprite );
+		}
+		else
+		{
+			printf( "no editor enabled for copy object: %s\n", new_sprite->name.c_str() );
+			delete new_sprite;
+			return NULL;
 		}
 	}
 	else
 	{
-		printf( "copy object unknown Array : %d\n", new_sprite->sprite_array );
+		printf( "copy object unknown array : %d\n", new_sprite->sprite_array );
 		delete new_sprite;
 		return NULL;
 	}
@@ -1065,7 +1150,7 @@ void cMouseCursor :: Delete( cSprite *sprite )
 		// delete object
 		if( editor_level_enabled )
 		{
-			pObjManager->Delete( sprite );
+			pLevel->pSprite_Manager->Delete( sprite );
 		}
 		else if( editor_world_enabled )
 		{
@@ -1075,12 +1160,11 @@ void cMouseCursor :: Delete( cSprite *sprite )
 				cLayer_Line *line = static_cast<cLayer_Line *>(static_cast<cLayer_Line_Point *>(sprite)->line);
 
 				// delete line
-				pActive_Overworld->pLayer->lines.erase( pActive_Overworld->pLayer->lines.begin() + pActive_Overworld->pLayer->Get_Line_num( line ) );
-				delete line;
+				pActive_Overworld->pLayer->Delete( line );
 			}
 			else
 			{
-				pActive_Overworld->object_manager->Delete( sprite );
+				pActive_Overworld->sprite_manager->Delete( sprite );
 			}
 		}
 
@@ -1125,9 +1209,9 @@ void cMouseCursor :: Draw_object_rects( void )
 	{
 		Color obj_color = white;
 
-		for( unsigned int i = 0; i < selected_objects.size(); i++ )
+		for( SelectedObjectList::iterator itr = selected_objects.begin(), itr_end = selected_objects.end(); itr != itr_end; ++itr )
 		{
-			cSprite *object = selected_objects[i]->obj;
+			cSprite *object = (*itr)->obj;
 
 			// check if visible on screen
 			if( !object->is_Visible_onScreen() )
@@ -1220,21 +1304,19 @@ void cMouseCursor :: Update_selection( void )
 		Clear_selected_objects();
 	}
 
-	cObjectManager *obj_manager = NULL;
+	cSprite_Manager *sprite_manager = NULL;
 
 	if( editor_level_enabled )
 	{
-		obj_manager = pObjManager;
+		sprite_manager = pLevel->pSprite_Manager;
 	}
 	else
 	{
-		obj_manager = pActive_Overworld->object_manager;
-
-
+		sprite_manager = pActive_Overworld->sprite_manager;
 	}
 
 	// add selected objects
-	for( ObjectList::iterator itr = obj_manager->items.begin(), itr_end = obj_manager->items.end(); itr != itr_end; ++itr )
+	for( SpriteList::iterator itr = sprite_manager->objects.begin(), itr_end = sprite_manager->objects.end(); itr != itr_end; ++itr )
 	{
 		cSprite *object = (*itr);
 
@@ -1294,9 +1376,10 @@ void cMouseCursor :: Mover_Update( Sint16 move_x, Sint16 move_y )
 		return;
 	}
 
+	// mouse moves the camera
 	pCamera->Move( move_x, move_y );
-
-	SDL_WarpMouse( x - move_x, y - move_y );
+	// keep mouse at it's position
+	SDL_WarpMouse( static_cast<Uint16>(x * global_upscalex), static_cast<Uint16>(y * global_upscaley) );
 
 	SDL_Event inEvent;
 	SDL_PeepEvents( &inEvent, 1, SDL_GETEVENT, SDL_MOUSEMOTIONMASK );
@@ -1355,7 +1438,7 @@ void cMouseCursor :: Editor_Update( void )
 			// object manager object
 			else
 			{
-				obj = pObjManager->Get_Pointer( col->number );
+				obj = pLevel->pSprite_Manager->Get_Pointer( col->number );
 			}
 		}
 		// World
@@ -1369,7 +1452,7 @@ void cMouseCursor :: Editor_Update( void )
 			// object manager object
 			else
 			{
-				obj = pActive_Overworld->object_manager->Get_Pointer( col->number );
+				obj = pActive_Overworld->sprite_manager->Get_Pointer( col->number );
 			}
 		}
 
@@ -1431,7 +1514,7 @@ void cMouseCursor :: Editor_Update( void )
 		if( col->type > CO_NOTHING && col->type <= CO_PASSIVE && mouse_object->obj )
 		{
 			// position text
-			string info = "X : " + int_to_string( (int)mouse_object->obj->posx ) + "  Y : " + int_to_string( (int)mouse_object->obj->posy );
+			string info = "X : " + int_to_string( static_cast<int>(mouse_object->obj->posx) ) + "  Y : " + int_to_string( static_cast<int>(mouse_object->obj->posy) );
 
 			// if in debug mode draw editor Z position
 			if( Game_debug )
@@ -1449,7 +1532,7 @@ void cMouseCursor :: Editor_Update( void )
 
 			// create request
 			cSurfaceRequest *request = new cSurfaceRequest();
-			position_info->Blit( (float)( x + 20 ), (float)( y + 35 ), 0.52f, request );
+			position_info->Blit( static_cast<float>( x + 20 ), static_cast<float>( y + 35 ), 0.52f, request );
 			request->delete_texture = 1;
 
 			// shadow
