@@ -3,24 +3,26 @@
                              -------------------
     copyright            :	(C) 2003 - 2007 by Florian Richter
  ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/ 
+/*
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
  
 #include "../objects/box.h"
 #include "../audio/audio.h"
 #include "../core/camera.h"
 #include "../core/framerate.h"
-#include "../core/obj_manager.h"
+#include "../level/level.h"
 #include "../core/game_core.h"
 #include "../video/animation.h"
 #include "../player/player.h"
 #include "../video/gl_surface.h"
+#include "../user/savegame.h"
 
 /* *** *** *** *** *** *** *** *** cBaseBox *** *** *** *** *** *** *** *** *** */
 
@@ -30,6 +32,7 @@ cBaseBox :: cBaseBox( float x /* = 0 */, float y /* = 0 */ )
 	type = TYPE_ACTIVESPRITE;
 	sprite_array = ARRAY_ACTIVE;
 	massivetype = MASS_MASSIVE;
+	can_be_ground = 1;
 
 	counter = 0;
 	box_type = TYPE_UNDEFINED;
@@ -55,8 +58,8 @@ cBaseBox :: ~cBaseBox( void )
 void cBaseBox :: Create_from_Stream( XMLAttributes &attributes )
 {
 	// position
-	Set_Pos( (float)attributes.getValueAsInteger( "posx" ), (float)attributes.getValueAsInteger( "posy" ), 1 );
-	if( box_type != TYPE_SPINBOX )
+	Set_Pos( static_cast<float>(attributes.getValueAsInteger( "posx" )), static_cast<float>(attributes.getValueAsInteger( "posy" )), 1 );
+	if( box_type != TYPE_SPINBOX && box_type != TYPE_TEXT_BOX )
 	{
 		// animation
 		Set_Animation( attributes.getValueAsString( "animation", anim_type ).c_str() );
@@ -70,16 +73,19 @@ void cBaseBox :: Create_from_Stream( XMLAttributes &attributes )
 void cBaseBox :: Save_to_Stream( ofstream &file )
 {
 	// position
-	file << "\t\t<Property name=\"posx\" value=\"" << (int)startposx << "\" />" << std::endl;
-	file << "\t\t<Property name=\"posy\" value=\"" << (int)startposy << "\" />" << std::endl;
+	file << "\t\t<Property name=\"posx\" value=\"" << static_cast<int>(startposx) << "\" />" << std::endl;
+	file << "\t\t<Property name=\"posy\" value=\"" << static_cast<int>(startposy) << "\" />" << std::endl;
+	// type
 	if( box_type == TYPE_SPINBOX )
 	{
-		// type
 		file << "\t\t<Property name=\"type\" value=\"spin\" />" << std::endl;
 	}
+	else if( box_type == TYPE_TEXT_BOX )
+	{
+		file << "\t\t<Property name=\"type\" value=\"text\" />" << std::endl;
+	}
 	else
 	{
-		// type
 		file << "\t\t<Property name=\"type\" value=\"bonus\" />" << std::endl;
 		// animation type
 		file << "\t\t<Property name=\"animation\" value=\"" << anim_type << "\" />" << std::endl;
@@ -92,6 +98,34 @@ void cBaseBox :: Save_to_Stream( ofstream &file )
 	file << "\t\t<Property name=\"useable_count\" value=\"" << start_useable_count << "\" />" << std::endl;
 }
 
+void cBaseBox :: Load_from_Savegame( cSave_Level_Object *save_object )
+{
+	// useable count
+	int save_useable_count = string_to_int( save_object->Get_Value( "useable_count" ) );
+	Set_UseableCount( save_useable_count );
+}
+
+cSave_Level_Object *cBaseBox :: Save_to_Savegame( void )
+{
+	// only save if needed
+	if( useable_count == start_useable_count )
+	{
+		return NULL;
+	}
+
+	cSave_Level_Object *save_object = new cSave_Level_Object();
+
+	// default values
+	save_object->type = type;
+	save_object->properties.push_back( cSave_Level_Object_Property( "posx", int_to_string( static_cast<int>(startposx) ) ) );
+	save_object->properties.push_back( cSave_Level_Object_Property( "posy", int_to_string( static_cast<int>(startposy) ) ) );
+
+	// Useable Count
+	save_object->properties.push_back( cSave_Level_Object_Property( "useable_count", int_to_string( useable_count ) ) );
+
+	return save_object;
+}
+
 void cBaseBox :: Set_Animation( string new_anim_type )
 {
 	// already set
@@ -140,7 +174,9 @@ void cBaseBox :: Set_Animation( string new_anim_type )
 	}
 	else if( anim_type.compare( "Spin" ) == 0 )
 	{
+		// disabled image
 		images.push_back( pVideo->Get_Surface( "game/box/yellow/spin/disabled.png" ) );
+		// animation images
 		images.push_back( pVideo->Get_Surface( "game/box/yellow/default.png" ) );
 		images.push_back( pVideo->Get_Surface( "game/box/yellow/spin/1.png" ) );
 		images.push_back( pVideo->Get_Surface( "game/box/yellow/spin/2.png" ) );
@@ -157,8 +193,8 @@ void cBaseBox :: Set_Animation( string new_anim_type )
 		Set_Animation( "Bonus" );
 	}
 
-	counter = (float)anim_counter_min;
-	Set_Image( (int)counter, 1, 0 );
+	counter = static_cast<float>(anim_counter_min);
+	Set_Image( static_cast<int>(counter), 1, 0 );
 }
 
 void cBaseBox :: Set_UseableCount( int count, bool new_startcount /* = 0 */ )
@@ -188,7 +224,7 @@ void cBaseBox :: Set_Invisible( unsigned int type )
 	// was invisible
 	if( box_invisible == 1 )
 	{
-		Set_Image( (int)counter, 1, 0 );
+		Set_Image( static_cast<int>(counter), 1, 0 );
 	}
 	// was ghost
 	else if( box_invisible == 2 )
@@ -240,11 +276,12 @@ void cBaseBox :: Activate_collision( ObjectDirection cdirection )
 	// if invisible go visible
 	if( box_invisible )
 	{
-		Set_Image( (int)counter, 0, 0 );
+		Set_Image( static_cast<int>(counter), 0, 0 );
 	}
 
 	// set collision direction
 	move_col_dir = cdirection;
+	Update_valid_update();
 
 	Check_Collision( Get_OppositeDirection( move_col_dir ) );
 	Activate();
@@ -291,14 +328,18 @@ void cBaseBox :: Update_collision( void )
 	{
 		printf( "Warning : wrong box collision direction %d\n", move_col_dir );
 		move_col_dir = DIR_UNDEFINED;
+		Update_valid_update();
 		return;
 	}
 
+	// speed mod
+	float mod = pFramerate->speedfactor * 0.05f;
+	// counter
+	move_counter += pFramerate->speedfactor * 0.2f;
+
 	// move into the given direction
 	if( !move_back )
 	{
-		// speed mod
-		float mod = pFramerate->speedfactor * 0.05f;
 		// final move value
 		float move_x = ( xcheck * pFramerate->speedfactor ) - ( ( rect.w * x_scale ) * mod );
 		float move_y = ( ycheck * pFramerate->speedfactor ) - ( ( rect.h * y_scale ) * mod );
@@ -310,8 +351,7 @@ void cBaseBox :: Update_collision( void )
 		Move( move_x, move_y, 1 );
 		// scale
 		Add_Scale( mod );
-		// counter
-		move_counter += pFramerate->speedfactor * 0.2f;
+
 		// Particles
 		Generate_active_Particles();
 
@@ -325,8 +365,6 @@ void cBaseBox :: Update_collision( void )
 	// move back to the original position
 	else
 	{
-		// speed mod
-		float mod = pFramerate->speedfactor * 0.05f;
 		// final move value
 		float move_x = ( -xcheck * pFramerate->speedfactor ) + ( ( rect.w * x_scale ) * mod );
 		float move_y = ( -ycheck * pFramerate->speedfactor ) + ( ( rect.h * y_scale ) * mod );
@@ -338,13 +376,12 @@ void cBaseBox :: Update_collision( void )
 		Move( move_x, move_y, 1 );
 		// scale
 		Add_Scale( -mod );
-		// counter
-		move_counter += pFramerate->speedfactor * 0.2f;
 
 		// check if reached original position
 		if(	move_counter > 1 )
 		{
 			move_col_dir = DIR_UNDEFINED;
+			Update_valid_update();
 			move_back = 0;
 			move_counter = 0;
 
@@ -361,54 +398,45 @@ void cBaseBox :: Update_collision( void )
 void cBaseBox :: Check_Collision( ObjectDirection cdirection )
 {
 	// additional direction based check position
-	float check_x = posx;
-	float check_y = posy;
-	// check rect size
-	float check_width = col_rect.w;
-	float check_height = col_rect.h;
+	float check_x = 0;
+	float check_y = 0;
 
 
-	// set the collision rect based on the collision direction
+	// set the collision size based on the collision direction
 	if( cdirection == DIR_BOTTOM )
 	{
 		check_y -= 10;
-		check_width *= 0.9f;
 	}
 	else if( cdirection == DIR_TOP )
 	{
 		check_y += 10;
-		check_width *= 0.9f;
 	}
 	else if( cdirection == DIR_LEFT ) 
 	{
-		check_x += 10;
-		check_height *= 0.8f;
+		check_x += 5;
 	}
 	else if( cdirection == DIR_RIGHT )
 	{
-		check_x -= 10;
-		check_height *= 0.8f;
+		check_x -= 5;
 	}
 
 	// collision count
-	unsigned int col_count = ColCheck( check_x, check_y, check_width, check_height ).size();
+	unsigned int col_count = Collision_Check_Relative( check_x, check_y, col_rect.w - ( check_x * 0.5f ), col_rect.h - ( check_y * 0.5f ) ).size();
 
 	// handle collisions
 	for( unsigned int i = 0; i < col_count; i++ )
 	{
 		cObjectCollision *col = Collision_Get_last();
-		cSprite *col_obj = pObjManager->Get_Pointer( col->number );
+		cSprite *col_obj = pLevel->pSprite_Manager->Get_Pointer( col->number );
 
-		 // Enemy collision
+		// send box collision
+		col_obj->Handle_Collision_Box( Get_OppositeDirection( col->direction ), &col_rect );
+
+		// Enemy collision
 		if( col->type == CO_ENEMY )
 		{
 			Col_Enemy( col_obj );
 		}
-		// Active
-		else if( col->type == CO_ACTIVE ) 
-		{
-			Col_Active( col_obj, cdirection );
-		}
 
 		Collision_Delete( col );
 	}
@@ -424,16 +452,6 @@ void cBaseBox :: Col_Enemy( cSprite *obj )
 	}
 }
 
-void cBaseBox :: Col_Active( cSprite *obj, ObjectDirection dir )
-{
-	// Mushroom collision
-	if( obj->type == TYPE_MUSHROOM_DEFAULT || obj->type == TYPE_MUSHROOM_LIVE_1 || obj->type == TYPE_MUSHROOM_POISON
-		|| obj->type == TYPE_MUSHROOM_BLUE || obj->type == TYPE_MUSHROOM_GHOST || obj->type == TYPE_FGOLDPIECE )
-	{
-		static_cast<cMovingSprite *>(obj)->Handle_Collision_Box( dir, &col_rect );
-	}
-}
-
 void cBaseBox :: Activate( void )
 {
 	// virtual
@@ -449,15 +467,15 @@ void cBaseBox :: Update( void )
 		{
 			counter += pFramerate->speedfactor * 0.4f;
 
-			if( counter >= anim_counter_max )
+			if( counter >= anim_counter_max + 1 )
 			{
-				counter = (float)anim_counter_min;
+				counter = static_cast<float>(anim_counter_min);
 			}
 
 			// Set_Image overwrites col_pos
 			GL_point col_pos_temp = col_pos;
 
-			Set_Image( (int)counter, 0, 0 );
+			Set_Image( static_cast<int>(counter) );
 
 			// save col_pos
 			col_pos = col_pos_temp;
@@ -491,12 +509,12 @@ void cBaseBox :: Draw( cSurfaceRequest *request /* = NULL */ )
 			// default invisible
 			if( box_invisible == 1 )
 			{
-				color = Color( (Uint8)240, 0, 30, 128 );
+				color = Color( static_cast<Uint8>(240), 0, 30, 128 );
 			}
 			// ghost
 			else if( box_invisible == 2 )
 			{
-				color = Color( (Uint8)20, 20, 150, 128 );
+				color = Color( static_cast<Uint8>(20), 20, 150, 128 );
 			}
 
 			pVideo->Draw_Rect( startposx - pCamera->x, startposy - pCamera->y, col_rect.w, col_rect.h, posz, &color );
@@ -511,7 +529,7 @@ void cBaseBox :: Draw( cSurfaceRequest *request /* = NULL */ )
 		if( item_image )
 		{
 			// auto position
-			item_image->Blit( startposx - ( ( item_image->w - rect.w ) / 2 ) - pCamera->x, startposy - ( ( item_image->h - rect.h ) / 2 ) - pCamera->y, posz + 0.000001f );
+			item_image->Blit( startposx - ( ( item_image->w - rect.w ) / 2 ) - pCamera->x, startposy - ( ( item_image->h - rect.h ) / 2 ) - pCamera->y, posz + 0.000003f );
 		}
 	}
 }
@@ -519,7 +537,7 @@ void cBaseBox :: Draw( cSurfaceRequest *request /* = NULL */ )
 void cBaseBox :: Generate_active_Particles( void )
 {
 	// no default/unimportant boxes
-	if( type == TYPE_SPINBOX || box_type == TYPE_GOLDPIECE )
+	if( type == TYPE_SPINBOX || type == TYPE_TEXT_BOX || box_type == TYPE_GOLDPIECE )
 	{
 		return;
 	}
@@ -528,7 +546,7 @@ void cBaseBox :: Generate_active_Particles( void )
 
 	while( particle_counter_active > 0 )
 	{
-		cParticleAnimation *anim = new cParticleAnimation( posx + ( rand() % (int)rect.w ), posy );
+		cParticleAnimation *anim = new cParticleAnimation( posx + ( rand() % static_cast<int>(rect.w) ), posy );
 		anim->Set_DirectionRange( 180, 180 );
 		anim->Set_Speed( 3.1f, 0.5f );
 		anim->Set_Scale( 0.7f, 0.1f );
@@ -537,31 +555,31 @@ void cBaseBox :: Generate_active_Particles( void )
 
 		if( box_type == TYPE_MUSHROOM_DEFAULT )
 		{
-			color = Color( (Uint8)( 100 + ( rand() % 100 ) ), 20 + ( rand() % 50 ), 10 + ( rand() % 30 ) );
+			color = Color( static_cast<Uint8>( 100 + ( rand() % 100 ) ), 20 + ( rand() % 50 ), 10 + ( rand() % 30 ) );
 		}
 		else if( box_type == TYPE_FIREPLANT )
 		{
-			color = Color( (Uint8)( 110 + ( rand() % 150 ) ), rand() % 50, rand() % 30 );
+			color = Color( static_cast<Uint8>( 110 + ( rand() % 150 ) ), rand() % 50, rand() % 30 );
 		}
 		else if( box_type == TYPE_MUSHROOM_BLUE )
 		{
-			color = Color( (Uint8)( rand() % 30 ), ( rand() % 30 ), 150 + ( rand() % 50 ) );
+			color = Color( static_cast<Uint8>( rand() % 30 ), ( rand() % 30 ), 150 + ( rand() % 50 ) );
 		}
 		else if( box_type == TYPE_MUSHROOM_GHOST )
 		{
-			color = Color( (Uint8)( 100 + ( rand() % 50 ) ), 100 + ( rand() % 50 ), 100 + ( rand() % 50 ) );
+			color = Color( static_cast<Uint8>( 100 + ( rand() % 50 ) ), 100 + ( rand() % 50 ), 100 + ( rand() % 50 ) );
 		}
 		else if( box_type == TYPE_MUSHROOM_LIVE_1 )
 		{
-			color = Color( (Uint8)( rand() % 30 ), 100 + ( rand() % 150 ), rand() % 30 );
+			color = Color( static_cast<Uint8>( rand() % 30 ), 100 + ( rand() % 150 ), rand() % 30 );
 		}
 		else if( box_type == TYPE_JSTAR )
 		{
-			color = Color( (Uint8)( 110 + ( rand() % 150 ) ), 80 + ( rand() % 50 ), rand() % 20 );
+			color = Color( static_cast<Uint8>( 110 + ( rand() % 150 ) ), 80 + ( rand() % 50 ), rand() % 20 );
 		}
 		else if( box_type == TYPE_MUSHROOM_POISON )
 		{
-			color = Color( (Uint8)( 50 + rand() % 50 ), 100 + ( rand() % 150 ), rand() % 10 );
+			color = Color( static_cast<Uint8>( 50 + rand() % 50 ), 100 + ( rand() % 150 ), rand() % 10 );
 		}
 
 		anim->Set_Time_to_Live( 0.3f );
@@ -574,6 +592,16 @@ void cBaseBox :: Generate_active_Particles( void )
 	}
 }
 
+bool cBaseBox :: is_Update_valid( void )
+{
+	if( !useable_count && move_col_dir == DIR_UNDEFINED )
+	{
+		return 0;
+	}
+
+	return 1;
+}
+
 bool cBaseBox :: is_Draw_valid( void )
 {
 	// if editor not enabled
@@ -638,14 +666,14 @@ unsigned int cBaseBox :: Validate_Collision( cSprite *obj )
 	return 0;
 }
 
-void cBaseBox :: Handle_Collision_Player( ObjectDirection cdirection )
+void cBaseBox :: Handle_Collision_Player( cObjectCollision *collision )
 {
 	// if player jumps from below or fly's against it
-	if( cdirection == DIR_BOTTOM && pPlayer->state != STA_FLY )
+	if( collision->direction == DIR_BOTTOM && pPlayer->state != STA_FLY )
 	{
 		if( useable_count != 0 )
 		{
-			Activate_collision( Get_OppositeDirection( cdirection ) );
+			Activate_collision( Get_OppositeDirection( collision->direction ) );
 		}
 		else
 		{
@@ -680,18 +708,21 @@ void cBaseBox :: Editor_Activate( void )
 {
 	WindowManager &wmgr = WindowManager::getSingleton();
 
-	// useable count
-	Editbox *editbox = static_cast<Editbox *>(wmgr.createWindow( "TaharezLook/Editbox", "editor_basebox_useable_count" ));
-	editbox->setTooltipText( "Useable Count" );
-	Editor_Add( editbox, 80 );
+	if( type != TYPE_TEXT_BOX )
+	{
+		// useable count
+		Editbox *editbox = static_cast<Editbox *>(wmgr.createWindow( "TaharezLook/Editbox", "editor_basebox_useable_count" ));
+		editbox->setTooltipText( "Useable Count" );
+		Editor_Add( editbox, 80 );
 
-	editbox->setText( int_to_string( start_useable_count ) );
-	editbox->subscribeEvent( Editbox::EventKeyUp, Event::Subscriber( &cBaseBox::Editor_Useable_Count_Key, this ) );
+		editbox->setText( int_to_string( start_useable_count ) );
+		editbox->subscribeEvent( Editbox::EventKeyUp, Event::Subscriber( &cBaseBox::Editor_Useable_Count_Key, this ) );
+	}
 
 	// Invisible
 	Combobox *combobox = static_cast<Combobox *>(wmgr.createWindow( "TaharezLook/Combobox", "editor_basebox_invisible" ));
 	combobox->getEditbox()->setTooltipText( "Invisible" );
-	Editor_Add( combobox, 120, 80 );
+	Editor_Add( combobox, 120, 100 );
 
 	combobox->addItem( new ListboxTextItem( "Enabled" ) );
 	combobox->addItem( new ListboxTextItem( "Disabled" ) );
@@ -711,6 +742,12 @@ void cBaseBox :: Editor_Activate( void )
 	}
 
 	combobox->subscribeEvent( Combobox::EventListSelectionAccepted, Event::Subscriber( &cBaseBox::Editor_Invisible_Select, this ) );
+
+	if( type == TYPE_SPINBOX )
+	{
+		// set position
+		Editor_pos_update();
+	}
 }
 
 bool cBaseBox :: Editor_Useable_Count_Key( const EventArgs &event )
@@ -750,10 +787,18 @@ void cBaseBox :: Create_Name( void )
 	{
 		name = "Box Empty";
 	}
+	else if( box_type == TYPE_POWERUP )
+	{
+		name = "Box Random";
+	}
 	else if( box_type == TYPE_SPINBOX )
 	{
 		name = "Spinbox";
 	}
+	else if( box_type == TYPE_TEXT_BOX )
+	{
+		name = "Textbox";
+	}
 	else if( box_type == TYPE_MUSHROOM_DEFAULT )
 	{
 		name = "Box Mushroom";
