@@ -3,19 +3,20 @@
                              -------------------
     copyright            :	(C) 2003 - 2007 Florian Richter
  ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "../objects/sprite.h"
 #include "../objects/movingsprite.h"
 #include "../core/game_core.h"
-#include "../core/obj_manager.h"
+#include "../level/level.h"
 #include "../core/camera.h"
 #include "../core/framerate.h"
 #include "../player/player.h"
@@ -107,6 +108,7 @@ bool cCollidingSprite :: Collision_Add( cObjectCollision *collision, bool add_if
 			// already in list
 			if( col->number == collision->number )
 			{
+				delete collision;
 				return 0;
 			}
 		}
@@ -188,7 +190,7 @@ cObjectCollision *cCollidingSprite :: Collision_Get_last( bool only_blocking /*
 			// get object pointer
 			cObjectCollision *col = (*itr);
 
-			cSprite *col_obj = pObjManager->Get_Pointer( col->number );
+			cSprite *col_obj = pLevel->pSprite_Manager->Get_Pointer( col->number );
 
 			// ignore passive
 			if( col_obj->massivetype == MASS_PASSIVE )
@@ -232,7 +234,7 @@ void cCollidingSprite :: Handle_Collision( cObjectCollision *collision )
 	// player
 	if( collision->type == CO_PLAYER )
 	{
-		Handle_Collision_Player( collision->direction );
+		Handle_Collision_Player( collision );
 	}
 	// enemy
 	else if( collision->type == CO_ENEMY )
@@ -246,7 +248,7 @@ void cCollidingSprite :: Handle_Collision( cObjectCollision *collision )
 	}
 }
 
-void cCollidingSprite :: Handle_Collision_Player( ObjectDirection direction )
+void cCollidingSprite :: Handle_Collision_Player( cObjectCollision *collision )
 {
 	// virtual
 }
@@ -325,7 +327,7 @@ void cSprite :: Init( void )
 	shadow_color = black;
 	no_camera = 0;
 
-	color = (Uint8)255;
+	color = static_cast<Uint8>(255);
 
 	combine_type = 0;
 	combine_col[0] = 0;
@@ -341,6 +343,7 @@ void cSprite :: Init( void )
 	visible = 1;
 	spawned = 0;
 	player_range = 1000;
+	can_be_ground = 0;
 
 	// rotation
 	start_rotx = 0;
@@ -356,6 +359,7 @@ void cSprite :: Init( void )
 	scaley = 1;
 
 	valid_draw = 1;
+	valid_update = 1;
 }
 
 cSprite *cSprite :: Copy( void )
@@ -365,6 +369,7 @@ cSprite *cSprite :: Copy( void )
 	basic_sprite->type = type;
 	basic_sprite->sprite_array = sprite_array;
 	basic_sprite->Set_Massivetype( massivetype );
+	basic_sprite->can_be_ground = can_be_ground;
 	basic_sprite->Set_Ignore_Camera( no_camera );
 	basic_sprite->Set_Shadow_Pos( shadow_pos );
 	basic_sprite->Set_Shadow_Color( shadow_color );
@@ -375,7 +380,7 @@ cSprite *cSprite :: Copy( void )
 void cSprite :: Create_from_Stream( XMLAttributes &attributes )
 {
 	// position
-	Set_Pos( (float)attributes.getValueAsInteger( "posx" ), (float)attributes.getValueAsInteger( "posy" ), 1 );
+	Set_Pos( static_cast<float>(attributes.getValueAsInteger( "posx" )), static_cast<float>(attributes.getValueAsInteger( "posy" )), 1 );
 	// image
 	Set_Image( pVideo->Get_Surface( attributes.getValueAsString( "image" ).c_str() ), 1 ) ;
 	// type
@@ -388,8 +393,8 @@ void cSprite :: Save_to_Stream( ofstream &file )
 	file << "\t<sprite>" << std::endl;
 
 	// position
-	file << "\t\t<Property name=\"posx\" value=\"" << (int)startposx << "\" />" << std::endl;
-	file << "\t\t<Property name=\"posy\" value=\"" << (int)startposy << "\" />" << std::endl;
+	file << "\t\t<Property name=\"posx\" value=\"" << static_cast<int>(startposx) << "\" />" << std::endl;
+	file << "\t\t<Property name=\"posy\" value=\"" << static_cast<int>(startposy) << "\" />" << std::endl;
 	// image
 	string img_filename;
 
@@ -403,13 +408,13 @@ void cSprite :: Save_to_Stream( ofstream &file )
 	}
 	else
 	{
-		img_filename = "No Image from type : " + int_to_string( type );
+		printf( "Warning: cSprite::Save_to_Stream no image from type %d\n", type );
 	}
 
 	// remove pixmaps directory from string
-	if( img_filename.find( PIXMAPS_DIR ) == 0 )
+	if( img_filename.find( DATA_DIR "/" GAME_PIXMAPS_DIR "/" ) == 0 )
 	{
-		img_filename.erase( 0, strlen( PIXMAPS_DIR ) + 1 );
+		img_filename.erase( 0, strlen( DATA_DIR "/" GAME_PIXMAPS_DIR "/" ) );
 	}
 	file << "\t\t<Property name=\"image\" value=\"" << img_filename << "\" />" << std::endl;
 	// type
@@ -419,6 +424,16 @@ void cSprite :: Save_to_Stream( ofstream &file )
 	file << "\t</sprite>" << std::endl;
 }
 
+void cSprite :: Load_from_Savegame( cSave_Level_Object *save_object )
+{
+	// virtual
+}
+
+cSave_Level_Object *cSprite :: Save_to_Savegame( void )
+{
+	return NULL;
+}
+
 void cSprite :: Set_Image( GL_Surface *new_image, bool new_start_image /* = 0 */, bool del_img /* = 0 */ )
 {
 	if( delete_image )
@@ -444,11 +459,11 @@ void cSprite :: Set_Image( GL_Surface *new_image, bool new_start_image /* = 0 */
 	{
 		// collision data
 		col_pos = image->col_pos;
-		col_rect.w = image->col_w;
-		col_rect.h = image->col_h;
+		col_rect.w = image->col_w * scalex;
+		col_rect.h = image->col_h * scaley;
 		// image data
-		rect.w = image->w;
-		rect.h = image->h;
+		rect.w = image->w * scalex;
+		rect.h = image->h * scaley;
 
 		delete_image = del_img;
 
@@ -490,7 +505,8 @@ void cSprite :: Set_Image( GL_Surface *new_image, bool new_start_image /* = 0 */
 		}
 	}
 	
-	Update_valid_draw();
+	// because col_pos could have changed
+	Update_Position_Rect();
 }
 
 void cSprite :: Set_Sprite_Type( SpriteType ntype )
@@ -502,26 +518,31 @@ void cSprite :: Set_Sprite_Type( SpriteType ntype )
 	{
 		sprite_array = ARRAY_MASSIVE;
 		Set_Massivetype( MASS_MASSIVE );
+		can_be_ground = 1;
 	}
 	else if( ntype == TYPE_PASSIVE )
 	{
 		sprite_array = ARRAY_PASSIVE;
 		Set_Massivetype( MASS_PASSIVE );
+		can_be_ground = 0;
 	}
 	else if( ntype == TYPE_FRONT_PASSIVE )
 	{
 		sprite_array = ARRAY_FRONT_PASSIVE;
 		Set_Massivetype( MASS_PASSIVE );
+		can_be_ground = 0;
 	}
 	else if( ntype == TYPE_HALFMASSIVE )
 	{
 		sprite_array = ARRAY_ACTIVE;
 		Set_Massivetype( MASS_HALFMASSIVE );
+		can_be_ground = 1;
 	}
 	else if( ntype == TYPE_CLIMBABLE )
 	{
 		sprite_array = ARRAY_ACTIVE;
 		Set_Massivetype( MASS_CLIMBABLE );
+		can_be_ground = 0;
 	}
 }
 
@@ -607,7 +628,7 @@ void cSprite :: Set_PosY( float y, bool new_startpos /* = 0 */ )
 
 	if( new_startpos )
 	{
-		startposx = y;
+		startposy = y;
 	}
 
 	Update_Position_Rect();
@@ -615,9 +636,16 @@ void cSprite :: Set_PosY( float y, bool new_startpos /* = 0 */ )
 
 void cSprite :: Set_Visible( bool enabled )
 {
+	// already set
+	if( visible == enabled )
+	{
+		return;
+	}
+
 	visible = enabled;
 
 	Update_valid_draw();
+	Update_valid_update();
 }
 
 void cSprite :: Set_Shadow( Color shadow, float pos )
@@ -744,20 +772,70 @@ void cSprite :: Add_Rotation( float x, float y, float z )
 	Set_RotationZ( rotz + z );
 }
 
-void cSprite :: Set_ScaleX( float scale )
+void cSprite :: Set_ScaleX( float scale, bool new_startscale /* = 0 */ )
 {
+	// invalid value
+	if( scale == 0 )
+	{
+		return;
+	}
+
+	// undo previous scale from rect
+	if( scalex != 1 )
+	{
+		col_rect.w /= scalex;
+		rect.w /= scalex;
+	}
+
 	scalex = scale;
+
+	// set new scale to rect
+	if( scalex != 1 )
+	{
+		col_rect.w *= scalex;
+		rect.w *= scalex;
+	}
+
+	if( new_startscale )
+	{
+		start_scalex = scalex;
+	}
 }
 
-void cSprite :: Set_ScaleY( float scale )
+void cSprite :: Set_ScaleY( float scale, bool new_startscale /* = 0 */ )
 {
+	// invalid value
+	if( scale == 0 )
+	{
+		return;
+	}
+
+	// undo previous scale from rect
+	if( scaley != 1 )
+	{
+		col_rect.h /= scaley;
+		rect.h /= scaley;
+	}
+
 	scaley = scale;
+
+	// set new scale to rect
+	if( scaley != 1 )
+	{
+		col_rect.h *= scaley;
+		rect.h *= scaley;
+	}
+
+	if( new_startscale )
+	{
+		start_scaley = scaley;
+	}
 }
 
-void cSprite :: Set_Scale( float scale )
+void cSprite :: Set_Scale( float scale, bool new_startscale /* = 0 */ )
 {
-	scalex = scale;
-	scaley = scale;
+	Set_ScaleX( scale, new_startscale );
+	Set_ScaleY( scale, new_startscale );
 }
 
 void cSprite :: Add_ScaleX( float val )
@@ -778,8 +856,8 @@ void cSprite :: Add_Scale( float val )
 
 void cSprite :: Set_onTop( cSprite *sprite, bool optimize_hor_pos /* = 1 */ )
 {
-	// set correct onground position
-	posy = sprite->col_rect.y - col_pos.y - col_rect.h - 0.01f;
+	// set ground position 0.1f over it
+	posy = sprite->col_rect.y - col_pos.y - col_rect.h - 0.1f;
 
 	// optimize the horizontal position if given
 	if( optimize_hor_pos && ( posx < sprite->posx || posx > sprite->posx + sprite->col_rect.w ) )
@@ -809,6 +887,11 @@ void cSprite :: Move( float move_x, float move_y, bool real /* = 0 */ )
 	Update_Position_Rect();
 }
 
+void cSprite :: Collide_Move( void )
+{
+	// virtual
+}
+
 void cSprite :: Update_Position_Rect( void )
 {
 	// if not editor mode
@@ -830,7 +913,7 @@ void cSprite :: Update_Position_Rect( void )
 		rect.y = startposy;
 		start_rect.x = startposx;
 		start_rect.y = startposy;
-		// Use not startposx/startposy because col_rect is not the editor/start rect
+		// Don't use startposx/startposy because col_rect is not the editor/start rect
 		col_rect.x = posx + col_pos.x; // todo : startcol_pos ?
 		col_rect.y = posy + col_pos.y;
 	}
@@ -848,6 +931,11 @@ void cSprite :: Update_valid_draw( void )
 	valid_draw = is_Draw_valid();
 }
 
+void cSprite :: Update_valid_update( void )
+{
+	valid_update = is_Update_valid();
+}
+
 void cSprite :: Draw( cSurfaceRequest *request /* = NULL */ )
 {
 	if( !valid_draw )
@@ -857,56 +945,55 @@ void cSprite :: Draw( cSurfaceRequest *request /* = NULL */ )
 
 	Draw_Image( request );
 
+	// draw debugging collision rects
+	if( Game_debug )
+	{
+		// create request
+		cRectRequest *rect_request = new cRectRequest();
+		// draw image rect
+		pVideo->Draw_Rect( &rect, posz + 0.000002f, &lightgrey, rect_request );
+		rect_request->no_camera = no_camera;
+		rect_request->filled = 0;
+		rect_request->blend_sfactor = GL_SRC_COLOR;
+		rect_request->blend_dfactor = GL_DST_ALPHA;
+		// todo : rotation
+		/*rect_request->rotx += rotx;
+		rect_request->roty += roty;
+		rect_request->rotz += rotz;*/
+		// scale
+		float scale_diff_width = rect_request->rect.w - (rect_request->rect.w * scalex);
+		float scale_diff_height = rect_request->rect.h - (rect_request->rect.h * scaley);
+		// todo : sprite scaling should also set the position
+		rect_request->rect.x += scale_diff_width * 0.5f;
+		rect_request->rect.y += scale_diff_height * 0.5f;
+		// add request
+		pRenderer->Add( rect_request );
+
+		// create request
+		rect_request = new cRectRequest();
+		// draw collision rect
+		pVideo->Draw_Rect( &col_rect, posz + 0.000001f, &Get_Sprite_Color( this ), rect_request );
+		rect_request->no_camera = no_camera;
+		// scale
+		scale_diff_width = rect_request->rect.w - (rect_request->rect.w * scalex);
+		scale_diff_height = rect_request->rect.h - (rect_request->rect.h * scaley);
+		// todo : sprite scaling should also set the position
+		rect_request->rect.x += scale_diff_width * 0.5f;
+		rect_request->rect.y += scale_diff_height * 0.5f;
+		// blend it
+		rect_request->blend_sfactor = GL_SRC_COLOR;
+		rect_request->blend_dfactor = GL_DST_ALPHA;
+
+		// add request
+		pRenderer->Add( rect_request );
+	}
+
 	// no editor drawing
 	if( !editor_enabled )
 	{
-		// draw debugging collision rects
+		// draw debug rect
 		if( Game_debug )
 		{
-			// create request
-			cRectRequest *rect_request = new cRectRequest();
-
-			if( type == TYPE_PLAYER )
-			{
-				pVideo->Draw_Rect( col_rect.x - pCamera->x, col_rect.y - pCamera->y, col_rect.w, col_rect.h, posz + 0.000001f, &lila, rect_request );
-			}
-			else if( sprite_array == ARRAY_ENEMY )
-			{
-				pVideo->Draw_Rect( col_rect.x - pCamera->x, col_rect.y - pCamera->y, col_rect.w, col_rect.h, posz + 0.000001f, &red, rect_request );
-			}
-			else if( sprite_array == ARRAY_ACTIVE )
-			{
-				pVideo->Draw_Rect( col_rect.x - pCamera->x, col_rect.y - pCamera->y, col_rect.w, col_rect.h, posz + 0.000001f, &blue, rect_request );
-			}
-			else if( sprite_array == ARRAY_MASSIVE )
-			{
-				pVideo->Draw_Rect( col_rect.x - pCamera->x, col_rect.y - pCamera->y, col_rect.w, col_rect.h, posz + 0.000001f, &orange, rect_request );
-			}
-			else if( sprite_array == ARRAY_PASSIVE )
-			{
-				pVideo->Draw_Rect( col_rect.x - pCamera->x, col_rect.y - pCamera->y, col_rect.w, col_rect.h, posz + 0.000001f, &green, rect_request );
-			}
-			else if( sprite_array == ARRAY_FRONT_PASSIVE )
-			{
-				pVideo->Draw_Rect( col_rect.x - pCamera->x, col_rect.y - pCamera->y, col_rect.w, col_rect.h, posz + 0.000001f, &greenyellow, rect_request );
-			}
-			else
-			{
-				pVideo->Draw_Rect( &col_rect, posz + 0.000001f, &lightgreyalpha64, rect_request );
-
-				if( !no_camera )
-				{
-					rect_request->rect.x -= pCamera->x;
-					rect_request->rect.y -= pCamera->y;
-				}
-			}
-
-			rect_request->blend_sfactor = GL_SRC_COLOR;
-			rect_request->blend_dfactor = GL_DST_ALPHA;
-
-			// add request
-			pRenderer->Add( rect_request );
-
 			if( type == TYPE_PLAYER )
 			{
 				// Get Moving Sprite
@@ -916,9 +1003,10 @@ void cSprite :: Draw( cSurfaceRequest *request /* = NULL */ )
 				if( moving_sprite->ground_object )
 				{
 					// create request
-					rect_request = new cRectRequest();
+					cRectRequest *rect_request = new cRectRequest();
 
-					pVideo->Draw_Rect( moving_sprite->ground_object->col_rect.x - pCamera->x, moving_sprite->ground_object->col_rect.y  - pCamera->y, moving_sprite->ground_object->col_rect.w, moving_sprite->ground_object->col_rect.h, posz + 0.000002f, &grey, rect_request );
+					pVideo->Draw_Rect( &moving_sprite->ground_object->col_rect, posz + 0.000002f, &grey, rect_request );
+					rect_request->no_camera = 0;
 
 					rect_request->blend_sfactor = GL_SRC_COLOR;
 					rect_request->blend_dfactor = GL_DST_ALPHA;
@@ -929,6 +1017,21 @@ void cSprite :: Draw( cSurfaceRequest *request /* = NULL */ )
 			}
 		}
 	}
+	// show obsolete images in editor
+	else if( image && image->obsolete )
+	{
+		// create request
+		cRectRequest *rect_request = new cRectRequest();
+
+		pVideo->Draw_Rect( &col_rect, posz + 0.000001f, &red, rect_request );
+		rect_request->no_camera = 0;
+
+		rect_request->blend_sfactor = GL_SRC_COLOR;
+		rect_request->blend_dfactor = GL_DST_ALPHA;
+
+		// add request
+		pRenderer->Add( rect_request );
+	}
 }
 
 void cSprite :: Draw_Image( cSurfaceRequest *request /* = NULL */ )
@@ -947,27 +1050,93 @@ void cSprite :: Draw_Image( cSurfaceRequest *request /* = NULL */ )
 		request = new cSurfaceRequest();
 	}
 
-	// blit surface
+	// editor
 	if( editor_enabled )
 	{
-		start_image->Blit( startposx, startposy, posz, request );
+		// blit surface
+		start_image->Blit_data( request );
+
+		// rotation
+		request->rotx += start_rotx;
+		request->roty += start_roty;
+		request->rotz += start_rotz;
+		// position x and
+		// scalex
+		if( start_scalex != 1 )
+		{
+			request->scale_x = start_scalex;
+			request->pos_x = startposx + ( start_image->int_x * start_scalex );
+		}
+		// no scalex
+		else
+		{
+			request->pos_x = startposx + start_image->int_x;
+		}
+		// position y and
+		// scaley
+		if( start_scaley != 1 )
+		{
+			request->scale_y = start_scaley;
+			request->pos_y = startposy + ( start_image->int_y * start_scaley );
+		}
+		// no scaley
+		else
+		{
+			request->pos_y = startposy + start_image->int_y;
+		}
+	
+		// if editor z position is given
+		if( editor_posz > 0 )
+		{
+			request->pos_z = editor_posz;
+		}
+		// normal position z
+		else
+		{
+			request->pos_z = posz;
+		}
 	}
+	// no editor
 	else
 	{
-		image->Blit( posx, posy, posz, request );
+		// blit surface
+		image->Blit_data( request );
+
+		// rotation
+		request->rotx += rotx;
+		request->roty += roty;
+		request->rotz += rotz;
+		// position x and
+		// scalex
+		if( scalex != 1 )
+		{
+			request->scale_x = scalex;
+			request->pos_x = posx + ( image->int_x * scalex );
+		}
+		// no scalex
+		else
+		{
+			request->pos_x = posx + image->int_x;
+		}
+		// position y and
+		// scaley
+		if( scaley != 1 )
+		{
+			request->scale_y = scaley;
+			request->pos_y = posy + ( image->int_y * scaley );
+		}
+		// no scaley
+		else
+		{
+			request->pos_y = posy + image->int_y;
+		}
+	
+		// position z
+		request->pos_z = posz;
 	}
 
-	// if editor z position is given
-	if( editor_enabled && editor_posz > 0 )
-	{
-		request->pos_z = editor_posz;
-	}
-
-	if( !no_camera )
-	{
-		request->pos_x -= pCamera->x;
-		request->pos_y -= pCamera->y;
-	}
+	// no camera setting
+	request->no_camera = no_camera;
 
 	// color
 	request->color = color;
@@ -987,28 +1156,6 @@ void cSprite :: Draw_Image( cSurfaceRequest *request /* = NULL */ )
 		request->shadow_color = shadow_color;
 	}
 
-
-	if( editor_enabled )
-	{
-		// rotation
-		request->rotx += start_rotx;
-		request->roty += start_roty;
-		request->rotz += start_rotz;
-		// scale
-		request->scale_x = start_scalex;
-		request->scale_y = start_scaley;
-	}
-	else
-	{
-		// rotation
-		request->rotx += rotx;
-		request->roty += roty;
-		request->rotz += rotz;
-		// scale
-		request->scale_x = scalex;
-		request->scale_y = scaley;
-	}
-
 	if( create_request )
 	{
 		// add request
@@ -1042,7 +1189,7 @@ void cSprite :: Set_Massivetype( MassiveType mtype )
 	massivetype = mtype;
 
 	// set correct Z position
-	pObjManager->Set_Z( this );
+	pLevel->pSprite_Manager->Set_Z( this );
 }
 
 bool cSprite :: is_onTop( cSprite *obj )
@@ -1100,12 +1247,24 @@ bool cSprite :: is_Visible_onScreen( void )
 
 bool cSprite :: is_Player_range( void )
 {
-	if( player_range > 300 && ( posx < pPlayer->posx - player_range || posy < pPlayer->posy - player_range ||
-		posx > pPlayer->posx + player_range || posy > pPlayer->posy + player_range ) )
+	// no player range set
+	if( player_range < 300 )
+	{
+		return is_Visible_onScreen();
+	}
+	// check if not in range
+	else if( posx < pPlayer->posx - player_range || posy < pPlayer->posy - player_range ||
+		posx > pPlayer->posx + player_range || posy > pPlayer->posy + player_range )
 	{
 		return 0;
 	}
 
+	// is in range
+	return 1;
+}
+
+bool cSprite :: is_Update_valid( void )
+{
 	return 1;
 }
 
@@ -1142,6 +1301,9 @@ bool cSprite :: is_Draw_valid( void )
 void cSprite :: Destroy( void )
 {
 	destroy = 1;
+	visible = 0;
+	valid_draw = 0;
+	valid_update = 0;
 }
 
 void cSprite :: Editor_Add( Window *obj, float obj_width, float obj_height )
@@ -1152,14 +1314,14 @@ void cSprite :: Editor_Add( Window *obj, float obj_width, float obj_height )
 	// needs to be always selectable
 	obj->setAlwaysOnTop( 1 );
 	// set size
-	obj->setWidth( UDim( 0, obj_width ) );
-	obj->setHeight( UDim( 0, obj_height ) );
+	obj->setWidth( UDim( 0, obj_width * global_upscalex ) );
+	obj->setHeight( UDim( 0, obj_height * global_upscaley ) );
 	// mouse hover events
 	// if combobox use the editbox
 	if( obj->getType() == "TaharezLook/Combobox" )
 	{
-		((Combobox *)obj)->getEditbox()->subscribeEvent( Window::EventMouseEnters, Event::Subscriber( &cSprite::Editor_Mouse_Enter, this ) );
-		((Combobox *)obj)->getEditbox()->subscribeEvent( Window::EventMouseLeaves, Event::Subscriber( &cSprite::Editor_Mouse_Leave, this ) );
+		static_cast<Combobox *>(obj)->getEditbox()->subscribeEvent( Window::EventMouseEnters, Event::Subscriber( &cSprite::Editor_Mouse_Enter, this ) );
+		static_cast<Combobox *>(obj)->getEditbox()->subscribeEvent( Window::EventMouseLeaves, Event::Subscriber( &cSprite::Editor_Mouse_Leave, this ) );
 	}
 	else
 	{
@@ -1167,7 +1329,6 @@ void cSprite :: Editor_Add( Window *obj, float obj_width, float obj_height )
 		obj->subscribeEvent( Window::EventMouseLeaves, Event::Subscriber( &cSprite::Editor_Mouse_Leave, this ) );
 	}
 
-
 	editor_windows.push_back( obj );
 }
 
@@ -1181,9 +1342,11 @@ void cSprite :: Editor_Deactivate( void )
 	// remove editor controls
 	WindowManager &wmgr = WindowManager::getSingleton();
 
-	for( unsigned int i = 0; i < editor_windows.size(); i++ )
+	for( WindowList::iterator itr = editor_windows.begin(), itr_end = editor_windows.end(); itr != itr_end; ++itr )
 	{
-		wmgr.destroyWindow( editor_windows[i] );
+		Window *obj = (*itr);
+
+		wmgr.destroyWindow( obj );
 	}
 
 	editor_windows.clear();
@@ -1194,27 +1357,34 @@ void cSprite :: Editor_pos_update( void )
 	float obj_posy = 0;
 
 	// set all positions
-	for( unsigned int i = 0; i < editor_windows.size(); i++ )
+	for( WindowList::iterator itr = editor_windows.begin(), itr_end = editor_windows.end(); itr != itr_end; ++itr )
 	{
-		editor_windows[i]->setXPosition( UDim( 0, posx + rect.w + 5 - pCamera->x ) );
-		editor_windows[i]->setYPosition( UDim( 0, posy + 5 + obj_posy - pCamera->y ) );
+		Window *obj = (*itr);
+
+		obj->setXPosition( UDim( 0, ( posx + rect.w + 5 - pCamera->x ) * global_upscalex ) );
+		obj->setYPosition( UDim( 0, ( posy + 5 + obj_posy - pCamera->y ) * global_upscaley ) );
 
 		// if combobox get the editbox height
-		if( editor_windows[i]->getType() == "TaharezLook/Combobox" )
+		if( obj->getType() == "TaharezLook/Combobox" )
 		{
-			obj_posy += ((Combobox *)editor_windows[i])->getEditbox()->getHeight().asAbsolute( GAME_RES_H );
+			obj_posy += static_cast<Combobox *>(obj)->getEditbox()->getHeight().asAbsolute( GAME_RES_H ) * global_downscaley;
 		}
 		// get default height
 		else
 		{
-			obj_posy += editor_windows[i]->getHeight().asAbsolute( GAME_RES_H );
+			obj_posy += obj->getHeight().asAbsolute( GAME_RES_H ) * global_downscaley;
 		}
 
 		// some space between objects
-		obj_posy += 1;
+		obj_posy += 0.5f;
 	}
 }
 
+void cSprite :: Editor_state_update( void )
+{
+	// virtual
+}
+
 bool cSprite :: Editor_Mouse_Enter( const EventArgs &event )
 {
 	const WindowEventArgs &windowEventArgs = static_cast<const WindowEventArgs&>( event );
