@@ -3,14 +3,15 @@
                              -------------------
     copyright            : (C) 2003 - 2007 by Florian Richter
  ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "../core/globals.h"
 #include "../core/main.h"
@@ -28,6 +29,9 @@
 #include "../input/joystick.h"
 #include "../input/keyboard.h"
 #include "../level/level.h"
+// boost filesystem
+#include "boost/filesystem/convenience.hpp"
+namespace fs = boost::filesystem;
 
 
 /* *** *** *** *** *** *** *** *** cOverworld_description *** *** *** *** *** *** *** *** *** */
@@ -37,8 +41,9 @@ cOverworld_description :: cOverworld_description( void )
 	path = "world_1";
 	name = "Unnamed";
 	visible = 1;
+	user = 0;
 
-	comment = "No Comment";
+	comment = "Empty";
 }
 
 cOverworld_description :: ~cOverworld_description( void )
@@ -48,32 +53,62 @@ cOverworld_description :: ~cOverworld_description( void )
 
 void cOverworld_description :: Load( void )
 {
-	string filename = Get_Description_filename();
+	string filename = Get_full_path() + "/description.xml";
 
 	// filename not valid
-	if( !valid_file( filename ) )
+	if( !file_exists( filename ) )
 	{
 		printf( "Error : Couldn't open World description file : %s\n", filename.c_str() );
 		return;
 	}
 
 	// Load Description
-	System::getSingleton().getXMLParser()->parseXMLFile( *this, filename.c_str(), SCHEMA_DIR "/World/Description.xsd", "" );
+	System::getSingleton().getXMLParser()->parseXMLFile( *this, filename.c_str(), DATA_DIR "/" GAME_SCHEMA_DIR "/World/Description.xsd", "" );
 }
 
-string cOverworld_description :: Get_World_filename( void )
+void cOverworld_description :: Save( void )
 {
-	return OVERWORLD_DIR "/" + path + "/world.xml";
+	string save_dir = user_data_dir + USER_WORLD_DIR + "/" + path;
+	string filename = save_dir + "/description.xml";
+
+	ofstream file( filename.c_str(), ios::out | ios::trunc );
+
+	if( !file )
+	{
+		debugdisplay->Set_Text( "Couldn't save world description " + filename );
+		return;
+	}
+
+	// xml info
+	file << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << std::endl;
+	// begin Description
+	file << "<Description>" << std::endl;
+
+	// begin World
+	file << "\t<World>" << std::endl;
+		// name
+		file << "\t\t<Property Name=\"name\" Value=\"" << string_to_xml_string( name ) << "\" />" << std::endl;
+		// visible
+		file << "\t\t<Property Name=\"visible\" Value=\"" << visible << "\" />" << std::endl;
+	// end World
+	file << "\t</World>" << std::endl;
+
+
+	// end Description
+	file << "</Description>" << std::endl;
+	file.close();
 }
 
-string cOverworld_description :: Get_Description_filename( void )
+string cOverworld_description :: Get_full_path( void )
 {
-	return OVERWORLD_DIR "/" + path + "/description.xml";
-}
+	// return user world if available
+	if( fs::exists( fs::path( user_data_dir + USER_WORLD_DIR + "/" + path + "/description.xml", fs::native ) ) )
+	{
+		return user_data_dir + USER_WORLD_DIR + "/" + path;
+	}
 
-string cOverworld_description :: Get_Layer_filename( void )
-{
-	return OVERWORLD_DIR "/" + path + "/layer.xml";
+	// return game world
+	return DATA_DIR "/" GAME_OVERWORLD_DIR "/" + path;
 }
 
 // XML element start
@@ -119,7 +154,7 @@ void cOverworld_description :: handle_world( const XMLAttributes &attributes )
 
 cOverworld :: cOverworld( void )
 {
-	object_manager = new cObjectManager( 500 );
+	sprite_manager = new cSprite_Manager( 500 );
 
 	description = new cOverworld_description();
 	pLayer = new cLayer();
@@ -131,9 +166,6 @@ cOverworld :: cOverworld( void )
 	hud_level_name = new cHudSprite( NULL, 350, 2, 0 );
 	hud_level_name->Set_Shadow( black, 1.5f );
 
-	entered = 0;
-	delayed_enter = 0;
-
 	next_level = 0;
 
 	player_start_waypoint = 0;
@@ -144,6 +176,7 @@ cOverworld :: ~cOverworld( void )
 {
 	Unload();
 
+	delete sprite_manager;
 	delete description;
 	delete pLayer;
 	delete hud_level_name;
@@ -160,30 +193,29 @@ void cOverworld :: Enter( bool delayed /* = 0 */ )
 	// enter on the next update
 	if( delayed )
 	{
-		delayed_enter = 1;
+		Game_Action = GA_ENTER_WORLD;
 		return;
 	}
 
 	// unload level if possible
 	if( pLevel->delayed_unload )
 	{
-		pLevel->Update();
+		pLevel->Unload();
 	}
 
-	delayed_enter = 0;
-
 	// if not loaded
-	if( !object_manager->size() )
+	if( !sprite_manager->size() )
 	{
 		return;
 	}
 
-	entered = 0;
+	// remember old game mode
+	GameMode old_game_mode = Game_Mode;
 	Change_Game_Mode( MODE_OVERWORLD );
 
 	pAudio->PlayMusic( musicfile, -1, 0, 3000 );
 
-	// reset camera
+	// center camera position
 	pCamera->Set_Pos( 0, 0 );
 
 	// if goto next level
@@ -200,7 +232,7 @@ void cOverworld :: Enter( bool delayed /* = 0 */ )
 	}
 
 	// if on start waypoint
-	if( pOverworld_Player->current_waypoint == player_start_waypoint )
+	if( pOverworld_Player->current_waypoint == static_cast<int>(player_start_waypoint) )
 	{
 		// if player state is walk
 		if( player_moving_state == STA_WALK )
@@ -210,41 +242,10 @@ void cOverworld :: Enter( bool delayed /* = 0 */ )
 		}
 	}
 
-
-	while( !entered )
+	if( old_game_mode == MODE_LEVEL )
 	{
-		// Update
-		Update();
-
-		// don't draw if entered
-		if( entered )
-		{
-			break;
-		}
-
-		// Draw
-		Draw();
-		// Render
-		pVideo->Render();
-
-		// update speedfactor
-		pFramerate->Update();
-	}
-
-	// Enter Menu
-	if( entered == 2 )
-	{
-		pMenuCore->Enter();
-	}
-	// Enter Credits Menu
-	else if( entered == 3 )
-	{
-		pMenuCore->Enter( MENU_CREDITS );
-	}
-	// Enter Level
-	else
-	{
-		Change_Game_Mode( MODE_LEVEL );
+		// draw fade in
+		Draw_Effect_in();
 	}
 }
 
@@ -298,17 +299,21 @@ bool cOverworld :: Key_Down( SDLKey key )
 	else if( key == SDLK_l && pOverworld_manager->debugmode )
 	{
 		// toggle layer drawing
-		pLayer->draw = !pLayer->draw;
+		pOverworld_manager->draw_layer = !pOverworld_manager->draw_layer;
 	}
 	else if( pKeyboard->keys[SDLK_g] && pKeyboard->keys[SDLK_o] && pKeyboard->keys[SDLK_d] )
 	{
 		// all waypoint access
 		pActive_Overworld->Set_Progress( pActive_Overworld->waypoints.size(), 1 );
 	}
+	else if( key == SDLK_F3 && pOverworld_manager->debugmode )
+	{
+		pActive_Overworld->Goto_next_Level();
+	}
 	else if( key == SDLK_ESCAPE || key == SDLK_BACKSPACE )
 	{
 		// exit
-		entered = 2;
+		Game_Action = GA_ENTER_MENU;
 	}
 	else if( key == SDLK_RETURN || key == SDLK_KP_ENTER || key == SDLK_SPACE )
 	{
@@ -317,7 +322,7 @@ bool cOverworld :: Key_Down( SDLKey key )
 		ClearInputEvents();
 	}
 	// ## editor
-	else if( pWorld_Editor->Key_Down( key )  )
+	else if( pWorld_Editor->Key_Down( key ) )
 	{
 		// processed by the editor
 		return 1;
@@ -332,6 +337,99 @@ bool cOverworld :: Key_Down( SDLKey key )
 	return 1;
 }
 
+bool cOverworld :: Key_Up( SDLKey key )
+{
+	// nothing yet
+	if( 0 )
+	{
+		//
+	}
+	else
+	{
+		// not processed
+		return 0;
+	}
+
+	// key got processed
+	return 1;
+}
+
+bool cOverworld :: Mouse_Down( Uint8 button )
+{
+	// ## editor
+	if( pWorld_Editor->Mouse_Down( button ) )
+	{
+		// processed by the editor
+		return 1;
+	}
+	else
+	{
+		// not processed
+		return 0;
+	}
+
+	// button got processed
+	return 1;
+}
+
+bool cOverworld :: Mouse_Up( Uint8 button )
+{
+	// ## editor
+	if( pWorld_Editor->Mouse_Up( button ) )
+	{
+		// processed by the editor
+		return 1;
+	}
+	else
+	{
+		// not processed
+		return 0;
+	}
+
+	// button got processed
+	return 1;
+}
+
+bool cOverworld :: Joy_Button_Down( Uint8 button )
+{
+	if( button == pPreferences->joy_button_exit )
+	{
+		// exit
+		Game_Action = GA_ENTER_MENU;
+	}
+	else if( button == pPreferences->joy_button_action )
+	{
+		// enter
+		pOverworld_Player->Activate_Waypoint();
+		ClearInputEvents();
+	}
+	else
+	{
+		// not processed
+		return 0;
+	}
+
+	// key got processed
+	return 1;
+}
+
+bool cOverworld :: Joy_Button_Up( Uint8 button )
+{
+	// nothing yet
+	if( 0 )
+	{
+		//
+	}
+	else
+	{
+		// not processed
+		return 0;
+	}
+
+	// key got processed
+	return 1;
+}
+
 bool cOverworld :: Load( void )
 {
 	Unload();
@@ -343,9 +441,9 @@ bool cOverworld :: Load( void )
 	description->Load();
 
 	// layer
-	string layer_filename = description->Get_Layer_filename();
+	string layer_filename = description->Get_full_path() + "/layer.xml";
 
-	if( !valid_file( layer_filename ) )
+	if( !file_exists( layer_filename ) )
 	{
 		printf( "Couldn't find Overworld Layer file : %s from %s\n", layer_filename.c_str(), description->path.c_str() );
 		return 0;
@@ -354,9 +452,9 @@ bool cOverworld :: Load( void )
 	pLayer->Load( layer_filename );
 
 	// world
-	string world_filename = description->Get_World_filename();
+	string world_filename = description->Get_full_path() + "/world.xml";
 
-	if( !valid_file( world_filename ) )
+	if( !file_exists( world_filename ) )
 	{
 		printf( "Couldn't find Overworld file : %s from %s\n", world_filename.c_str(), description->path.c_str() );
 		return 0;
@@ -365,20 +463,17 @@ bool cOverworld :: Load( void )
 	try
 	{
 		// parse overworld
-		System::getSingleton().getXMLParser()->parseXMLFile( *this, world_filename.c_str(), SCHEMA_DIR "/World/World.xsd", "" );
+		System::getSingleton().getXMLParser()->parseXMLFile( *this, world_filename.c_str(), DATA_DIR "/" GAME_SCHEMA_DIR "/World/World.xsd", "" );
 	}
 	// catch CEGUI Exceptions
 	catch( Exception &ex )
 	{
-		printf( "Overworld Loading CEGUI Exception %s\n", ex.getMessage().c_str() );
+		printf( "Loading Overworld %s CEGUI Exception %s\n", world_filename.c_str(), ex.getMessage().c_str() );
 		debugdisplay->Set_Text( "Overworld Loading failed : " + (string)ex.getMessage().c_str() );
 	}
 
 	hud_world_name->Set_Image( pFont->RenderText( pFont->font_normal, description->name, yellow ), 1, 1 );
 
-	// center camera position
-	pCamera->Set_Pos( 0, 0 );
-
 	return 1;
 }
 
@@ -390,9 +485,9 @@ void cOverworld :: Unload( void )
 	// Waypoints
 	waypoints.clear();
 	// Layer
-	pLayer->Unload();
+	pLayer->Delete_All();
 	// Objects
-	object_manager->Delete_All();
+	sprite_manager->Delete_All();
 
 	pActive_Overworld = NULL;
 }
@@ -401,7 +496,14 @@ void cOverworld :: Save( void )
 {
 	pAudio->PlaySound( "editor/save.ogg" );
 
-	string filename = description->Get_World_filename();
+	string save_dir = user_data_dir + USER_WORLD_DIR + "/" + description->path;
+	// Create directory if new world
+	if( !fs::exists( fs::path( save_dir, fs::native ) ) )
+	{
+		fs::create_directory( fs::path( save_dir, fs::native ) );
+	}
+
+	string filename = save_dir + "/world.xml";
 
 	ofstream file( filename.c_str(), ios::out | ios::trunc );
 
@@ -437,9 +539,9 @@ void cOverworld :: Save( void )
 	// begin background
 	file << "\t<background>" << std::endl;
 		// color
-		file << "\t\t<Property name=\"color_red\" value=\"" << (int)background_color.red << "\" />" << std::endl;
-		file << "\t\t<Property name=\"color_green\" value=\"" << (int)background_color.green << "\" />" << std::endl;
-		file << "\t\t<Property name=\"color_blue\" value=\"" << (int)background_color.blue << "\" />" << std::endl;
+		file << "\t\t<Property name=\"color_red\" value=\"" << static_cast<int>(background_color.red) << "\" />" << std::endl;
+		file << "\t\t<Property name=\"color_green\" value=\"" << static_cast<int>(background_color.green) << "\" />" << std::endl;
+		file << "\t\t<Property name=\"color_blue\" value=\"" << static_cast<int>(background_color.blue) << "\" />" << std::endl;
 	// end background
 	file << "\t</background>" << std::endl;
 
@@ -448,12 +550,12 @@ void cOverworld :: Save( void )
 		// start waypoint
 		file << "\t\t<Property name=\"waypoint\" value=\"" << player_start_waypoint << "\" />" << std::endl;
 		// moving state
-		file << "\t\t<Property name=\"moving_state\" value=\"" << (int)player_moving_state << "\" />" << std::endl;
+		file << "\t\t<Property name=\"moving_state\" value=\"" << static_cast<int>(player_moving_state) << "\" />" << std::endl;
 	// end player
 	file << "\t</player>" << std::endl;
 
 	// objects
-	for( ObjectList::iterator itr = object_manager->items.begin(), itr_end = object_manager->items.end(); itr != itr_end; ++itr )
+	for( SpriteList::iterator itr = sprite_manager->objects.begin(), itr_end = sprite_manager->objects.end(); itr != itr_end; ++itr )
 	{
 		cSprite *obj = (*itr);
 
@@ -472,26 +574,35 @@ void cOverworld :: Save( void )
 	file.close();
 
 	// save Layer
-	if( pLayer->Save( description->Get_Layer_filename() ) )
+	if( pLayer->Save( save_dir + "/layer.xml" ) )
 	{
 		debugdisplay->Set_Text( "Overworld " + description->name + " saved" );
 	}
+
+	// save description
+	description->Save();
 }
 
 void cOverworld :: Set_Progress( unsigned int normal_level, bool force /* = 1 */ )
 {
-	for( unsigned int i = 0; i < waypoints.size(); i++ )
+	unsigned int level_num = 0;
+
+	for( WaypointList::iterator itr = waypoints.begin(), itr_end = waypoints.end(); itr != itr_end; ++itr )
 	{
+		cWaypoint *obj = (*itr);
+
 		// accessible
-		if( normal_level >= i )
+		if( normal_level >= level_num )
 		{
-			waypoints[i]->Set_Access( 1 );
+			obj->Set_Access( 1 );
 		}
 		// force unset
 		else if( force )
 		{
-			waypoints[i]->Set_Access( 0 );
+			obj->Set_Access( 0 );
 		}
+
+		level_num++;
 	}
 }
 
@@ -572,7 +683,7 @@ void cOverworld :: Update_Waypoint_text( void )
 	cWaypoint *waypoint = waypoints[pOverworld_Player->current_waypoint];
 
 	// set color
-	Color color = (Uint8)0;
+	Color color = static_cast<Uint8>(0);
 
 	if( waypoint->waypoint_type == WAYPOINT_NORMAL )
 	{
@@ -597,19 +708,30 @@ bool cOverworld :: Goto_next_Level( void )
 
 	next_level = 0;
 
-	// no Waypoint or invalid current Waypoint
-	if( !waypoints.size() || pOverworld_Player->current_waypoint < 0 || pOverworld_Player->current_waypoint >= (int)waypoints.size() )
+	cWaypoint *current_waypoint = pOverworld_Player->Get_Waypoint();
+
+	// no Waypoint
+	if( !current_waypoint )
 	{
 		return 0;
 	}
 
-	// Waypoint direction is invalid
-	if( waypoints[pOverworld_Player->current_waypoint]->direction_forward == DIR_UNDEFINED )
+	// Waypoint forward direction is invalid/unset
+	if( current_waypoint->direction_forward == DIR_UNDEFINED )
 	{
 		return 0;
 	}
 
-	cWaypoint *next_waypoint = Get_Waypoint( pOverworld_Player->current_waypoint + 1 );
+	// Get Layer Line in front
+	cLayer_Line *front_line = pOverworld_Player->Get_Front_line( current_waypoint->direction_forward );
+
+	if( !front_line )
+	{
+		return 0;
+	}
+
+	// Get forward Waypoint
+	cWaypoint *next_waypoint = front_line->Get_End_Waypoint();
 
 	// if no next waypoint available
 	if( !next_waypoint )
@@ -637,27 +759,29 @@ bool cOverworld :: Goto_next_Level( void )
 		// World Waypoint
 		if( next_waypoint->waypoint_type == WAYPOINT_WORLD_LINK )
 		{
-			anim->Set_Color( lightgreen, Color( (Uint8)60, 0, 60 ) );
+			anim->Set_Color( lightgreen, Color( static_cast<Uint8>(60), 0, 60 ) );
 		}
 		else
 		{
-			anim->Set_Color( orange, Color( (Uint8)6, 60, 20 ) );
+			anim->Set_Color( orange, Color( static_cast<Uint8>(6), 60, 20 ) );
 		}
 
 		// add animation
 		pAnimationManager->Add( anim );
 	}
 
-	pOverworld_Player->Start_Walk( waypoints[pOverworld_Player->current_waypoint]->direction_forward );
+	pOverworld_Player->Start_Walk( current_waypoint->direction_forward );
 
 	return  1;
 }
 
 void cOverworld :: Reset_Waypoints( void )
 {
-	for( unsigned int i = 0; i < waypoints.size(); i++ )
+	for( WaypointList::iterator itr = waypoints.begin(), itr_end = waypoints.end(); itr != itr_end; ++itr )
 	{
-		waypoints[i]->Set_Access( waypoints[i]->access_default );
+		cWaypoint *obj = (*itr);
+
+		obj->Set_Access( obj->access_default );
 	}
 }
 
@@ -690,18 +814,18 @@ void cOverworld :: elementEnd( const String &element )
 			// Music
 			musicfile = xml_attributes.getValueAsString( "music" ).c_str();
 			// Camera Limits
-			//pCamera->Set_Limits( GL_rect( (float)xml_attributes.getValueAsInteger( "cam_limit_x" ), (float)(float)xml_attributes.getValueAsInteger( "cam_limit_y" ), (float)(float)xml_attributes.getValueAsInteger( "cam_limit_w" ), (float)(float)xml_attributes.getValueAsInteger( "cam_limit_h" ) ) );
+			//pCamera->Set_Limits( GL_rect( static_cast<float>(xml_attributes.getValueAsInteger( "cam_limit_x" )), static_cast<float>(xml_attributes.getValueAsInteger( "cam_limit_y" )), static_cast<float>(xml_attributes.getValueAsInteger( "cam_limit_w" )), static_cast<float>(xml_attributes.getValueAsInteger( "cam_limit_h" )) ) );
 		}
 		else if( element == "player" )
 		{
 			// Start Waypoint
 			player_start_waypoint = xml_attributes.getValueAsInteger( "waypoint" );
 			// Moving State
-			player_moving_state = (Moving_state)xml_attributes.getValueAsInteger( "moving_state" );
+			player_moving_state = static_cast<Moving_state>(xml_attributes.getValueAsInteger( "moving_state" ));
 		}
 		else if( element == "background" )
 		{
-			background_color = Color( (Uint8)xml_attributes.getValueAsInteger( "color_red" ), xml_attributes.getValueAsInteger( "color_green" ), xml_attributes.getValueAsInteger( "color_blue" ) );
+			background_color = Color( static_cast<Uint8>(xml_attributes.getValueAsInteger( "color_red" )), xml_attributes.getValueAsInteger( "color_green" ), xml_attributes.getValueAsInteger( "color_blue" ) );
 		}
 		else
 		{
@@ -735,7 +859,7 @@ void cOverworld :: Draw( void )
 	pVideo->Draw_Rect( NULL, 0.00001f, &background_color );
 
 	// Map
-	object_manager->Draw_items();
+	sprite_manager->Draw_items();
 	// Animations
 	pAnimationManager->Draw();
 	// Player
@@ -747,8 +871,6 @@ void cOverworld :: Draw( void )
 	pLayer->Draw();
 	// Editor
 	pWorld_Editor->Draw();
-	// Mouse
-	pMouseCursor->Draw();
 }
 
 void cOverworld :: Draw_HUD( void )
@@ -757,10 +879,10 @@ void cOverworld :: Draw_HUD( void )
 	if( !editor_world_enabled )
 	{
 		// Background
-		Color color = Color( (Uint8)230, 170, 0, 128 );
+		Color color = Color( static_cast<Uint8>(230), 170, 0, 128 );
 		pVideo->Draw_Rect( 0, 0, GAME_RES_W, 30, 0.12f, &color );
 		// Line
-		color = Color( (Uint8)200, 150, 0, 128 );
+		color = Color( static_cast<Uint8>(200), 150, 0, 128 );
 		pVideo->Draw_Rect( 0, 30, GAME_RES_W, 5, 0.121f, &color );
 
 		// Overworld name and level
@@ -775,45 +897,15 @@ void cOverworld :: Draw_HUD( void )
 
 void cOverworld :: Update( void )
 {
-	// Mouse
-	pMouseCursor->Update_Position();
-	pMouseCursor->Update();
-
-	// input
-	while( SDL_PollEvent( &input_event ) )
-	{
-		// was handled
-		if( Handle_Input_Global( &input_event ) )
-		{
-			continue;
-		}
-
-		switch( input_event.type )
-		{
-			case SDL_QUIT:
-			{
-				entered = 1;
-				break;
-			}
-			default:
-			{
-				break;
-			}
-		}
-	}
-
 	// editor
 	pWorld_Editor->Process_Input();
 
-	// Audio
-	pAudio->Update();
-
 	if( !editor_world_enabled )
 	{
 		// Camera
 		Update_Camera();
 		// Map
-		object_manager->Update_items();
+		sprite_manager->Update_items();
 		// Player
 		pOverworld_Player->Update();
 		// hud
@@ -872,7 +964,7 @@ void cOverworld :: Add_Map_Object( cSprite *object )
 	object->player_range = 0;
 
 	// add
-	object_manager->Add( object );
+	sprite_manager->Add( object );
 
 	// Add to Waypoints array
 	if( object->type == TYPE_OW_WAYPOINT )
