@@ -3,14 +3,15 @@
                              -------------------
     copyright            : (C) 2003 - 2007 by Florian Richter
  ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/ 
+/*
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
  
 #include "../overworld/ow_player.h"
 #include "../core/game_core.h"
@@ -45,8 +46,8 @@ cOverworld_Player :: cOverworld_Player( void )
 
 	fixed_walking = 0;
 
-	line_hor.clear();
-	line_ver.clear();
+	line_hor = cLine_collision();
+	line_ver = cLine_collision();
 
 	debug_current_line_last = -100;
 	debug_lines_last = -100;
@@ -117,7 +118,7 @@ void cOverworld_Player :: Load_Images( void )
 	}
 	else
 	{
-		printf( "Unsupported Maryo state : %d\n", (unsigned int)maryo_state );
+		printf( "Unsupported Maryo state : %d\n", static_cast<unsigned int>(maryo_state) );
 		maryo_state = MARYO_SMALL;
 		return;
 	}
@@ -163,7 +164,7 @@ void cOverworld_Player :: Update( void )
 	}
 	else
 	{
-		printf( "Error : Unknown Overworld Animation state %d\n", (unsigned int)maryo_state );
+		printf( "Error : Unknown Overworld Animation state %d\n", static_cast<unsigned int>(maryo_state) );
 	}
 
 	if( direction == DIR_UNDEFINED )
@@ -187,9 +188,30 @@ void cOverworld_Player :: Update( void )
 
 void cOverworld_Player :: Draw( cSurfaceRequest *request /* = NULL */ )
 {
-	// Draw Player
+	bool create_request = 0;
+
+	if( !request )
+	{
+		create_request = 1;
+		// create request
+		request = new cSurfaceRequest();
+	}
+
+	// Draw player
 	cImageObjectSprite::Draw( request );
-	// Draw Debug text
+	// alpha in debug mode
+	if( pOverworld_manager->debugmode )
+	{
+		request->color.alpha = 64;
+	}
+
+	if( create_request )
+	{
+		// add request
+		pRenderer->Add( request );
+	}
+
+	// Draw debug text
 	Draw_Debugtext();
 }
 
@@ -201,9 +223,9 @@ void cOverworld_Player :: Draw_Debugtext( void )
 	}
 
 	// Update Lines Texts
-	if( pActive_Overworld->pLayer->lines.size() != debug_lines_last )
+	if( static_cast<int>(pActive_Overworld->pLayer->size()) != debug_lines_last )
 	{
-		debug_lines->Set_Image( pFont->RenderText( pFont->font_small, "Lines : " + int_to_string( pActive_Overworld->pLayer->lines.size() ), blue ), 1, 1 );
+		debug_lines->Set_Image( pFont->RenderText( pFont->font_small, "Lines : " + int_to_string( pActive_Overworld->pLayer->size() ), blue ), 1, 1 );
 	}
 	// Update Current Line Texts
 	if( current_line != debug_current_line_last )
@@ -287,7 +309,7 @@ void cOverworld_Player :: Activate_Waypoint( void )
 	}
 
 	// get waypoint
-	cWaypoint *waypoint =pActive_Overworld->waypoints[current_waypoint];
+	cWaypoint *waypoint = Get_Waypoint();
 
 	// normal level waypoint
 	if( waypoint->waypoint_type == WAYPOINT_NORMAL )
@@ -297,8 +319,8 @@ void cOverworld_Player :: Activate_Waypoint( void )
 		// valid level
 		if( pLevel->is_Loaded() )
 		{
-			pActive_Overworld->entered = 1;
-			DrawEffect( FIXED_COLORBOX_FADEOUT );
+			// Enter Level
+			Game_Action = GA_ENTER_LEVEL;
 		}
 		// not a valid level - just walk to the next
 		else
@@ -319,8 +341,8 @@ void cOverworld_Player :: Activate_Waypoint( void )
 		// Enter Credits Menu ( World End )
 		if( str_world.compare( "credits" ) == 0 )
 		{
-			DrawEffect( HORIZONTAL_VERTICAL_FADE );
-			pActive_Overworld->entered = 3;
+			Draw_Effect_out( EFFECT_OUT_HORIZONTAL_VERTICAL );
+			Game_Action = GA_ENTER_MENU_CREDITS;
 		}
 		// world link
 		else if( pOverworld_manager->Set_Active( str_world ) )
@@ -336,28 +358,18 @@ void cOverworld_Player :: Activate_Waypoint( void )
 				Set_Waypoint( waypoint_num );
 			}
 
-			DrawEffect( TILE_PIXELATION );
+			Draw_Effect_out( EFFECT_OUT_TILE_PIXELATION );
 		}
 		else
 		{
-			printf( "Error : Overworld world not found %s\n", str_world.c_str() );
+			printf( "Error : Overworld not found %s\n", str_world.c_str() );
 		}
 	}
 }
 
 void cOverworld_Player :: Update_Vel( void )
 {
-	if( direction == DIR_LEFT )
-	{
-		velx = -3;
-		vely = 0;
-	}
-	else if( direction == DIR_RIGHT )
-	{
-		velx = 3;
-		vely = 0;
-	}
-	else if( direction == DIR_UP )
+	if( direction == DIR_UP )
 	{
 		velx = 0;
 		vely = -3;
@@ -367,6 +379,16 @@ void cOverworld_Player :: Update_Vel( void )
 		velx = 0;
 		vely = 3;
 	}
+	else if( direction == DIR_RIGHT )
+	{
+		velx = 3;
+		vely = 0;
+	}
+	else if( direction == DIR_LEFT )
+	{
+		velx = -3;
+		vely = 0;
+	}
 }
 
 bool cOverworld_Player :: Start_Walk( ObjectDirection new_direction )
@@ -391,35 +413,62 @@ bool cOverworld_Player :: Start_Walk( ObjectDirection new_direction )
 
 		if( current_waypoint > 0 )
 		{
-			printf( "Waypoint Direction Forward : %d Backward : %d\n", pActive_Overworld->waypoints[current_waypoint]->direction_forward, pActive_Overworld->waypoints[current_waypoint]->direction_backward );
+			printf( "Waypoint Direction Forward : %d Backward : %d\n", Get_Waypoint()->direction_forward, Get_Waypoint()->direction_backward );
 		}
 	}
 
 	// a start from waypoint
 	if( current_waypoint >= 0 && direction == DIR_UNDEFINED )
 	{
-		// forward
-		if( pActive_Overworld->waypoints[current_waypoint]->direction_forward == new_direction )
+		// Get Layer Line in front
+		cLayer_Line *front_line = Get_Front_line( new_direction );
+
+		if( !front_line )
 		{
+			printf( "Waypoint Front line not detected\n" );
+			return 0;
+		}
+
+		if( pOverworld_manager->debugmode )
+		{
+			printf( "Waypoint Front line id %d, origin id %d\n", pActive_Overworld->pLayer->Get_Array_num( front_line ), front_line->origin );
+		}
+
+		// forward
+		if( Get_Waypoint()->direction_forward == new_direction )
+		{
+			cWaypoint *next_waypoint = front_line->Get_End_Waypoint();
+
+			if( !next_waypoint )
+			{
+				printf( "Next waypoint not detected\n" );
+				return 0;
+			}
+
 			// next waypoint is not accessible
-			if( current_waypoint >= pActive_Overworld->Get_LastValidWaypoint() )
+			if( !next_waypoint->access )
 			{
 				if( pOverworld_manager->debugmode )
 				{
-					printf( "no access to next waypoint\n" );
+					printf( "No access to next waypoint\n" );
 				}
 
 				return 0;
 			}
 
-			// set line waypoint to current waypoint
-			line_waypoint = current_waypoint;
+			if( pOverworld_manager->debugmode )
+			{
+				printf( "Next waypoint id : %d\n", current_waypoint );
+			}
+
+			// set line waypoint
+			line_waypoint = front_line->origin;
 		}
 		// backward
-		else if( pActive_Overworld->waypoints[current_waypoint]->direction_backward == new_direction )
+		else if( Get_Waypoint()->direction_backward == new_direction )
 		{
-			// set line waypoint to previous waypoint
-			line_waypoint = current_waypoint - 1;
+			// set line waypoint
+			line_waypoint = front_line->origin;
 		}
 		// invalid direction
 		else
@@ -452,7 +501,7 @@ bool cOverworld_Player :: Start_Walk( ObjectDirection new_direction )
 	}
 
 	// if maryo is not on his current waypoint collide with every waypoint
-	if( current_waypoint >= 0 && !Col_Box( &col_rect, &pActive_Overworld->waypoints[current_waypoint]->rect ) )
+	if( current_waypoint >= 0 && !Col_Box( &col_rect, &Get_Waypoint()->rect ) )
 	{
 		current_waypoint = -1;
 	}
@@ -480,7 +529,14 @@ void cOverworld_Player :: Update_Walk( void )
 	{
 		if( pOverworld_manager->debugmode )
 		{
-			printf( "line connection broken : num %d diff %f\n", line_hor.line_number, line_hor.difference );
+			if( direction == DIR_UP || direction == DIR_DOWN )
+			{
+				printf( "horizontal line connection broken : num %d diff %f\n", line_hor.line_number, line_hor.difference );
+			}
+			else if( direction == DIR_LEFT || direction == DIR_RIGHT )
+			{
+				printf( "vertical line connection broken : num %d diff %f\n", line_ver.line_number, line_ver.difference );
+			}
 		}
 
 		current_waypoint = -1;
@@ -495,7 +551,7 @@ void cOverworld_Player :: Update_Walk( void )
 	for( unsigned int i = 0; i < pActive_Overworld->waypoints.size(); i++ )
 	{
 		// skip the start waypoint
-		if( (int)i == current_waypoint )
+		if( static_cast<int>(i) == current_waypoint )
 		{
 			continue;
 		}
@@ -526,7 +582,7 @@ void cOverworld_Player :: Update_Waypoint_Walk( void )
 	}
 
 	unsigned int reached = 0;
-	cWaypoint *waypoint = pActive_Overworld->waypoints[current_waypoint];
+	cWaypoint *waypoint = Get_Waypoint();
 
 	// horizontal position to waypoint
 	float x = col_rect.x + ( col_rect.w * 0.5f );
@@ -590,14 +646,29 @@ bool cOverworld_Player :: Set_Waypoint( unsigned int waypoint, bool new_startpos
 	return 1;
 }
 
+cWaypoint *cOverworld_Player :: Get_Waypoint( void )
+{
+	if( current_waypoint < 0 )
+	{
+		return NULL;
+	}
+
+	return pActive_Overworld->Get_Waypoint( current_waypoint );
+}
+
+cLayer_Line *cOverworld_Player :: Get_Front_line( ObjectDirection dir )
+{
+	return pActive_Overworld->pLayer->Get_Line_Collision_Direction( col_rect.x + ( col_rect.w * 0.5f ), col_rect.y + ( col_rect.h * 0.5f ), dir ).line;
+}
+
 void cOverworld_Player :: Update_Path_Diff( unsigned int csize /* = 25 */ )
 {
 	float x = col_rect.x + ( col_rect.w * 0.5f ) + velx;
 	float y = col_rect.y + ( col_rect.h * 0.5f ) + vely;
 
 	// bigger direction check size
-	float hor_advance = 0;
-	float ver_advance = 0;
+	unsigned int hor_advance = 0;
+	unsigned int ver_advance = 0;
 	if( direction == DIR_LEFT || direction == DIR_RIGHT )
 	{
 		ver_advance += 10;
@@ -608,14 +679,14 @@ void cOverworld_Player :: Update_Path_Diff( unsigned int csize /* = 25 */ )
 	}
 
 	// get nearest lines
-	line_hor = pActive_Overworld->pLayer->Get_diff( x, y, DIR_HORIZONTAL, csize + (unsigned int)hor_advance );
- 	line_ver = pActive_Overworld->pLayer->Get_diff( x, y, DIR_VERTICAL, csize + (unsigned int)ver_advance );
+	line_hor = pActive_Overworld->pLayer->Get_nearest( x, y, DIR_HORIZONTAL, csize + hor_advance, line_waypoint );
+ 	line_ver = pActive_Overworld->pLayer->Get_nearest( x, y, DIR_VERTICAL, csize + ver_advance, line_waypoint );
 }
 
 void cOverworld_Player :: Change_Direction( void )
 {
-	line_hor.clear();
-	line_ver.clear();
+	line_hor = cLine_collision();
+	line_ver = cLine_collision();
 
 	float x = col_rect.x + ( col_rect.w * 0.5f );
 	float y = col_rect.y + ( col_rect.h * 0.5f );
@@ -629,7 +700,7 @@ void cOverworld_Player :: Change_Direction( void )
 		// search for a line with increasing distance
 		for( unsigned int i = 0; i < 30; i++ )
 		{
-			line_ver = pActive_Overworld->pLayer->Get_diff( x - i, y, DIR_VERTICAL, 80 );
+			line_ver = pActive_Overworld->pLayer->Get_nearest( x - i, y, DIR_VERTICAL, 80, line_waypoint );
 
 			// found
 			if( line_ver.line_number >= 0 )
@@ -637,7 +708,7 @@ void cOverworld_Player :: Change_Direction( void )
 				break;
 			}
 
-			line_ver = pActive_Overworld->pLayer->Get_diff( x + i, y, DIR_VERTICAL, 80 );
+			line_ver = pActive_Overworld->pLayer->Get_nearest( x + i, y, DIR_VERTICAL, 80, line_waypoint );
 
 			// found
 			if( line_ver.line_number >= 0 )
@@ -669,7 +740,7 @@ void cOverworld_Player :: Change_Direction( void )
 		// search for a line with increasing distance
 		for( unsigned int i = 0; i < 30; i++ )
 		{
-			line_hor = pActive_Overworld->pLayer->Get_diff( x, y + i, DIR_HORIZONTAL, 80 );
+			line_hor = pActive_Overworld->pLayer->Get_nearest( x, y + i, DIR_HORIZONTAL, 80, line_waypoint );
 
 			// found
 			if( line_hor.line_number >= 0 )
@@ -677,7 +748,7 @@ void cOverworld_Player :: Change_Direction( void )
 				break;
 			}
 
-			line_hor = pActive_Overworld->pLayer->Get_diff( x, y - i, DIR_HORIZONTAL, 80 );
+			line_hor = pActive_Overworld->pLayer->Get_nearest( x, y - i, DIR_HORIZONTAL, 80, line_waypoint );
 
 			// found
 			if( line_hor.line_number >= 0 )
@@ -706,13 +777,13 @@ void cOverworld_Player :: Change_Direction( void )
 	}
 	else
 	{
-		printf( "Warning : Unknown Maryo direction %d\n", (int)direction );
+		printf( "Warning : Unknown Maryo direction %d\n", static_cast<int>(direction) );
 		direction = DIR_DOWN;
 	}
 
 	if( pOverworld_manager->debugmode )
 	{
-		printf( "Changed direction to %d\n", direction );
+		printf( "Changed direction to %s\n", Get_Direction_name( direction ).c_str() );
 	}
 
 	Update_Vel();
