@@ -3,14 +3,15 @@
                              -------------------
     copyright            : (C) 2003 - 2007 by Florian Richter
  ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/ 
+/*
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "../core/globals.h"
 #include "../user/savegame.h"
@@ -20,8 +21,9 @@
 #include "../core/obj_manager.h"
 #include "../level/level.h"
 #include "../overworld/worlds.h"
-#include "../objects/bonusbox.h"
-#include "../objects/goldpiece.h"
+#include "../player/player.h"
+#include "../overworld/overworld.h"
+
 
 /* *** *** *** *** *** *** *** cSave_Overworld_Waypoint *** *** *** *** *** *** *** *** *** *** */
 
@@ -72,6 +74,21 @@ cSave_Level_Object :: ~cSave_Level_Object( void )
 	properties.clear();
 }
 
+bool cSave_Level_Object :: exists( string val_name )
+{
+	for( unsigned int i = 0; i < properties.size(); i++ )
+	{
+		if( properties[i].Name.compare( val_name ) == 0 )
+		{
+			// found
+			return 1;
+		}
+	}
+
+	// not found
+	return 0;
+}
+
 string cSave_Level_Object :: Get_Value( string val_name )
 {
 	for( unsigned int i = 0; i < properties.size(); i++ )
@@ -110,6 +127,7 @@ void cSave :: Clear( void )
 	lives = 0;
 	points = 0;
 	goldpieces = 0;
+	player_type = 0;
 	player_state = 0;
 	itembox_item = 0;
 	
@@ -143,7 +161,7 @@ cSavegame :: cSavegame( void )
 	save_temp = NULL;
 	debug = 0;
 
-	savegame_dir = user_data_dir + SAVE_DIR;
+	savegame_dir = user_data_dir + USER_SAVEGAME_DIR;
 }
 
 cSavegame :: ~cSavegame( void )
@@ -171,10 +189,10 @@ int cSavegame :: Load_Game( unsigned int save_slot )
 	// level available
 	if( savegame->level_name.length() )
 	{
-		string level_name = pPreferences->level_dir + "/" + savegame->level_name;
+		string level_name = savegame->level_name;
 
 		// level not found
-		if( !valid_file( level_name + ".txt" ) && !valid_file( level_name + ".smclvl" ) )
+		if( !pLevel->Get_path( level_name ) )
 		{
 			printf( "Warning : Savegame %d : Level not found : %s\n", save_slot, level_name.c_str() );
 		}
@@ -182,8 +200,6 @@ int cSavegame :: Load_Game( unsigned int save_slot )
 
 	// unload level
 	pLevel->Unload();
-	// clear deleted objects
-	pObjManager->Update();
 	// reset player
 	pPlayer->ResetSave();
 
@@ -194,35 +210,39 @@ int cSavegame :: Load_Game( unsigned int save_slot )
 	{
 		// Use first Overworld
 		cSave_Overworld *save_overworld = new cSave_Overworld();
-		save_overworld->name = pOverworld_manager->Get( 0 )->description->path;
+		save_overworld->name = pOverworld_manager->Get_Pointer( 0 )->description->path;
 
 		savegame->Overworlds.push_back( save_overworld );
 	}
 	// set overworld progress
 	else
 	{
-		for( unsigned int i = 0; i < savegame->Overworlds.size(); i++ )
+		for( Save_OverworldList::iterator itr = savegame->Overworlds.begin(), itr_end = savegame->Overworlds.end(); itr != itr_end; ++itr )
 		{
-			cOverworld *overworld = pOverworld_manager->Get_from_Name( savegame->Overworlds[i]->name );
+			// get savegame overworld pointer
+			cSave_Overworld *save_overworld = (*itr);
+
+			// get overworld
+			cOverworld *overworld = pOverworld_manager->Get_from_Name( save_overworld->name );
 
 			if( !overworld )
 			{
-				printf( "Warning : Savegame %d : Overworld %s not found\n", save_slot, savegame->Overworlds[i]->name.c_str() );
+				printf( "Warning : Savegame %d : Overworld %s not found\n", save_slot, save_overworld->name.c_str() );
 				continue;
 			}
 
-			for( unsigned int w = 0; w < savegame->Overworlds[i]->Waypoints.size() ; w++ )
+			for( Save_Overworld_WaypointList::iterator wp_itr = save_overworld->Waypoints.begin(), wp_itr_end = save_overworld->Waypoints.end(); wp_itr != wp_itr_end; ++wp_itr )
 			{
-				// Get Savegame Waypoint
-				cSave_Overworld_Waypoint *save_waypoint = savegame->Overworlds[i]->Waypoints[w];
+				// get savegame waypoint pointer
+				cSave_Overworld_Waypoint *save_waypoint = (*wp_itr);
 
-				// Get Overworld Waypoint
+				// get overworld waypoint
 				cWaypoint *waypoint = overworld->Get_Waypoint( overworld->Get_Waypoint_num( save_waypoint->destination ) );
 
 				// not found
 				if( !waypoint )
 				{
-					printf( "Warning : Savegame %d : Overworld %s Waypoint %s not found\n", save_slot, savegame->Overworlds[i]->name.c_str(), save_waypoint->destination.c_str() );
+					printf( "Warning : Savegame %d : Overworld %s Waypoint %s not found\n", save_slot, save_overworld->name.c_str(), save_waypoint->destination.c_str() );
 					continue;
 				}
 
@@ -247,7 +267,20 @@ int cSavegame :: Load_Game( unsigned int save_slot )
 
 	// #### Player ####
 
-	pPlayer->Set_Type( (Maryo_type)savegame->player_state, 0, 0 );
+	// below version 8 the sate was the type
+	if( savegame->version < 8 )
+	{
+		// type
+		pPlayer->Set_Type( (Maryo_type)savegame->player_state, 0, 0 );
+	}
+	else
+	{
+		// type
+		pPlayer->Set_Type( (Maryo_type)savegame->player_type, 0, 0 );
+		// state
+		pPlayer->state = (Moving_state)savegame->player_state;
+	}
+
 
 	// in a level
 	if( !savegame->level_name.empty() )
@@ -256,52 +289,29 @@ int cSavegame :: Load_Game( unsigned int save_slot )
 		if( pLevel->Load( savegame->level_name ) )
 		{
 			// position
-			pPlayer->Set_Pos( savegame->level_posx, savegame->level_posy - (float)GAME_RES_H );
+			pPlayer->Set_Pos( savegame->level_posx, savegame->level_posy - static_cast<float>(GAME_RES_H) );
 
 			// Level Objects
-			for( unsigned int i = 0; i < savegame->level_objects.size(); i++ )
+			for( Save_Level_ObjectList::iterator itr = savegame->level_objects.begin(), itr_end = savegame->level_objects.end(); itr != itr_end; ++itr )
 			{
 				// get object pointer
-				cSave_Level_Object *save_object = savegame->level_objects[i];
+				cSave_Level_Object *save_object = (*itr);
 
 				// get position
 				int posx = string_to_int( save_object->Get_Value( "posx" ) );
 				int posy = string_to_int( save_object->Get_Value( "posy" ) );
 
 				// get level object
-				cSprite *level_object = pObjManager->Get_from_Position( posx, posy, save_object->type );
+				cSprite *level_object = pLevel->pSprite_Manager->Get_from_Position( posx, posy, save_object->type );
 
 				// if not anymore available
 				if( !level_object )
 				{
-					printf( "Warning : Savegame object type %d on pos x %d, y %d not available\n", save_object->type, posx, posy );
+					printf( "Warning : Savegame object type %d on x %d, y %d not available\n", save_object->type, posx, posy );
 					continue;
 				}
 
-				if( level_object->type == TYPE_BONUSBOX )
-				{
-					cBonusBox *bonusbox = (cBonusBox *)level_object;
-
-					// useable count
-					int useable_count = string_to_int( save_object->Get_Value( "useable_count" ) );
-					bonusbox->Set_UseableCount( useable_count );
-				}
-				else if( level_object->type == TYPE_GOLDPIECE )
-				{
-					cGoldpiece *goldpiece = (cGoldpiece *)level_object;
-
-					// visible
-					bool visible = string_to_int( save_object->Get_Value( "visible" ) ) > 0;
-					goldpiece->visible = visible;
-				}
-				else if( level_object->type == TYPE_MOON )
-				{
-					cMoon *moon = (cMoon *)level_object;
-
-					// visible
-					bool visible = string_to_int( save_object->Get_Value( "visible" ) ) > 0;
-					moon->visible = visible;
-				}
+				level_object->Load_from_Savegame( save_object );
 			}
 
 			// invincible for a second
@@ -316,16 +326,11 @@ int cSavegame :: Load_Game( unsigned int save_slot )
 	pointsdisplay->Set_Points( savegame->points );
 	golddisplay->SetGold( savegame->goldpieces );
 	livedisplay->SetLives( savegame->lives );
-	Itembox->Set_Item( (SpriteType)savegame->itembox_item );
+	Itembox->Set_Item( (SpriteType)savegame->itembox_item, 0 );
 
-	pCamera->Set_Pos( 0, 0 );
+	pCamera->Center();
 
-	pPlayer->Ball_clear();
-
-	debugdisplay->Update();
-	pointsdisplay->Update();
-	pPlayer->Update();
-	golddisplay->Update();
+	pHudManager->Update();
 
 	debugdisplay->Set_Text( "Savegame " + int_to_string( save_slot ) + " loaded" );
 
@@ -346,7 +351,7 @@ bool cSavegame :: Save_Game( unsigned int save_slot, string description )
 {
 	if( pPlayer->maryo_type == MARYO_DEAD || pPlayer->lives < 0 )
 	{
-		printf( "Couldn't save Savegame %s\n", description.c_str() );
+		printf( "Error : Couldn't save savegame %s because of invalid game state\n", description.c_str() );
 		return 0;
 	}
 
@@ -368,72 +373,20 @@ bool cSavegame :: Save_Game( unsigned int save_slot, string description )
 		savegame->level_posy = pPlayer->posy + GAME_RES_H - 5;
 
 		// Level Objects
-		for( ObjectList::iterator itr = pObjManager->items.begin(), itr_end = pObjManager->items.end(); itr != itr_end; ++itr )
+		for( SpriteList::iterator itr = pLevel->pSprite_Manager->objects.begin(), itr_end = pLevel->pSprite_Manager->objects.end(); itr != itr_end; ++itr )
 		{
 			// get object pointer
 			cSprite *object = (*itr);
-			// savegame object;
-			cSave_Level_Object *save_object = NULL;
 
-			// Bonusbox
-			if( object->type == TYPE_BONUSBOX )
-			{
-				cBonusBox *bonusbox = (cBonusBox *)object;
+			// get save data
+			cSave_Level_Object *save_object = object->Save_to_Savegame();
 
-				// only save if needed
-				if( bonusbox->useable_count == bonusbox->start_useable_count )
-				{
-					continue;
-				}
-
-				save_object = new cSave_Level_Object();
-
-				// Bonusbox Useable Count
-				save_object->properties.push_back( cSave_Level_Object_Property( "useable_count", int_to_string( bonusbox->useable_count ) ) );
-			}
-			// Goldpiece
-			else if( object->type == TYPE_GOLDPIECE )
-			{
-				cGoldpiece *goldpiece = (cGoldpiece *)object;
-
-				// only save if needed
-				if( goldpiece->visible == 1 )
-				{
-					continue;
-				}
-
-				save_object = new cSave_Level_Object();
-
-				// Goldpiece visible
-				save_object->properties.push_back( cSave_Level_Object_Property( "visible", int_to_string( goldpiece->visible ) ) );
-			}
-			// Moon
-			else if( object->type == TYPE_MOON )
-			{
-				cMoon *moon = (cMoon *)object;
-
-				// only save if needed
-				if( moon->visible == 1 )
-				{
-					continue;
-				}
-
-				save_object = new cSave_Level_Object();
-
-				// Moon visible
-				save_object->properties.push_back( cSave_Level_Object_Property( "visible", int_to_string( moon->visible ) ) );
-			}
-			// other object
-			else
+			// nothing to save
+			if( !save_object )
 			{
 				continue;
 			}
 
-			// save default values
-			save_object->type = object->type;
-			save_object->properties.push_back( cSave_Level_Object_Property( "posx", int_to_string( (int)object->startposx ) ) );
-			save_object->properties.push_back( cSave_Level_Object_Property( "posy", int_to_string( (int)object->startposy ) ) );
-
 			// add
 			savegame->level_objects.push_back( save_object );
 		}
@@ -444,16 +397,18 @@ bool cSavegame :: Save_Game( unsigned int save_slot, string description )
 	// Points
 	savegame->points = pPlayer->points;
 
-	// Player State
-	savegame->player_state = pPlayer->maryo_type;
+	// Player type
+	savegame->player_type = pPlayer->maryo_type;
+	// Player state
+	savegame->player_state = pPlayer->state;
 	// Itembox Item
 	savegame->itembox_item = Itembox->item_id;
 
 	// save overworld progress
-	for( unsigned int i = 0; i < pOverworld_manager->worlds.size(); i++ )
+	for( vector<cOverworld *>::iterator itr = pOverworld_manager->objects.begin(), itr_end = pOverworld_manager->objects.end(); itr != itr_end; ++itr )
 	{
 		// Get Overworld
-		cOverworld *overworld = pOverworld_manager->worlds[i];
+		cOverworld *overworld = (*itr);
 
 		// create Overworld
 		cSave_Overworld *save_overworld = new cSave_Overworld();
@@ -461,16 +416,19 @@ bool cSavegame :: Save_Game( unsigned int save_slot, string description )
 		save_overworld->name = overworld->description->name;
 		
 		// Waypoints
-		for( ObjectList::iterator itr = overworld->object_manager->items.begin(), itr_end = overworld->object_manager->items.end(); itr != itr_end; ++itr )
+		for( SpriteList::iterator wp_itr = overworld->sprite_manager->objects.begin(), wp_itr_end = overworld->sprite_manager->objects.end(); wp_itr != wp_itr_end; ++wp_itr )
 		{
 			// get waypoint
-			cWaypoint *waypoint = static_cast<cWaypoint *>(*itr);
+			cSprite *obj = static_cast<cSprite *>(*wp_itr);
 
-			if( waypoint->type != TYPE_OW_WAYPOINT )
+			if( obj->type != TYPE_OW_WAYPOINT )
 			{
 				continue;
 			}
 
+			// get waypoint
+			cWaypoint *waypoint = static_cast<cWaypoint *>(obj);
+
 			// create savegame waypoint
 			cSave_Overworld_Waypoint *save_waypoint = new cSave_Overworld_Waypoint();
 			
@@ -526,7 +484,7 @@ cSave *cSavegame :: Load( unsigned int save_slot )
 
 	string filename = savegame_dir + "/" + int_to_string( save_slot ) + ".save";
 
-	if( !valid_file( filename ) )
+	if( !file_exists( filename ) )
 	{
 		printf( "Error : Savegame Loading : No Savegame found at Slot : %s\n", filename.c_str() );
 		return save_temp;
@@ -534,12 +492,12 @@ cSave *cSavegame :: Load( unsigned int save_slot )
 
 	try
 	{
-		System::getSingleton().getXMLParser()->parseXMLFile( *this, filename.c_str(), SCHEMA_DIR "/Savegame.xsd", "" );
+		System::getSingleton().getXMLParser()->parseXMLFile( *this, filename.c_str(), DATA_DIR "/" GAME_SCHEMA_DIR "/Savegame.xsd", "" );
 	}
 	// catch CEGUI Exceptions
 	catch( Exception &ex )
 	{
-		printf( "Savegame Loading CEGUI Exception %s\n", ex.getMessage().c_str() );
+		printf( "Loading Savegame %s CEGUI Exception %s\n", filename.c_str(), ex.getMessage().c_str() );
 		debugdisplay->Set_Text( "Savegame Loading failed : " + (string)ex.getMessage().c_str() );
 	}
 
@@ -565,7 +523,7 @@ int cSavegame :: Save( unsigned int save_slot, cSave *savegame )
 {
 	string filename = savegame_dir + "/" + int_to_string( save_slot ) + ".save";
 
-	if( valid_file( filename ) )
+	if( file_exists( filename ) )
 	{
 		ifstream ifs( filename.c_str(), ios::trunc ); // Delete existing
 		ifs.close();
@@ -581,7 +539,7 @@ int cSavegame :: Save( unsigned int save_slot, cSave *savegame )
 	
 	if( !file.is_open() )
 	{
-		printf( "Error : couldn't open savegame file for saving. Is the file read-only ?" );
+		printf( "Error : Couldn't open savegame file for saving. Is the file read-only ?" );
 		return 0;
 	}
 
@@ -613,10 +571,10 @@ int cSavegame :: Save( unsigned int save_slot, cSave *savegame )
 		file << "\t\t<Property Name=\"player_posy\" Value=\"" << savegame->level_posy << "\" />" << std::endl;
 
 		// Level Objects
-		for( unsigned int i = 0; i < savegame->level_objects.size(); i++ )
+		for( Save_Level_ObjectList::iterator itr = savegame->level_objects.begin(), itr_end = savegame->level_objects.end(); itr != itr_end; ++itr )
 		{
 			// get object pointer
-			cSave_Level_Object *object = savegame->level_objects[i];
+			cSave_Level_Object *object = (*itr);
 
 			// begin Level Object
 			file << "\t\t<Level_Object>" << std::endl;
@@ -625,10 +583,10 @@ int cSavegame :: Save( unsigned int save_slot, cSave *savegame )
 			file << "\t\t\t<Property Name=\"type\" Value=\"" << object->type << "\" />" << std::endl;
 
 			// Properties
-			for( unsigned int i = 0; i < object->properties.size(); i++ )
+			for( Save_Level_Object_ProprtyList::iterator prop_itr = object->properties.begin(), prop_itr_end = object->properties.end(); prop_itr != prop_itr_end; ++prop_itr )
 			{
 				// get properties pointer
-				cSave_Level_Object_Property Property = object->properties[i];
+				cSave_Level_Object_Property Property = (*prop_itr);
 
 				// property
 				file << "\t\t\t<Property Name=\"" << Property.Name <<"\" Value=\"" << Property.Value << "\" />" << std::endl;
@@ -649,6 +607,8 @@ int cSavegame :: Save( unsigned int save_slot, cSave *savegame )
 		file << "\t\t<Property Name=\"points\" Value=\"" << savegame->points << "\" />" << std::endl;
 		// Goldpieces
 		file << "\t\t<Property Name=\"goldpieces\" Value=\"" << savegame->goldpieces << "\" />" << std::endl;
+		// type
+		file << "\t\t<Property Name=\"type\" Value=\"" << savegame->player_type << "\" />" << std::endl;
 		// state
 		file << "\t\t<Property Name=\"state\" Value=\"" << savegame->player_state << "\" />" << std::endl;
 		// Itembox item
@@ -665,17 +625,22 @@ int cSavegame :: Save( unsigned int save_slot, cSave *savegame )
 	// end Overworld_Data
 	file << "\t</Overworld_Data>" << std::endl;
 
-	for( unsigned int i = 0; i < savegame->Overworlds.size(); i++ )
+	// Overworlds
+	for( Save_OverworldList::iterator itr = savegame->Overworlds.begin(), itr_end = savegame->Overworlds.end(); itr != itr_end; ++itr )
 	{
+		// get object pointer
+		cSave_Overworld *overworld = (*itr);
+
 		// begin Overworld
 		file << "\t<Overworld>" << std::endl;
 
 		// current Overworld
-		file << "\t\t<Property Name=\"name\" Value=\"" << savegame->Overworlds[i]->name << "\" />" << std::endl;
+		file << "\t\t<Property Name=\"name\" Value=\"" << overworld->name << "\" />" << std::endl;
 
-		for( unsigned int w = 0; w < savegame->Overworlds[i]->Waypoints.size(); w++ )
+		for( Save_Overworld_WaypointList::iterator wp_itr = overworld->Waypoints.begin(), wp_itr_end = overworld->Waypoints.end(); wp_itr != wp_itr_end; ++wp_itr )
 		{
-			cSave_Overworld_Waypoint *overworld_waypoint = savegame->Overworlds[i]->Waypoints[w];
+			// get object pointer
+			cSave_Overworld_Waypoint *overworld_waypoint = (*wp_itr);
 
 			// skip empty waypoints
 			if( overworld_waypoint->destination.empty() )
@@ -718,7 +683,7 @@ string cSavegame :: Get_Description( unsigned int save_slot, bool only_descripti
 
 	savefile = savegame_dir + "/" + int_to_string( save_slot ) + ".save";
 
-	if( !valid_file( savefile ) )
+	if( !file_exists( savefile ) )
 	{
 		str_description = int_to_string( save_slot ) + ". Free Save";
 		return str_description;
@@ -760,7 +725,7 @@ string cSavegame :: Get_Description( unsigned int save_slot, bool only_descripti
 
 bool cSavegame :: is_valid( unsigned int save_slot )
 {
-	return valid_file( savegame_dir + "/" + int_to_string( save_slot ) + ".save" );
+	return file_exists( savegame_dir + "/" + int_to_string( save_slot ) + ".save" );
 }
 
 // XML element start
@@ -851,35 +816,35 @@ void cSavegame :: Handle_Level_Object( const XMLAttributes &attributes )
 
 	// type
 	object->type = (SpriteType)type;
+	xml_attributes.remove( "type" );
 
-	// position
-	object->properties.push_back( cSave_Level_Object_Property( "posx", xml_attributes.getValueAsString( "posx" ).c_str() ) );
-	object->properties.push_back( cSave_Level_Object_Property( "posy", xml_attributes.getValueAsString( "posy" ).c_str() ) );
 
-	if( type == TYPE_BONUSBOX )
+	// Get Properties
+	for( unsigned int i = 0; i < xml_attributes.getCount(); i++ )
 	{
-		// useable count
-		object->properties.push_back( cSave_Level_Object_Property( "useable_count", xml_attributes.getValueAsString( "useable_count" ).c_str() ) );
+		// get property
+		string property_name = xml_attributes.getName( i ).c_str();
 
-		// clear
-		xml_attributes.remove( "useable_count" );
+		// ignore level attributes
+		if( property_name.compare( "level_name" ) == 0 || property_name.compare( "player_posx" ) == 0 || property_name.compare( "player_posy" ) == 0 )
+		{
+			continue;
+		}
+
+		object->properties.push_back( cSave_Level_Object_Property( property_name, xml_attributes.getValue( i ).c_str() ) );
 	}
-	else if( type == TYPE_GOLDPIECE || type == TYPE_MOON )
-	{
-		// visible
-		object->properties.push_back( cSave_Level_Object_Property( "visible", xml_attributes.getValueAsString( "visible" ).c_str() ) );
 
-		// clear
-		xml_attributes.remove( "visible" );
+	// remove used Properties
+	for( Save_Level_Object_ProprtyList::iterator prop_itr = object->properties.begin(), prop_itr_end = object->properties.end(); prop_itr != prop_itr_end; ++prop_itr )
+	{
+		// get property pointer
+		cSave_Level_Object_Property Property = (*prop_itr);
+
+		xml_attributes.remove( Property.Name );
 	}
 
 	// add object
 	level_objects.push_back( object );
-
-	// clear
-	xml_attributes.remove( "type" );
-	xml_attributes.remove( "posx" );
-	xml_attributes.remove( "posy" );
 }
 
 void cSavegame :: Handle_Player( const XMLAttributes &attributes )
@@ -887,6 +852,7 @@ void cSavegame :: Handle_Player( const XMLAttributes &attributes )
 	save_temp->lives = xml_attributes.getValueAsInteger( "lives" );
 	save_temp->points = xml_attributes.getValueAsInteger( "points" );
 	save_temp->goldpieces = xml_attributes.getValueAsInteger( "goldpieces" );
+	save_temp->player_type = xml_attributes.getValueAsInteger( "type" );
 	save_temp->player_state = xml_attributes.getValueAsInteger( "state" );
 	save_temp->itembox_item = xml_attributes.getValueAsInteger( "itembox_item" );
 }
