@@ -3,14 +3,15 @@
                              -------------------
     copyright            : (C) 2003 - 2007 by Florian Richter
  ***************************************************************************/
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/ 
+/*
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "../video/animation.h"
 #include "../core/framerate.h"
@@ -20,8 +21,8 @@
 
 /* *** *** *** *** *** *** *** Base Animation class *** *** *** *** *** *** *** *** *** *** */
 
-cAnimation :: cAnimation( float posx, float posy )
-: cImageObjectSprite( posx, posy )
+cAnimation :: cAnimation( float x, float y )
+: cImageObjectSprite( x, y )
 {
 	sprite_array = ARRAY_ANIM;
 	type = TYPE_ACTIVESPRITE;
@@ -93,13 +94,19 @@ cBlinkAnimation :: cBlinkAnimation( float posx, float posy, unsigned int height
 
 	for( unsigned int i = 0; i < 4; i++ )
 	{
-		cSprite object( NULL, (float)( rand() % ( width ) ), (float)( rand() % ( height ) ) );
-		objects.push_back( object );
+		cSprite *obj = new cSprite( NULL, static_cast<float>( rand() % ( width ) ), static_cast<float>( rand() % ( height ) ) );
+		objects.push_back( obj );
 	}
 }
 
 cBlinkAnimation :: ~cBlinkAnimation( void )
 {
+	// clear
+	for( BlinkPointList::iterator itr = objects.begin(), itr_end = objects.end(); itr != itr_end; ++itr )
+	{
+		delete *itr;
+	}
+
 	objects.clear();
 }
 
@@ -198,7 +205,7 @@ void cBlinkAnimation :: Update( void )
 		}
 		}
 
-		objects[i].Set_Scale( 1.1f - ( time_to_live / 12 ) );
+		objects[i]->Set_Scale( 1.1f - ( time_to_live / 12 ) );
 	}
 }
 
@@ -209,30 +216,35 @@ void cBlinkAnimation :: Draw( cSurfaceRequest *request /* = NULL */ )
 		return;
 	}
 
-	// draw the fixed points
-	for( unsigned int i = 0; i < objects.size(); i++ )
+	if( !image )
 	{
-		if( image )
-		{
-			// create request
-			cSurfaceRequest *request = new cSurfaceRequest();
-			image->Blit( objects[i].posx - ( pCamera->x - posx ), objects[i].posy - ( pCamera->y - posy ), posz, request );
+		Set_Visible( 0 );
+		return;
+	}
 
-			// scale
-			request->scale_x = objects[i].scalex;
-			request->scale_y = objects[i].scaley;
+	// draw the fixed points
+	for( BlinkPointList::iterator itr = objects.begin(), itr_end = objects.end(); itr != itr_end; ++itr )
+	{
+		cSprite *obj = (*itr);
 
-			// color
-			request->color = color;
+		// create request
+		cSurfaceRequest *request = new cSurfaceRequest();
+		image->Blit( obj->posx - ( pCamera->x - posx ), obj->posy - ( pCamera->y - posy ), posz, request );
 
-			// add request
-			pRenderer->Add( request );
-		}
+		// scale
+		request->scale_x = obj->scalex;
+		request->scale_y = obj->scaley;
 
-		if( time_to_live > 11 || time_to_live < 0 )
-		{
-			visible = 0;
-		}
+		// color
+		request->color = color;
+
+		// add request
+		pRenderer->Add( request );
+	}
+	
+	if( time_to_live > 11 || time_to_live < 0 )
+	{
+		Set_Visible( 0 );
 	}
 }
 
@@ -243,41 +255,51 @@ cFireAnimation :: cFireAnimation( float posx, float posy, unsigned int power /*
 {
 	animtype = FIRE_EXPLOSION;
 
-	images.push_back( pVideo->Get_Surface( "animation/fire_1/4.png" ) );
-	images.push_back( pVideo->Get_Surface( "animation/fire_1/3.png" ) );
-	images.push_back( pVideo->Get_Surface( "animation/fire_1/2.png" ) );
-	images.push_back( pVideo->Get_Surface( "animation/fire_1/1.png" ) );
-
-	// fill the objects list
+	// create objects
 	for( unsigned int i = 0; i < power; i++ )
 	{
-		FireAnimation_item temp;
-		objects.push_back( temp );
+		FireAnimation_item *obj = new FireAnimation_item();
+		objects.push_back( obj );
 	}
 }
 
 cFireAnimation :: ~cFireAnimation( void )
 {
+	// clear
+	for( FireAnimList::iterator itr = objects.begin(), itr_end = objects.end(); itr != itr_end; ++itr )
+	{
+		delete *itr;
+	}
+
 	objects.clear();
 }
 
 void cFireAnimation :: Init( void )
 {
-	for( unsigned int i = 0; i < objects.size(); i++ )
+	for( FireAnimList::iterator itr = objects.begin(), itr_end = objects.end(); itr != itr_end; ++itr )
 	{
-		// position
-		objects[i].velx = ( (float)( rand() % ( 50 ) - 25 ) ) / 10;
-		objects[i].vely = ( (float)( rand() % ( 50 ) - 25 ) ) / 10;
+		FireAnimation_item *obj = (*itr);
+
+		// images
+		obj->images.push_back( pVideo->Get_Surface( "animation/fire_1/4.png" ) );
+		obj->images.push_back( pVideo->Get_Surface( "animation/fire_1/3.png" ) );
+		obj->images.push_back( pVideo->Get_Surface( "animation/fire_1/2.png" ) );
+		obj->images.push_back( pVideo->Get_Surface( "animation/fire_1/1.png" ) );
+		obj->Set_Image( 0 );
+
+		// velocity
+		obj->velx = static_cast<float>( rand() % ( 50 ) - 25 ) / 10;
+		obj->vely = static_cast<float>( rand() % ( 50 ) - 25 ) / 10;
 
 		// Z position
-		objects[i].posz = posz;
+		obj->posz = posz;
 		if( posz_rand > 0.001f )
 		{
-			objects[i].posz += (float)( rand() % (int)( posz_rand * 1000 ) ) / 1000;
+			obj->posz += static_cast<float>( rand() % static_cast<int>( posz_rand * 1000 ) ) / 1000;
 		}
 
 		// lifetime
-		objects[i].counter = (float)( 8 + rand() % ( 5 ) );
+		obj->counter = static_cast<float>( 8 + rand() % ( 5 ) );
 	}
 }
 
@@ -291,34 +313,36 @@ void cFireAnimation :: Update( void )
 	time_to_live += pFramerate->speedfactor * fading_speed;
 
 	// update objects
-	for( unsigned int i = 0; i < objects.size(); i++ )
+	for( FireAnimList::iterator itr = objects.begin(), itr_end = objects.end(); itr != itr_end; ++itr )
 	{
-		if( objects[i].counter > 8 )
+		FireAnimation_item *obj = (*itr);
+
+		if( obj->counter > 8 )
 		{
-			Set_Image( 0 );
+			obj->Set_Image( 0 );
 		}
-		else if( objects[i].counter > 5 )
+		else if( obj->counter > 5 )
 		{
-			Set_Image( 1 );
+			obj->Set_Image( 1 );
 		}
-		else if( objects[i].counter > 3 )
+		else if( obj->counter > 3 )
 		{
-			Set_Image( 2 );
+			obj->Set_Image( 2 );
 		}
-		else if( objects[i].counter > 0 )
+		else
 		{
-			Set_Image( 3 );
+			obj->Set_Image( 3 );
 		}
 
-		objects[i].counter -= pFramerate->speedfactor * fading_speed;
+		obj->counter -= pFramerate->speedfactor * fading_speed;
 
-		objects[i].Set_Scale( objects[i].counter / 10 );
-		objects[i].Move( objects[i].velx, objects[i].vely );
+		obj->Set_Scale( obj->counter / 10 );
+		obj->Move( obj->velx, obj->vely );
 	}
 
 	if( time_to_live > 12 || time_to_live < 0 )
 	{
-		visible = 0;
+		Set_Visible( 0 );
 	}
 }
 
@@ -330,24 +354,23 @@ void cFireAnimation :: Draw( cSurfaceRequest *request /* = NULL */ )
 	}
 
 	// draw objects
-	for( unsigned int i = 0; i < objects.size(); i++ )
+	for( FireAnimList::iterator itr = objects.begin(), itr_end = objects.end(); itr != itr_end; ++itr )
 	{
-		if( image )
-		{
-			// create request
-			cSurfaceRequest *request = new cSurfaceRequest();
-			image->Blit( objects[i].posx - ( pCamera->x - posx ), objects[i].posy - ( pCamera->y - posy ), objects[i].posz, request );
+		FireAnimation_item *obj = (*itr);
 
-			// scale
-			request->scale_x = objects[i].scalex;
-			request->scale_y = objects[i].scaley;
+		// create request
+		cSurfaceRequest *request = new cSurfaceRequest();
+		obj->image->Blit( obj->posx - ( pCamera->x - posx ), obj->posy - ( pCamera->y - posy ), obj->posz, request );
 
-			// color
-			request->color = color;
+		// scale
+		request->scale_x = obj->scalex;
+		request->scale_y = obj->scaley;
 
-			// add request
-			pRenderer->Add( request );
-		}
+		// color
+		request->color = color;
+
+		// add request
+		pRenderer->Add( request );
 	}
 }
 
@@ -372,19 +395,18 @@ cParticle :: ~cParticle( void )
 void cParticle :: Update( void )
 {
 	// update fade modifier
-	fade_pos -= ( pFramerate->speedfactor * ( (float)DESIRED_FPS * 0.001f ) ) / time_to_live;
+	fade_pos -= ( pFramerate->speedfactor * ( static_cast<float>(DESIRED_FPS) * 0.001f ) ) / time_to_live;
 
 	// finished fading
 	if( fade_pos <= 0 )
 	{
 		fade_pos = 0;
-		visible = 0;
+		Set_Visible( 0 );
 		return;
 	}
 
-	// position
-	posx += velx * pFramerate->speedfactor;
-	posy += vely * pFramerate->speedfactor;
+	// move
+	Move( velx, vely );
 	
 	// constant rotation
 	if( const_rotx > 0.0001f )
@@ -458,9 +480,12 @@ void cParticle :: Draw( cParticleAnimation *origin )
 
 /* *** *** *** *** *** *** *** cParticleAnimation *** *** *** *** *** *** *** *** *** *** */
 
-cParticleAnimation :: cParticleAnimation( float posx, float posy )
-: cAnimation( posx, posy )
+cParticleAnimation :: cParticleAnimation( float x /* = 0 */, float y /* = 0 */, float w /* = 0 */, float h /* = 0 */ )
+: cAnimation( x, y )
 {
+	rect.w = w;
+	rect.h = h;
+
 	animtype = PARTICLE_EXPLOSION;
 
 	// 0 = 1 emit
@@ -478,11 +503,14 @@ cParticleAnimation :: cParticleAnimation( float posx, float posy )
 	const_rotx_rand = 0;
 	const_roty_rand = 0;
 	const_rotz_rand = 0;
+	// angle
 	angle_start = 0;
 	angle_range = 360;
+	// scale
 	size_scale = 1;
 	size_scale_rand = 0;
-	color_rand = Color( (Uint8)0, 0, 0, 0 );
+	// color
+	color_rand = Color( static_cast<Uint8>(0), 0, 0, 0 );
 	// default 1 second
 	time_to_live = 1;
 	// default fading is alpha
@@ -516,8 +544,20 @@ void cParticleAnimation :: Emit( void )
 	{
 		cParticle *particle = new cParticle();
 
-		// Position
-		particle->Set_Pos( posx, posy, 1 );
+		// X Position
+		float x = posx;
+		if( rect.w > 0 )
+		{
+			x += ( rand() % static_cast<int>(rect.w) );
+		}
+		// Y Position
+		float y = posy;
+		if( rect.h > 0 )
+		{
+			y += ( rand() % static_cast<int>(rect.h) );
+		}
+		// Set Position
+		particle->Set_Pos( x, y, 1 );
 
 		// Image
 		particle->Set_Image( image, 1, 0 );
@@ -526,26 +566,22 @@ void cParticleAnimation :: Emit( void )
 		particle->posz = posz;
 		if( posz_rand > 0.001f )
 		{
-			particle->posz += (float)( rand() % (int)( posz_rand * 1000 ) ) / 1000;
+			particle->posz += static_cast<float>( rand() % static_cast<int>( posz_rand * 1000 ) ) / 1000;
 		}
 
 		// angle range
-		float dir_angle = 0;
+		float dir_angle = angle_start;
 		// start angle
-		if( angle_start > 0.001f )
-		{
-			dir_angle += angle_start;
-		}
 		if( angle_range > 0.001f )
 		{
-			dir_angle += (float)( rand() % (int)( angle_range * 100 ) ) / 100;
+			dir_angle += static_cast<float>( rand() % static_cast<int>( angle_range * 10 ) ) / 10;
 		}
 
 		// Velocity
 		float speed = vel;
 		if( vel_rand > 0.001f )
 		{
-			speed += (float)( rand() % (int)( vel_rand * 1000 ) ) / 1000;
+			speed += static_cast<float>( rand() % static_cast<int>( vel_rand * 1000 ) ) / 1000;
 		}
 		// set Velocity
 		particle->Set_Direction( dir_angle, speed, 1 );
@@ -561,22 +597,22 @@ void cParticleAnimation :: Emit( void )
 		particle->const_rotz = const_rotz;
 		if( const_rotx_rand > 0.001f )
 		{
-			particle->const_rotx += (float)( rand() % (int)( const_rotx_rand * 100 ) ) / 100;
+			particle->const_rotx += static_cast<float>( rand() % static_cast<int>( const_rotx_rand * 100 ) ) / 100;
 		}
 		if( const_roty_rand > 0.001f )
 		{
-			particle->const_roty += (float)( rand() % (int)( const_roty_rand * 100 ) ) / 100;
+			particle->const_roty += static_cast<float>( rand() % static_cast<int>( const_roty_rand * 100 ) ) / 100;
 		}
 		if( const_rotz_rand > 0.001f )
 		{
-			particle->const_rotz += (float)( rand() % (int)( const_rotz_rand * 100 ) ) / 100;
+			particle->const_rotz += static_cast<float>( rand() % static_cast<int>( const_rotz_rand * 100 ) ) / 100;
 		}
 
 		// Scale
 		particle->Set_Scale( size_scale );
 		if( size_scale_rand > 0.001f )
 		{
-			particle->Add_Scale( (float)( rand() % (int)( size_scale_rand * 100 ) ) / 100 );
+			particle->Add_Scale( static_cast<float>( rand() % static_cast<int>( size_scale_rand * 100 ) ) / 100 );
 		}
 
 		// Color
@@ -602,7 +638,7 @@ void cParticleAnimation :: Emit( void )
 		particle->time_to_live = time_to_live;
 		if( time_to_live_rand > 0.001f )
 		{
-			particle->time_to_live += (float)( rand() % (int)( time_to_live_rand * 1000 ) ) / 1000;
+			particle->time_to_live += static_cast<float>( rand() % static_cast<int>( time_to_live_rand * 1000 ) ) / 1000;
 		}
 
 		objects.push_back( particle );
@@ -631,7 +667,7 @@ void cParticleAnimation :: Update( void )
 		return;
 	}
 	
-	emitter_living_time += pFramerate->speedfactor * ( (float)DESIRED_FPS * 0.001f );
+	emitter_living_time += pFramerate->speedfactor * ( static_cast<float>(DESIRED_FPS) * 0.001f );
 
 	// update objects
 	for( ParticleList::iterator itr = objects.begin(); itr != objects.end(); )
@@ -655,8 +691,8 @@ void cParticleAnimation :: Update( void )
 		}
 	}
 
-	// if able to emit
-	if( emitter_living_time < emitter_time_to_live )
+	// if able to emit or endless emitter
+	if( emitter_living_time < emitter_time_to_live || emitter_time_to_live == -1 )
 	{
 		// emit
 		while( emit_counter > emitter_iteration_interval )
@@ -665,12 +701,12 @@ void cParticleAnimation :: Update( void )
 			emit_counter -= emitter_iteration_interval;
 		}
 
-		emit_counter += pFramerate->speedfactor * ( (float)DESIRED_FPS * 0.001f );
+		emit_counter += pFramerate->speedfactor * ( static_cast<float>(DESIRED_FPS) * 0.001f );
 	}
 	// no particles are active
 	else if( !objects.size() )
 	{
-		visible = 0;
+		Set_Visible( 0 );
 	}
 }
 
@@ -687,6 +723,29 @@ void cParticleAnimation :: Draw( cSurfaceRequest *request /* = NULL */ )
 	}
 }
 
+void cParticleAnimation :: Set_Emitter_Rect( float x, float y, float w /* = 0 */, float h /* = 0 */ )
+{
+	Set_Pos( x, y, 1 );
+	rect.w = w;
+	rect.h = h;
+
+	// invalid width
+	if( rect.w < 0 )
+	{
+		rect.w = 0;
+	}
+	// invalid height
+	if( rect.h < 0 )
+	{
+		rect.h = 0;
+	}
+}
+
+void cParticleAnimation :: Set_Emitter_Rect( GL_rect r )
+{
+	Set_Emitter_Rect( r.x, r.y, r.w, r.h );
+}
+
 void cParticleAnimation :: Set_Emitter_Time_to_Live( float time )
 {
 	emitter_time_to_live = time;
@@ -702,7 +761,7 @@ void cParticleAnimation :: Set_Quota( unsigned int size )
 	emitter_quota = size;
 }
 
-void cParticleAnimation :: Set_Speed( float vel_base, float vel_random /* = 1 */ )
+void cParticleAnimation :: Set_Speed( float vel_base, float vel_random /* = 2 */ )
 {
 	vel = vel_base;
 	vel_rand = vel_random;
@@ -726,7 +785,7 @@ void cParticleAnimation :: Set_ConstRotationZ( float rot, float rot_random /* =
 	const_rotz_rand = rot_random;
 }
 
-void cParticleAnimation :: Set_DirectionRange( float start, float range /* = 20 */ )
+void cParticleAnimation :: Set_DirectionRange( float start, float range /* = 0 */ )
 {
 	angle_start = start;
 	angle_range = range;
@@ -738,7 +797,7 @@ void cParticleAnimation :: Set_Scale( float nscale, float scale_random /* = 0 */
 	size_scale_rand = scale_random;
 }
 
-void cParticleAnimation :: Set_Color( Color col, Color col_rand )
+void cParticleAnimation :: Set_Color( Color col, Color col_rand /* = Color( static_cast<Uint8>(0) ) */ )
 {
 	color = col;
 	color_rand = col_rand;
